package com.xxh.cloud.etl.common.framework.utils;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.XXEnumUtil;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;

import java.util.*;

public class HospitalSyncDateUtils {


    public static String getStart(String startDateStr ,int minute){
        DateTime startDate = DateUtil.parse(startDateStr, DatePattern.NORM_DATETIME_FORMAT);
        DateTime endDate = DateUtil.offsetMinute(startDate,0-minute);
        return DateUtil.format(endDate,DatePattern.NORM_DATETIME_FORMAT);
    }

    public static String getEnd(String startDateStr ,int maxMinute){
        DateTime startDate = DateUtil.parse(startDateStr, DatePattern.NORM_DATETIME_FORMAT);
        Date nowDate = new Date();
        return getEnd(startDate,nowDate,maxMinute);
    }


    public static String getEnd(DateTime startDate ,Date nowDate ,int maxMinute){
        int i =  nowDate.compareTo(startDate);
        if(i<=0){
            return null;
        }

        long minuteNum= DateUtil.between(startDate,nowDate, DateUnit.MINUTE);

        if(minuteNum>maxMinute){
            DateTime endDate = DateUtil.offsetMinute(startDate,maxMinute);
            return DateUtil.format(endDate,DatePattern.NORM_DATETIME_FORMAT);
        }else {
            return DateUtil.format(nowDate,DatePattern.NORM_DATETIME_FORMAT);
        }
    }

    public static Object getMapVal(String key,Map<String, Object> map){
        if(key.startsWith("__")){
            return null;
        }
        Map<String,SyncDataKeyEnum> dataKeyMap= XXEnumUtil.getEnumMap(SyncDataKeyEnum.class);

        if(dataKeyMap.containsKey(key)){
            SyncDataKeyEnum dataKey=dataKeyMap.get(key);
            if(SyncDataKeyEnum.SDKCV_DateNow==dataKey){
                return new Date();
            }else if(SyncDataKeyEnum.SDKCV_UserNow==dataKey){
                return "-1";
            }else if(SyncDataKeyEnum.SDKCV_UUID==dataKey){
                return IdUtil.fastSimpleUUID();
            }else if(SyncDataKeyEnum.SDKCV_SyncFlag==dataKey){
                return YN.NO.getCode();
            } else {
                return null;
            }
        }else if(key.startsWith(SyncDataKeyEnum.SDKCV_DEF.getCode())){
            String val= StrUtil.subAfter(key,SyncDataKeyEnum.SDKCV_DEF.getCode(),true);
            return val;
        }

        return  map.get(key);
    }




    public static HwdsWriteConfigBean getHwdsWriteConfigBean(String dbType, String mqType , String mqTopic, List<HwdsWriteDBConfigBean> dbConfigs){
        HwdsWriteConfigBean config=new HwdsWriteConfigBean();

        if(StrUtil.isEmpty(dbType)){
            config.setDbType("none");
        }else {
            config.setDbType(dbType);
        }

        config.setDbConfigs(dbConfigs);


        if(StrUtil.isEmpty(mqType)){
            config.setMqType("none");
        }else {
            config.setMqType(mqType);
        }

        if(StrUtil.isEmpty(mqTopic)){
            config.setMqTopic("none");
        }else {
            config.setMqTopic(mqTopic);
        }
        return config;
    }


    public static HwdsWriteConfigBean getHwdsWriteConfigBean(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic, String table, Map<String,String> mapping){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        dbConfigs.add(new HwdsWriteDBConfigBean("DEF_ADD",dbUrl,dbUserName,dbUserPassword,table,mapping));
        HwdsWriteConfigBean config= getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }


    public static String getDefaultValue(String str){
        return StrUtil.format("{}{}", SyncDataKeyEnum.SDKCV_DEF.getCode(),str);
    }


    public static void main(String[] args) {

        //yyyy-MM-dd HH:mm:ss
//        DateTime startDate = DateUtil.parse("2024-07-12 16:04:00", DatePattern.NORM_DATETIME_FORMAT);


        String str= HospitalSyncDateUtils.getStart("2024-07-12 16:04:00",10);
        System.out.println(str);

    }




}
