package com.xxh.cloud.etl.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.etl.framework.modules.base.entity.EtlDataCleanConfigInfo;
import com.xxh.cloud.etl.framework.modules.base.service.EtlDataCleanConfigInfoService;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.ConfigDataCleanListDTO;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.ConfigDataCleanQueryDTO;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.ConfigDataCleanSaveDTO;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.ConfigDataCleanUpdateDTO;
import com.xxh.cloud.etl.framework.modules.biz.bean.vos.DataCleanConfigVO;
import com.xxh.cloud.etl.framework.modules.biz.convert.DataCleanConfigConvert;
import com.xxh.cloud.etl.framework.modules.biz.service.DataCleanConfigBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class DataCleanConfigBizServiceImpl implements DataCleanConfigBizService {

    @Autowired
    private EtlDataCleanConfigInfoService etlDataCleanConfigInfoService;


    @Override
    public PageData<DataCleanConfigVO> query(ConfigDataCleanQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<EtlDataCleanConfigInfo> page = etlDataCleanConfigInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<EtlDataCleanConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getCode()),"code",params.getCode())
                        .eq(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .orderByDesc("code")
        );
        if (page == null || CollUtil.isEmpty(page.getRecords())) {
            return null;
        }
        PageData<DataCleanConfigVO> data= new PageConverter<EtlDataCleanConfigInfo, DataCleanConfigVO>() {
            @Override
            public DataCleanConfigVO converter(EtlDataCleanConfigInfo entity) {
                return DataCleanConfigConvert.getEtlDataCleanConfigVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<DataCleanConfigVO> list(ConfigDataCleanListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<EtlDataCleanConfigInfo> list= etlDataCleanConfigInfoService.list(
                new QueryWrapper<EtlDataCleanConfigInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getCode()),"code",params.getCode())
                        .eq(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
        );
        List<DataCleanConfigVO> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(DataCleanConfigConvert.getEtlDataCleanConfigVO(item));
            });
        }
        return data;
    }

    @Override
    public DataCleanConfigVO save(ConfigDataCleanSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        EtlDataCleanConfigInfo entity=new EtlDataCleanConfigInfo();
        entity.setCode(params.getCode());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        //todo:日志
        etlDataCleanConfigInfoService.save(entity);
        return DataCleanConfigConvert.getEtlDataCleanConfigVO(entity);
    }

    @Override
    public DataCleanConfigVO update(ConfigDataCleanUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        EtlDataCleanConfigInfo entity=etlDataCleanConfigInfoService.getById(params.getCode());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        entity.setType(params.getType());

        etlDataCleanConfigInfoService.updateById(entity);

        //todo:缓存
        //todo:日志

        return DataCleanConfigConvert.getEtlDataCleanConfigVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        EtlDataCleanConfigInfo entity=etlDataCleanConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        etlDataCleanConfigInfoService.updateById(entity);
        //todo:缓存
        //todo:日志
    }

    @Override
    public DataCleanConfigVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        EtlDataCleanConfigInfo entity=etlDataCleanConfigInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DataCleanConfigConvert.getEtlDataCleanConfigVO(entity);
    }


}
