package com.xxh.cloud.etl.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.etl.framework.modules.base.entity.EtlDataCleanConfigInfo;
import com.xxh.cloud.etl.framework.modules.base.entity.EtlDataCleanConfigItemInfo;
import com.xxh.cloud.etl.framework.modules.base.service.EtlDataCleanConfigItemInfoService;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.etl.framework.modules.biz.bean.vos.DataCleanConfigDetailVO;
import com.xxh.cloud.etl.framework.modules.biz.convert.DataCleanConfigDetailConvert;
import com.xxh.cloud.etl.framework.modules.biz.service.DataCleanConfigDetailBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class DataCleanConfigDetailBizServiceImpl implements DataCleanConfigDetailBizService {


    @Autowired
    private EtlDataCleanConfigItemInfoService etlDataCleanConfigItemInfoService;


    @Override
    public List<DataCleanConfigDetailVO> list(DataCleanConfigDetailListDTO params) {
        List<EtlDataCleanConfigItemInfo> list= etlDataCleanConfigItemInfoService.list(
                new QueryWrapper<EtlDataCleanConfigItemInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .eq("data_clean_code",params.getDataCleanCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .like(StrUtil.isNotEmpty(params.getSourceName()),"source_name",params.getSourceName())
                        .like(StrUtil.isNotEmpty(params.getStandardName()),"standard_name",params.getStandardName())
                        .eq(StrUtil.isNotEmpty(params.getStandardCode()),"standard_code",params.getStandardCode())
                        .orderByDesc("update_date")
        );
        List<DataCleanConfigDetailVO> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(DataCleanConfigDetailConvert.getDataCleanConfigDetailVO(item));
            });
        }
        return data;
    }

    @Override
    public PageData<DataCleanConfigDetailVO> query(DataCleanConfigDetailPageDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<EtlDataCleanConfigItemInfo> page = etlDataCleanConfigItemInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<EtlDataCleanConfigItemInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .eq("data_clean_code",params.getDataCleanCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getProjectCode()),"project_code",params.getProjectCode())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .like(StrUtil.isNotEmpty(params.getSourceName()),"source_name",params.getSourceName())
                        .like(StrUtil.isNotEmpty(params.getStandardName()),"standard_name",params.getStandardName())
                        .eq(StrUtil.isNotEmpty(params.getStandardCode()),"standard_code",params.getStandardCode())
                        .orderByDesc("update_date")
        );


        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }


        PageData<DataCleanConfigDetailVO> data= new PageConverter<EtlDataCleanConfigItemInfo, DataCleanConfigDetailVO>() {
            @Override
            public DataCleanConfigDetailVO converter(EtlDataCleanConfigItemInfo entity) {
                return DataCleanConfigDetailConvert.getDataCleanConfigDetailVO(entity);
            }
        }.getPageData(page);

        return data;
    }


    @Override
    public DataCleanConfigDetailVO save(DataCleanConfigDetailSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        EtlDataCleanConfigItemInfo entity= DataCleanConfigDetailConvert.getEtlDataCleanConfigItemInfo_Add(params);
        etlDataCleanConfigItemInfoService.save(entity);
        return DataCleanConfigDetailConvert.getDataCleanConfigDetailVO(entity);
    }

    @Override
    public DataCleanConfigDetailVO update(DataCleanConfigDetailUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        EtlDataCleanConfigItemInfo entity=etlDataCleanConfigItemInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        entity.setSourceName(params.getSourceName());
        entity.setSourceCode(params.getSourceCode());
        entity.setStandardName(params.getStandardName());
        entity.setStandardCode(params.getStandardCode());
        entity.setRemark(params.getRemark());
        if(YN.NO.getCode().equals(entity.getStatus())){
            entity.setStatus(YN.YES.getCode());
            entity.setHandleDate(new Date());
        }
        etlDataCleanConfigItemInfoService.updateById(entity);
        return DataCleanConfigDetailConvert.getDataCleanConfigDetailVO(entity);
    }

    @Override
    public void process(DataCleanConfigDetailProcessDTO params) {
        //todo:del
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        EtlDataCleanConfigItemInfo entity=etlDataCleanConfigItemInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        etlDataCleanConfigItemInfoService.updateById(entity);
    }

    @Override
    public DataCleanConfigDetailVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        EtlDataCleanConfigItemInfo entity=etlDataCleanConfigItemInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return DataCleanConfigDetailConvert.getDataCleanConfigDetailVO(entity);
    }


    private String getKey(String ruleCode,String sourceCode,String sourceName){
        if(StrUtil.isEmpty(ruleCode)){
            ruleCode="RULECODE";
        }
        if(StrUtil.isEmpty(sourceCode)){
            sourceCode="SourceCode";
        }
        if(StrUtil.isEmpty(sourceName)){
            sourceCode="SourceName";
        }
        return StrUtil.format("{}-{}-{}",ruleCode,sourceCode,sourceName);
    }

    @Override
    public void saveBatch(List<DataCleanConfigDetailSimpleSaveDTO> params) {
        //ValidatorUtils.validateEntity(params);

        Set<String> ruleCodes=new HashSet<>();
        Map<String,DataCleanConfigDetailSimpleSaveDTO> paramMap=new HashMap<>();
        params.forEach(item->{
            ruleCodes.add(item.getRuleCode());
            String key = getKey(item.getRuleCode(),item.getSourceCode(),item.getSourceName());

            DataCleanConfigDetailSimpleSaveDTO paraItem= paramMap.get(key);
            if(paraItem==null){
                paramMap.put(key,item);
            }

        });


        Map<String,String> dbDataMap=new HashMap<>();
        List<EtlDataCleanConfigItemInfo> dbList= etlDataCleanConfigItemInfoService.list(
                new QueryWrapper<EtlDataCleanConfigItemInfo>()
                        .in("data_clean_code",ruleCodes)
                        .eq("rows_status",YN.YES.getCode())
        );
        if(CollUtil.isNotEmpty(dbList)){
            dbList.forEach(item->{
              String key =  getKey(item.getDataCleanCode(),item.getSourceCode(),item.getSourceName());
                dbDataMap.put(key,key);
            });
        }

        List<EtlDataCleanConfigItemInfo> list=new ArrayList<>();

        paramMap.forEach((k,item)->{
            String key = getKey(item.getRuleCode(),item.getSourceCode(),item.getSourceName());
            String objVal= dbDataMap.get(key);
            if(StrUtil.isEmpty(objVal)){
                EtlDataCleanConfigItemInfo entity=new EtlDataCleanConfigItemInfo();
                entity.setId(IdUtil.getSnowflake(1,2).nextIdStr());
                entity.setDataCleanCode(item.getRuleCode());
                entity.setProjectCode("WBMD");
                entity.setSourceName(item.getSourceName());
                entity.setSourceCode(item.getSourceCode());
                entity.setStatus(YN.NO.getCode());
                entity.setRowsStatus(YN.YES.getCode());
                list.add(entity);
            }
        });
        System.out.println("====================");
        etlDataCleanConfigItemInfoService.saveBatch(list);
    }




}
