package com.xxh.cloud.etl.framework.modules.cache.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.etl.client.api.bean.bos.DataCleanConfigBO;
import com.xxh.cloud.etl.client.api.bean.bos.DataCleanConfigDetailBO;
import com.xxh.cloud.etl.client.api.bean.constant.DataCleanStandardEnum;
import com.xxh.cloud.etl.framework.modules.base.entity.EtlDataCleanConfigInfo;
import com.xxh.cloud.etl.framework.modules.base.entity.EtlDataCleanConfigItemInfo;
import com.xxh.cloud.etl.framework.modules.base.service.EtlDataCleanConfigInfoService;
import com.xxh.cloud.etl.framework.modules.base.service.EtlDataCleanConfigItemInfoService;
import com.xxh.cloud.etl.framework.modules.cache.convert.DataCleanConfigCacheConvert;
import com.xxh.cloud.etl.framework.modules.cache.keys.DataCleanConfigCacheKeys;
import com.xxh.cloud.etl.framework.modules.cache.service.DataCleanConfigCacheService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.metadata.client.api.StandardKeywordsClientService;
import com.xxh.cloud.metadata.client.api.StandardLaboratoryProjectClientService;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardKeywordsBO;
import com.xxh.cloud.metadata.client.api.bean.bos.standard.StandardLaboratoryProjectBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class DataCleanConfigCacheServiceImpl implements DataCleanConfigCacheService {

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private EtlDataCleanConfigInfoService etlDataCleanConfigInfoService;
    @Autowired
    private EtlDataCleanConfigItemInfoService etlDataCleanConfigItemInfoService;

    @Autowired
    private StandardKeywordsClientService standardKeywordsClientService;

    @Autowired
    private StandardLaboratoryProjectClientService standardLaboratoryProjectClientService;



    @Override
    public DataCleanConfigBO getDataCleanConfigByCache(String code) {
        String key= DataCleanConfigCacheKeys.getDataCleanConfigKey(code);
        Object cacheObj= redisUtils.get(key);
        if(cacheObj!=null){
            return JSON.parseObject(cacheObj.toString(),DataCleanConfigBO.class);
        }
        DataCleanConfigBO data=this.getDataCleanConfigByDB(code);
        if(data==null){
            return null;
        }
        redisUtils.set(key,data,RedisUtils.HOUR_ONE_EXPIRE);
        return data;
    }

    @Override
    public List<DataCleanConfigBO> getDataCleanConfigListByCache(Collection<String> codes) {
        List<DataCleanConfigBO> data=this.getDataCleanConfigListByDB(codes);
        return data;
    }

    @Override
    public Map<String, DataCleanConfigBO> getDataCleanConfigMapByCache(Collection<String> codes) {
        if(CollUtil.isEmpty(codes)){
            return null;
        }
        Map<String, DataCleanConfigBO> data=this.getDataCleanConfigMapByDB(codes);
        return data;
    }


    @Override
    public void removeDataCleanConfig(String code) {
        String key= DataCleanConfigCacheKeys.getDataCleanConfigKey(code);
        redisUtils.delete(key);
    }

    @Override
    public void removeDataCleanConfig(Collection<String> codes) {
        Set<String> keys=new HashSet<>();
        codes.forEach(code->{
            keys.add(DataCleanConfigCacheKeys.getDataCleanConfigKey(code));
        });
        redisUtils.delete(keys);
    }


    @Override
    public DataCleanConfigBO getDataCleanConfigByDB(String code) {
        List<EtlDataCleanConfigItemInfo> list= etlDataCleanConfigItemInfoService.list(
                new QueryWrapper<EtlDataCleanConfigItemInfo>()
                        .eq("data_clean_code",code)
                        .eq("rows_status", YN.YES.getCode())
        );
        if(CollUtil.isEmpty(list)){
            return null;
        }
        DataCleanConfigBO data=new DataCleanConfigBO();
        list.forEach(item->{
            DataCleanConfigDetailBO dataItem= DataCleanConfigCacheConvert.getDataCleanConfigDetail(item);
            data.addDetail(dataItem);
        });
        return data;
    }

    @Override
    public List<DataCleanConfigBO> getDataCleanConfigListByDB(Collection<String> codes) {
        List<EtlDataCleanConfigInfo> configList= etlDataCleanConfigInfoService.list(
                new QueryWrapper<EtlDataCleanConfigInfo>()
                        .in("code",codes)
                        .eq("rows_status",YN.YES.getCode())

        );
        List<DataCleanConfigBO> data=new ArrayList<>();

        if(CollUtil.isEmpty(configList)){
            return data;
        }


        Set<String> cleanConfigCodes=new HashSet<>();
        configList.forEach(configItem->{
            data.add(new DataCleanConfigBO(configItem.getCode(),configItem.getName(),configItem.getType()));
            cleanConfigCodes.add(configItem.getCode());
        });


        List<EtlDataCleanConfigItemInfo> configItemList= etlDataCleanConfigItemInfoService.list(
                new QueryWrapper<EtlDataCleanConfigItemInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .eq("status",YN.YES.getCode())
                        .in("data_clean_code",cleanConfigCodes)
                        .orderByDesc("data_clean_code")
        );

        if(CollUtil.isEmpty(configItemList)){
            return data;
        }


        Set<String> keywordCodes=new HashSet<>();
        Set<String> labProjectCodes=new HashSet<>();
        for(DataCleanConfigBO dataCleanConfig:data){
            if(DataCleanStandardEnum.DCS_Keywords.getCode().equals(dataCleanConfig.getType())){
                configItemList.forEach(item->{
                    if(dataCleanConfig.getCode().equals(item.getDataCleanCode())){
                        keywordCodes.add(item.getStandardCode());
                    }
                });
            } else if(DataCleanStandardEnum.DCS_LabProject.getCode().equals(dataCleanConfig.getType())){
                configItemList.forEach(item->{
                    if(dataCleanConfig.getCode().equals(item.getDataCleanCode())){
                        labProjectCodes.add(item.getStandardCode());
                    }
                });
            }
        }

        Map<String, StandardKeywordsBO> keywordsMap= standardKeywordsClientService.getMap(keywordCodes);
        Map<String, StandardLaboratoryProjectBO> laboratoryProjectMap = standardLaboratoryProjectClientService.getMap(labProjectCodes);

        for(DataCleanConfigBO dataCleanConfig:data){
            configItemList.forEach(item->{
                if(dataCleanConfig.getCode().equals(item.getDataCleanCode())){
                    if(DataCleanStandardEnum.DCS_Keywords.getCode().equals(dataCleanConfig.getType())){
                        DataCleanConfigDetailBO detailItem= DataCleanConfigCacheConvert.getDataCleanConfigDetailByKeywords(item,keywordsMap);
                        dataCleanConfig.addDetail(detailItem);
                    } else if(DataCleanStandardEnum.DCS_LabProject.getCode().equals(dataCleanConfig.getType())){
                        DataCleanConfigDetailBO detailItem= DataCleanConfigCacheConvert.getDataCleanConfigDetailByLab(item,laboratoryProjectMap);
                        dataCleanConfig.addDetail(detailItem);
                    }else {
                        DataCleanConfigDetailBO detailItem= DataCleanConfigCacheConvert.getDataCleanConfigDetail(item);
                        dataCleanConfig.addDetail(detailItem);
                    }
                }
            });
        }

        return data;
    }




    @Override
    public Map<String, DataCleanConfigBO> getDataCleanConfigMapByDB(Collection<String> codes) {
        List<DataCleanConfigBO> list=getDataCleanConfigListByDB(codes);
        Map<String, DataCleanConfigBO> data=new HashMap<>();
        list.forEach(item->{
            data.put(item.getCode(),item);
        });
        return data;
    }


}
