package com.xxh.cloud.etl.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.etl.framework.modules.biz.bean.vos.DataCleanConfigDetailVO;
import com.xxh.cloud.etl.framework.modules.biz.service.DataCleanConfigDetailBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/etl/cleanConfDetail")
@Api(tags = "运营端,数据清洗规则配置明细管理")
public class DataCleanConfigDetailManagerController {


    @Autowired
    private DataCleanConfigDetailBizService dataCleanConfigDetailBizService;

    @SaCheckPermission("sys:etl:cleanConfDetail:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = DataCleanConfigDetailVO.class)
    public Result<List<DataCleanConfigDetailVO>> query() {
        DataCleanConfigDetailListDTO params=new DataCleanConfigDetailListDTO();
        params.setDataCleanCode("lab_result_two_category");
        List<DataCleanConfigDetailVO> data=  dataCleanConfigDetailBizService.list(params);
        return new Result<List<DataCleanConfigDetailVO>>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConfDetail:queryPage")
    @PostMapping("/queryPage")
    @ApiOperation(value = "查询", response = DataCleanConfigDetailVO.class)
    public Result<PageData<DataCleanConfigDetailVO>> queryPage(@RequestBody DataCleanConfigDetailPageDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<DataCleanConfigDetailVO> data=  dataCleanConfigDetailBizService.query(params);
        return new Result<PageData<DataCleanConfigDetailVO>>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConfDetail:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DataCleanConfigDetailVO.class)
    public Result<List<DataCleanConfigDetailVO>> list(@RequestBody DataCleanConfigDetailListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DataCleanConfigDetailVO> data=  dataCleanConfigDetailBizService.list(params);
        return new Result<List<DataCleanConfigDetailVO>>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConfDetail:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = DataCleanConfigDetailVO.class)
    public  Result<DataCleanConfigDetailVO> save(@RequestBody DataCleanConfigDetailSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        DataCleanConfigDetailVO data=  dataCleanConfigDetailBizService.save(params);
        return new Result<DataCleanConfigDetailVO>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConfDetail:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = DataCleanConfigDetailVO.class)
    public  Result<DataCleanConfigDetailVO> update(@RequestBody DataCleanConfigDetailUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataCleanConfigDetailVO data=  dataCleanConfigDetailBizService.update(params);
        return new Result<DataCleanConfigDetailVO>().ok(data);
    }



    @SaCheckPermission("sys:etl:cleanConfDetail:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = DataCleanConfigDetailVO.class)
    public  Result<DataCleanConfigDetailVO> info(@RequestBody DataCleanConfigDetailGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DataCleanConfigDetailVO data=  dataCleanConfigDetailBizService.getById(params.getId());
        return new Result<DataCleanConfigDetailVO>().ok(data);
    }



    @SaCheckPermission("sys:etl:cleanConfDetail:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody DataCleanConfigDetailGetDTO params) {
        ValidatorUtils.validateEntity(params);
        dataCleanConfigDetailBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }






}
