package com.xxh.cloud.etl.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.etl.framework.modules.biz.bean.vos.DataCleanConfigVO;
import com.xxh.cloud.etl.framework.modules.biz.service.DataCleanConfigBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Log4j2
@RestController
@RequestMapping("/sys/etl/cleanConf")
@Api(tags = "运营端,数据清洗规则配置管理")
public class DataCleanConfigManagerController {


    @Autowired
    private DataCleanConfigBizService dataCleanConfigBizService;


    @SaCheckPermission("sys:etl:cleanConf:query")
    @PostMapping("/query")
    @ApiOperation(value = "查询", response = Map.class)
    public  Result<PageData<DataCleanConfigVO>> query(@RequestBody ConfigDataCleanQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<DataCleanConfigVO>  data= dataCleanConfigBizService.query(params);
        return new Result<PageData<DataCleanConfigVO>>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConf:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = DataCleanConfigVO.class)
    public  Result<List<DataCleanConfigVO>> list(@RequestBody ConfigDataCleanListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DataCleanConfigVO> data=  dataCleanConfigBizService.list(params);
        return new Result<List<DataCleanConfigVO>>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConf:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = DataCleanConfigVO.class)
    public  Result<DataCleanConfigVO> save(@RequestBody ConfigDataCleanSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        DataCleanConfigVO data=  dataCleanConfigBizService.save(params);
        return new Result<DataCleanConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConf:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = DataCleanConfigVO.class)
    public  Result<DataCleanConfigVO> update(@RequestBody ConfigDataCleanUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataCleanConfigVO data=  dataCleanConfigBizService.update(params);
        return new Result<DataCleanConfigVO>().ok(data);
    }



    @SaCheckPermission("sys:etl:cleanConf:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = DataCleanConfigVO.class)
    public  Result<DataCleanConfigVO> info(@RequestBody ConfigDataCleanGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DataCleanConfigVO data=  dataCleanConfigBizService.getById(params.getCode());
        return new Result<DataCleanConfigVO>().ok(data);
    }



    @SaCheckPermission("sys:etl:cleanConf:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody ConfigDataCleanGetDTO params) {
        ValidatorUtils.validateEntity(params);
        dataCleanConfigBizService.delById(params.getCode());
        return new Result<String>().ok("ok");
    }





















}
