package com.xxh.cloud.etl.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.etl.framework.modules.biz.bean.vos.TableCleanConfigVO;
import com.xxh.cloud.etl.framework.modules.biz.service.TableCleanConfigBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/etl/cleanConf2Meta")
@Api(tags = "运营端,清洗规则与表关系管理(弃)")
public class DataCleanConfigRelMetaManagerController {

    @Autowired
    private TableCleanConfigBizService tableCleanConfigBizService;


    @SaCheckPermission("sys:etl:cleanConf2Meta:list")
    @PostMapping("/list")
    @ApiOperation(value = "列表", response = TableCleanConfigVO.class)
    public Result<List<TableCleanConfigVO>> list(@RequestBody TableCleanConfigListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<TableCleanConfigVO> data = tableCleanConfigBizService.list(params);
        return new Result<List<TableCleanConfigVO>>().ok(data);
    }

    @SaCheckPermission("sys:etl:cleanConf2Meta:save")
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = TableCleanConfigVO.class)
    public Result<TableCleanConfigVO> save(@RequestBody TableCleanConfigSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        TableCleanConfigVO data= tableCleanConfigBizService.save(params);
        return new Result<TableCleanConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:etl:cleanConf2Meta:saveBatch")
    @PostMapping("/saveBatch")
    @ApiOperation(value = "批量新增", response = String.class)
    public Result<String> save(@RequestBody TableCleanConfigSaveBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        tableCleanConfigBizService.saveBatch(params);
        return new Result<String>().ok("ok");
    }


    @SaCheckPermission("sys:etl:cleanConf2Meta:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改", response = TableCleanConfigVO.class)
    public Result<TableCleanConfigVO> update(@RequestBody TableCleanConfigUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        TableCleanConfigVO data=  tableCleanConfigBizService.update(params);
        return new Result<TableCleanConfigVO>().ok(data);
    }


    @SaCheckPermission("sys:etl:cleanConf2Meta:info")
    @PostMapping("/info")
    @ApiOperation(value = "获取", response = TableCleanConfigVO.class)
    public  Result<TableCleanConfigVO> info(@RequestBody TableMetaRelDataCleanGetDTO params) {
        ValidatorUtils.validateEntity(params);
        TableCleanConfigVO data =  tableCleanConfigBizService.getById(params.getId());
        return new Result<TableCleanConfigVO>().ok(data);
    }

    @SaCheckPermission("sys:etl:cleanConf2Meta:del")
    @PostMapping("/del")
    @ApiOperation(value = "删除", response = String.class)
    public Result<String> del(@RequestBody TableMetaRelDataCleanGetDTO params) {
        ValidatorUtils.validateEntity(params);
        tableCleanConfigBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }




}
