package com.xxh.cloud.data.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.xxh.cloud.data.framework.modules.biz.bean.dto.DwTableDataGetDTO;
import com.xxh.cloud.data.framework.modules.biz.bean.dto.DwTableDataQueryDTO;
import com.xxh.cloud.data.framework.modules.biz.bean.vos.DwDataInfoVO;
import com.xxh.cloud.data.framework.modules.biz.bean.vos.DwDataQueryVO;
import com.xxh.cloud.data.framework.modules.biz.service.DwDataBaseQueryService;
import com.xxh.cloud.data.framework.modules.cache.service.DwTableEntityCacheService;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.metadata.framework.modules.script.bean.dto.EntityOperateGetByIdDTO;
import com.xxh.cloud.metadata.framework.modules.script.bean.dto.EntityOperateQueryDTO;
import com.xxh.cloud.metadata.framework.modules.script.bean.vos.TableEntityPropertyVO;
import com.xxh.cloud.metadata.framework.modules.script.bean.vos.TableEntityVO;
import com.xxh.cloud.metadata.framework.modules.script.service.EntityOperateApiService;
import com.xxh.cloud.metadata.framework.modules.script.sql.operate.bean.OrderByBean;
import com.xxh.cloud.metadata.framework.modules.script.sql.operate.bean.SelectBean;
import com.xxh.cloud.metadata.framework.modules.script.sql.operate.bean.WhereAndOrBean;
import com.xxh.cloud.metadata.framework.modules.script.sql.operate.bean.WhereItemBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
@DS("wd")
@Service
public class DwDataBaseQueryServiceImpl implements DwDataBaseQueryService {


//    @Autowired
//    private DbTableOperateApiService dbTableOperateApiService;

    @Autowired
    public EntityOperateApiService entityOperateApiService;

    @Autowired
    private DwTableEntityCacheService dwTableEntityCacheService;


    @Override
    public DwDataQueryVO query(DwTableDataQueryDTO params) {
        TableEntityVO tableEntity =dwTableEntityCacheService.getTableColumn(params.getTableId());
        if(tableEntity==null){
            return null;
        }
        Map<String, TableEntityPropertyVO> tableColumnMap=new HashMap<>();
        tableEntity.getProps().forEach(item->{
            tableColumnMap.put(item.getCode(),item);
        });
        DwDataQueryVO data=new DwDataQueryVO();
        IPage<Map<String, Object>> queryData = this._query(params,tableColumnMap);
        data.setData(queryData);
        data.setProps(tableEntity.getProps());
        return data;
    }

    @Override
    public DwDataInfoVO get(DwTableDataGetDTO params) {
        ValidatorUtils.validateEntity(params);

        TableEntityVO tableEntity =dwTableEntityCacheService.getTableColumn(params.getTableId());
        if(tableEntity==null){
            return null;
        }

        EntityOperateGetByIdDTO getDTO=new EntityOperateGetByIdDTO();
        getDTO.setTableCode(params.getTableId());
        getDTO.setId(params.getId());
        List<SelectBean> select=new ArrayList<>();
        tableEntity.getProps().forEach(item->{
            select.add(new SelectBean(params.getTableId(),item.getCode(),item.getEntityDataType()));
        });
        getDTO.setSelect(select);
        Map<String, Object> tableData= entityOperateApiService.getById(getDTO);

        DwDataInfoVO data=new DwDataInfoVO();
        data.setData(tableData);
        data.setProps(tableEntity.getProps());
        return data;
    }


    private IPage<Map<String, Object>> _query(DwTableDataQueryDTO params, Map<String, TableEntityPropertyVO>  tableColumnMap){
        EntityOperateQueryDTO queryDTO=new EntityOperateQueryDTO();
        queryDTO.setTableCode(params.getTableId());
        queryDTO.setPage(params.getPage());
        queryDTO.setRows(params.getRows());

        //select
        List<SelectBean> select=new ArrayList<>();
        tableColumnMap.forEach((key,item)->{
            select.add(new SelectBean(params.getTableId(),item.getCode(),item.getEntityDataType()));
        });

        queryDTO.setSelect(select);

        //where
        List<WhereAndOrBean> where=new ArrayList<>();
        if(CollUtil.isNotEmpty(params.getCondtions())){
            params.getCondtions().forEach(item->{
                TableEntityPropertyVO tableColumn= tableColumnMap.get(item.getPropCode());
                if(tableColumn!=null){
                    where.add(new WhereAndOrBean(new WhereItemBean(params.getTableId(),item.getPropCode(),tableColumn.getEntityDataType(),
                            item.getOp(),item.getVal(),item.getVal2(),item.getVals())));
                }
            });
        }

        queryDTO.setWhere(where);

        //order by
        if(CollUtil.isNotEmpty(params.getOrderBys())){
            List<OrderByBean> orderBy=new ArrayList<>();
            params.getOrderBys().forEach(item->{
                orderBy.add(new OrderByBean(params.getTableId(),item.getPropCode(),item.getOrderByType()));
            });
            queryDTO.setOrderBy(orderBy);
        }

        IPage<Map<String, Object>> page= entityOperateApiService.query(queryDTO);
        return page;


    }








}
