package com.xxh.cloud.etl.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.data.framework.modules.biz.bean.dto.DwTableDataGetDTO;
import com.xxh.cloud.data.framework.modules.biz.bean.dto.DwTableDataQueryDTO;
import com.xxh.cloud.data.framework.modules.biz.bean.dto.DwTableNameQueryDTO;
import com.xxh.cloud.data.framework.modules.biz.bean.vos.DwDataInfoVO;
import com.xxh.cloud.data.framework.modules.biz.bean.vos.DwDataQueryVO;
import com.xxh.cloud.data.framework.modules.biz.service.DwDataBaseQueryService;
import com.xxh.cloud.data.framework.modules.cache.service.DwTableEntityCacheService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.metadata.framework.modules.script.bean.vos.TableEntityVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/dw/data/dwd")
@Api(tags = "运营端,ODS数据管理")
public class DwDataDwdManagerController {




    @Autowired
    private DwDataBaseQueryService dwDataBaseQueryService;

    @Autowired
    private DwTableEntityCacheService dwTableEntityCacheService;


    @SaCheckPermission("sys:dw:data:dwd:tableList")
    @PostMapping("/tableList")
    @ApiOperation(value = "DWD表查询", response = TableEntityVO.class)
    public Result<List<TableEntityVO>> tableList(@RequestBody DwTableNameQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<TableEntityVO> data= dwTableEntityCacheService.getTableListLike("dwd");
        return new Result<List<TableEntityVO>>().ok(data);
    }

    @SaCheckPermission("sys:dw:data:dwd:queryData")
    @PostMapping("/queryData")
    @ApiOperation(value = "查询数据", response = DwDataQueryVO.class)
    public Result<DwDataQueryVO> queryData(@RequestBody DwTableDataQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        DwDataQueryVO data= dwDataBaseQueryService.query(params);
        return new Result<DwDataQueryVO>().ok(data);
    }

    @SaCheckPermission("sys:dw:data:dwd:getDataInfo")
    @PostMapping("/getDataInfo")
    @ApiOperation(value = "获取数据", response = DwDataInfoVO.class)
    public Result<DwDataInfoVO> getDataInfo(@RequestBody DwTableDataGetDTO params) {
        ValidatorUtils.validateEntity(params);
        DwDataInfoVO data= dwDataBaseQueryService.get(params);
        return new Result<DwDataInfoVO>().ok(data);
    }





}
