package com.xxh.cloud.exeEtl.framework.lab.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LabRegexUtil {

    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     *
     * 格式：
     * 例子：
     * 12--24
     * 23.4--54.1
     * 0.38 --4.34
     * 0.38 --  4.34
     * -0.38 --  4.34
     * - 0.38 --  4.34
     * - 0.38 --  -4.34
     * - 0.38 --- 4.34  X  不判断后面的值是否是负数
     *
     * 数字可以是整数也可以是小数；双横杠；横杠前后可以有空格（最多一个）
     *
     * @param str 给定需要匹配的字符串
     * @return 返回匹配到的字符串集合
     */
    public static List<String> erange(String str) {
        return LabRegexUtil.listPatternStr(str, "(-?\\s*)?[0-9]+\\.?[0-9]*\\s*--(\\s*|\\s+-\\s*)[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistErange(String str) {
        return LabRegexUtil.erange(str).size() > 0 ? true : false;
    }

    public static int erangeCount(String str) {
        return LabRegexUtil.erange(str).size();
    }

    /**
     * 格式：
     * 例子：
     * <5.0
     * < 5.0
     * < -5.0
     * < - 5.0
     *
     * 小于号；
     * 数字可以是整数也可以是小数；
     *
     * @param str
     * @return
     */
    public static List<String> rrange(String str) {
        return LabRegexUtil.listPatternStr(str, "<(\\s*|\\s*-\\s*)[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistRrange(String str) {
        return LabRegexUtil.rrange(str).size() > 0 ? true : false;
    }

    public static int rrangeCount(String str) {
        return LabRegexUtil.rrange(str).size();
    }


    /**
     * ≤5.0
     * ≤ 5.0
     * ≤ -5.0
     * ≤ - 5.0
     *
     * 小于等于号；
     * 数字可以是整数也可以是小数；
     *
     * @param str
     * @return
     */
    public static List<String> rerange(String str) {
        return LabRegexUtil.listPatternStr(str, "≤(\\s*|\\s*-\\s*)[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistRerange(String str) {
        return LabRegexUtil.rerange(str).size() > 0 ? true : false;
    }

    public static int rerangeCount(String str) {
        return LabRegexUtil.rerange(str).size();
    }


    /**
     * 格式：
     * 例子：
     * >5.0
     * > 5.0
     * > -5.0
     * > - 5.0
     *
     * 大于号；
     * 数字可以是整数也可以是小数；
     *
     * @param str
     * @return
     */
    public static List<String> lrange(String str) {
        return LabRegexUtil.listPatternStr(str, ">(\\s*|\\s*-\\s*)[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistLrange(String str) {
        return LabRegexUtil.lrange(str).size() > 0 ? true : false;
    }

    public static int lrangeCount(String str) {
        return LabRegexUtil.lrange(str).size();
    }


    /**
     * 格式：
     * 例子：
     * ≥5.0
     * ≥ 5.0
     * ≥ -5.0
     * ≥ - 5.0
     *
     * 大于等于号；
     * 数字可以是整数也可以是小数；
     *
     * @param str
     * @return
     */
    public static List<String> lerange(String str) {
        return LabRegexUtil.listPatternStr(str, "≥(\\s*|\\s*-\\s*)[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistLerange(String str) {
        return LabRegexUtil.lerange(str).size() > 0 ? true : false;
    }

    public static int lerangeCount(String str) {
        return LabRegexUtil.lerange(str).size();
    }


    /**
     * 格式：
     * 例子：
     * 5.0E+001--6.0E+001
     * 5.0E+001  --6.0E+001
     * 5.0E+001--  6.0E+001
     * 5.0E+001  --  6.0E+001
     * -5.0E+001  --  6.0E+001
     * - 5.0E+001  --  6.0E+001
     * - 5.0E+001  --  -6.0E+001
     * - 5.0E+001  --  - 6.0E+001
     * - 5.0E+001  --- 6.0E+001     X   不支持
     *
     *
     * @param str
     * @return
     */
    public static List<String> serange(String str) {
        return LabRegexUtil.listPatternStr(str, "(-?\\s*)?[0-9]+\\.?[0-9]*E\\+[0-9]+\\s*(-|--)(\\s*|\\s+-\\s*)[0-9]+\\.?[0-9]*E\\+[0-9]+");
    }

    public static boolean isExistSerange(String str) {
        return LabRegexUtil.serange(str).size() > 0 ? true : false;
    }

    public static int serangeCount(String str) {
        return LabRegexUtil.serange(str).size();
    }


    /**
     * 格式：
     * 例子：
     * <5.0E+001
     * < 5.0E+001
     * < -5.0E+001
     * < - 5.0E+001
     *
     * @param str
     * @return
     */
    public static List<String> srrange(String str) {
        return LabRegexUtil.listPatternStr(str, "<(\\s*|\\s+-\\s*)[0-9]+\\.?[0-9]*E\\+[0-9]+");
    }

    public static boolean isExistSrrange(String str) {
        return LabRegexUtil.srrange(str).size() > 0 ? true : false;
    }

    public static int srrangeCount(String str) {
        return LabRegexUtil.srrange(str).size();
    }

    /**
     * 格式：
     * 例子：
     * ≤5.0E+001
     * ≤ 5.0E+001
     * ≤ -5.0E+001
     * ≤ - 5.0E+001
     *
     * @param str
     * @return
     */
    public static List<String> srerange(String str) {
        return LabRegexUtil.listPatternStr(str, "≤(\\s*|\\s+-\\s*)[0-9]+\\.?[0-9]*E\\+[0-9]+");
    }

    public static boolean isExistSrerange(String str) {
        return LabRegexUtil.srerange(str).size() > 0 ? true : false;
    }

    public static int srerangeCount(String str) {
        return LabRegexUtil.srerange(str).size();
    }


    /**
     * 格式：
     * 例子：
     * >5.0E+001
     * > 5.0E+001
     * > -5.0E+001
     * > - 5.0E+001
     *
     * @param str
     * @return
     */
    public static List<String> slrange(String str) {
        return LabRegexUtil.listPatternStr(str, ">(\\s*|\\s+-\\s*)[0-9]+\\.?[0-9]*E\\+[0-9]+");
    }

    public static boolean isExistSlrange(String str) {
        return LabRegexUtil.slrange(str).size() > 0 ? true : false;
    }

    public static int slrangeCount(String str) {
        return LabRegexUtil.slrange(str).size();
    }

    /**
     * 格式：
     * 例子：
     * ≥5.0E+001
     * ≥ 5.0E+001
     * ≥ -5.0E+001
     * ≥ - 5.0E+001
     *
     * @param str
     * @return
     */
    public static List<String> slerange(String str) {
        return LabRegexUtil.listPatternStr(str, "≥(\\s*|\\s+-\\s*)[0-9]+\\.?[0-9]*E\\+[0-9]+");
    }

    public static boolean isExistSlerange(String str) {
        return LabRegexUtil.slerange(str).size() > 0 ? true : false;
    }

    public static int slerangeCount(String str) {
        return LabRegexUtil.slerange(str).size();
    }




    /**
     * 例子：
     * 滤泡期：0.4-2.5；排卵期：1.2-24.8黄体期：5.5-24.8； 绝经期：0.4-1.4滤泡期：0.4-2.5孕早期：4.2--94.8孕晚期：26.8--228.2
     * 卧位：25--60||立位：50--120||
     *
     * @param str
     * @return
     */
    public static List<String> terange(String str) {
        return LabRegexUtil.listPatternStr(str, "[\\u4e00-\\u9fa5]+\\s*(：|:)(\\s*|\\s*-\\s*)[0-9]+\\.?[0-9]*\\s*(-|--)(\\s*|\\s+-\\s*)[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistTerange(String str) {
        return LabRegexUtil.terange(str).size() > 0 ? true : false;
    }

    public static int terangeCount(String str) {
        return LabRegexUtil.terange(str).size();
    }



    /**
     * 例子：
     * 滤泡期：<0.1--0.6      黄体期：1.0--15.2;绝经期：<0.1--0.4      孕早期：4.2--94.8;孕中期：22.9--112.6   孕晚期：26.8--228.2
     *
     * @param str
     * @return
     */
    public static List<String> pattern3(String str) {
        return LabRegexUtil.listPatternStr(str, "[^|]\\D{2}\\s*(：|:)\\s*(<|≤)?[0-9]+\\.?[0-9]*\\s*(-|--)\\s*[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern3(String str) {
        return LabRegexUtil.pattern3(str).size() > 0 ? true : false;
    }

    public static int pattern3Count(String str) {
        return LabRegexUtil.pattern3(str).size();
    }



    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     *
     * 格式：
     * 例子：
     * -12--24
     * 23.4-- -54.1
     * 0.38 -- -4.34
     * -0.38 --  -4.34
     *
     * 数字可以是整数也可以是小数；双横杠；横杠前后可以有空格（最多一个）
     *
     * @param str 给定需要匹配的字符串
     * @return 返回匹配到的字符串集合
     */
    public static List<String> pattern9(String str) {
        return LabRegexUtil.listPatternStr(str, "-?[0-9]+\\.?[0-9]*\\s*--(\\s*|\\s+-)?[0-9]+\\.?[0-9]*");
    }

    public static boolean isExistPattern9(String str) {
        return LabRegexUtil.pattern9(str).size() > 0 ? true : false;
    }

    public static int pattern9Count(String str) {
        return LabRegexUtil.pattern9(str).size();
    }



    /**
     * 返回给定字符串中符合正则表达式的字符串格内容
     *
     * @param str  需要匹配的数据字符串
     * @param regex 正则表达式
     * @return
     */
    public static List<String> listPatternStr(String str, String regex) {
        List<String> list = new ArrayList<String>();
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(str);
        while(m.find()) {
            list.add(m.group());
        }
        return list;
    }



    public static void main(String[] args) {
        // List<String> list1 = pattern1("ghdashgk32.344 --12.56dsg12--24jk12  --24dh12--  24ag12  --  24k10-- 4grhnhkfns");
        // list1.forEach(i -> System.out.println(i));

        // System.out.println("\n-----\n");

        // List<String> list2 = pattern2("ddbs滤泡期：0.4- 2.5；排卵期：1.2 -24.8黄体期：5.5 - 24.8； 绝经期：0.4-  1.4滤泡期：0.4-2.5ewtryg");
        // list2.forEach(i -> System.out.println(i));

        // System.out.println("\n-----\n");

        // List<String> list3 = terange("ddbshd滤泡期：<0.1  --0.6      黄体期：1.0-- 15.2;绝经期：<0.1--0.4      孕早期：4.2--  94.8;孕中期：22.9   --112.6   孕晚期：26.8  --  228.2ewtr卧位：- 25--  60||立位：50- -120||yg");
        // list3.forEach(i -> System.out.println(i));

        // System.out.println("\n-----\n");

        // List<String> list4 = pattern2("ddbsh卧位：25--60||立位：50--120||wtryg");
        // list4.forEach(i -> System.out.println(i));

        // System.out.println("\n-----\n");

        // List<String> list6 = pattern6("ddbshd<5.0E+001ewtryg");
        // list6.forEach(i -> System.out.println(i));

        // System.out.println("\n-----\n");

        // List<String> list7 = pattern7("ddbshd≤5.0tryg");
        // list7.forEach(i -> System.out.println(i));

        // List<String> list8 = pattern8("dd5.0E+001--6.0E+001bshd5.0E+001  --  6.0E+001tryg");
        // list8.forEach(i -> System.out.println(i));


        // List<String> list9 = pattern9("dd5.0E+001--6.0E+001-12-- -24bshd5.0E+001  --  6.0E+001tryg");
        // list9.forEach(i -> System.out.println(i));

    }

}
