package com.xxh.cloud.exeEtl.framework.modules.biz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.exeEtl.framework.modules.base.entity.DataCleanErrorLogInfo;
import com.xxh.cloud.exeEtl.framework.modules.base.service.DataCleanErrorLogInfoService;
import com.xxh.cloud.exeEtl.framework.modules.biz.bean.dto.DataCleanErrorLoQueryDTO;
import com.xxh.cloud.exeEtl.framework.modules.biz.bean.dto.DataCleanErrorLogAddDTO;
import com.xxh.cloud.exeEtl.framework.modules.biz.bean.vos.DataCleanErrorLogVO;
import com.xxh.cloud.exeEtl.framework.modules.biz.service.DataCleanErrorLogBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;


@Service
public class DataCleanErrorLogBizServiceImpl implements DataCleanErrorLogBizService {

    @Autowired
    private DataCleanErrorLogInfoService dataCleanErrorLogInfoService;


    @Override
    public PageData<DataCleanErrorLogVO> query(DataCleanErrorLoQueryDTO params) {
        ValidatorUtils.validateEntity(params);


        IPage<DataCleanErrorLogInfo> page = dataCleanErrorLogInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<DataCleanErrorLogInfo>()
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getProp()),"prop",params.getProp())
                        .eq(StrUtil.isNotEmpty(params.getRuleCode()),"rule_code",params.getRuleCode())
                        .like(StrUtil.isNotEmpty(params.getRuleName()),"rule_name",params.getRuleName())
                        .orderByDesc("op_date")
        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        PageData<DataCleanErrorLogVO> data= new PageConverter<DataCleanErrorLogInfo, DataCleanErrorLogVO>() {
            @Override
            public DataCleanErrorLogVO converter(DataCleanErrorLogInfo entity) {
                DataCleanErrorLogVO dataItem=new DataCleanErrorLogVO();
                BeanUtil.copyProperties(entity,dataItem);
                return dataItem;
            }
        }.getPageData(page);

        return data;

    }

    @Override
    public void add(DataCleanErrorLogAddDTO params) {
        ValidatorUtils.validateEntity(params);
        DataCleanErrorLogInfo entity=getErrorLogEntity(params);
        dataCleanErrorLogInfoService.save(entity);
    }

    @Override
    public void addBatch(List<DataCleanErrorLogAddDTO> params) {
        if(CollUtil.isEmpty(params)){
            return;
        }
        List<DataCleanErrorLogInfo> list=new ArrayList<>();
        for(DataCleanErrorLogAddDTO item:params){
            DataCleanErrorLogInfo entityItem=getErrorLogEntity(item);
            list.add(entityItem);
        }
        dataCleanErrorLogInfoService.saveBatch(list);
    }


    private DataCleanErrorLogInfo getErrorLogEntity(DataCleanErrorLogAddDTO param){
        DataCleanErrorLogInfo entity=new DataCleanErrorLogInfo();
        entity.setId(IdUtil.getSnowflake(1,1).nextIdStr());
        entity.setName(param.getName());
        entity.setProp(param.getProp());
        entity.setRuleCode(param.getRuleCode());
        entity.setRuleName(param.getRuleName());
        entity.setObjContent(param.getObjContent());
        entity.setErrorInfo(param.getErrorInfo());
        entity.setStatus(YN.NO.getCode());
        entity.setOpUser("-1");
        entity.setOpDate(param.getOpDate());
        return entity;
    }


}

