package com.xxh.cloud.exeEtl.framework.modules.biz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.etl.framework.modules.base.entity.EtlDataCleanConfigItemInfo;
import com.xxh.cloud.etl.framework.modules.biz.bean.vos.DataCleanConfigDetailVO;
import com.xxh.cloud.etl.framework.modules.biz.convert.DataCleanConfigDetailConvert;
import com.xxh.cloud.exeEtl.framework.modules.base.entity.DataCleanLogInfo;
import com.xxh.cloud.exeEtl.framework.modules.base.service.DataCleanLogInfoService;
import com.xxh.cloud.exeEtl.framework.modules.biz.bean.dto.DataCleanLogAddDTO;
import com.xxh.cloud.exeEtl.framework.modules.biz.bean.dto.DataCleanLogQueryDTO;
import com.xxh.cloud.exeEtl.framework.modules.biz.bean.vos.DataCleanLogVO;
import com.xxh.cloud.exeEtl.framework.modules.biz.service.DataCleanLogBizService;
import com.xxh.cloud.exeEtl.framework.modules.clean.bean.dto.ExeDataCleanLogAddDTO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class DataCleanLogBizServiceImpl implements DataCleanLogBizService {

    @Autowired
    private DataCleanLogInfoService dataCleanLogInfoService;


    @Override
    public PageData<DataCleanLogVO> query(DataCleanLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);

        IPage<DataCleanLogInfo> page = dataCleanLogInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<DataCleanLogInfo>()
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getProp()),"prop",params.getProp())
                        .eq(StrUtil.isNotEmpty(params.getRuleCode()),"rule_code",params.getRuleCode())
                        .like(StrUtil.isNotEmpty(params.getRuleName()),"rule_name",params.getRuleName())
                        .orderByDesc("start_date")
        );
        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        PageData<DataCleanLogVO> data= new PageConverter<DataCleanLogInfo, DataCleanLogVO>() {
            @Override
            public DataCleanLogVO converter(DataCleanLogInfo entity) {
                DataCleanLogVO dataItem=new DataCleanLogVO();
                BeanUtil.copyProperties(entity,dataItem);
                return dataItem;
            }
        }.getPageData(page);

        return data;

    }

    @Override
    public void add(DataCleanLogAddDTO params) {
        ValidatorUtils.validateEntity(params);
    }

    @Override
    public void saveBatch(List<ExeDataCleanLogAddDTO> params) {
        if(CollUtil.isEmpty(params)){
            return;
        }
        List<DataCleanLogInfo> list=new ArrayList<>();
        params.forEach(item->{
            list.add(getDataCleanLogInfo(item,"-1"));
        });

        if(list.size()>0){
            dataCleanLogInfoService.saveBatch(list);
        }
    }


    private DataCleanLogInfo getDataCleanLogInfo(ExeDataCleanLogAddDTO item,String opUserId){
        DataCleanLogInfo entity=new DataCleanLogInfo();
        entity.setId(IdUtil.getSnowflake(6,1).nextIdStr());
        entity.setName(item.getTable());
        entity.setProp(item.getCol());
        entity.setRuleCode(item.getRuleCode());
        entity.setRuleName(item.getRuleName());
        entity.setTotalNum(item.getTotalNum());
        entity.setTotalSuccess(item.getSuccessNum());
        entity.setTotalFail(item.getFailNum());
        entity.setStartDate(item.getStartDate());
        entity.setEndDate(item.getEndDate());
        entity.setOpUser(opUserId);
        entity.setOpDate(new Date());
        return entity;
    }



    @Override
    public void saveBatch(Map<String, ExeDataCleanLogAddDTO> params) {
        if(MapUtil.isEmpty(params)){
            return;
        }
        List<ExeDataCleanLogAddDTO> list=new ArrayList<>();
        params.forEach((k,v)->{
            list.add(v);
        });
        this.saveBatch(list);
    }

    @Override
    public DataCleanLogVO getById(String id) {
        return null;
    }


}
