package com.xxh.cloud.exeEtl.framework.modules.clean.bean.dto;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.bean.MQPatientEventBean;
import com.xxh.cloud.common.mq.bean.MQRuleLabelBean;
import com.xxh.cloud.etl.common.framework.constant.PatientEventBO;
import com.xxh.cloud.etl.common.framework.constant.PatientEventType;
import com.xxh.cloud.etl.common.framework.modules.bean.dwd.DiagnosisReportDBRO;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.DataCleanConfigDetailSimpleSaveDTO;
import com.xxh.cloud.exeEtl.framework.modules.biz.bean.dto.DataCleanErrorLogAddDTO;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import lombok.Getter;
import lombok.Setter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Getter
@Setter
public class DataCleanProcessTwoContext {


    private static final Logger logger = LoggerFactory.getLogger(DataCleanProcessTwoContext.class);




    /**
     *
     * */
    private Map<String,ExeDataCleanLogAddDTO> logMap;

    /**
     *
     * */
    private List<DataCleanConfigDetailSimpleSaveDTO> needAddList;


    /**患者数据日期*/
    private List<Map<String,Object>> patientDataTimeList;

//    /**诊断 DiagnosisReportDBRO*/
//    private List<Map<String,Object>> diagnosisList;

    /**错误日志*/
    private List<DataCleanErrorLogAddDTO> errorLogs;



    /**
     * 规则标记
     * */
    private boolean calFlag;

    /**
     * 规则计算数据
     * */
    private List<MQRuleLabelBean> calDataList=new ArrayList<>();

    //todo:2024-7-18
//    /**
//     * 规则计算
//     * */
//    @Deprecated
//    private Set<String> calPatientIds=new HashSet<>();


    /**
     * 事件标记
     * */
    private boolean eventFlag;

    /**
     * 事件数据
     * */
    private List<MQPatientEventBean> eventList;


    /**
     * 数据MQ标记
     * */
    private boolean dataMqFlag;

    /**数据MQ队列*/
    private Map<String,List<Object>> dataQueueMap;

    /**数据MQ广播*/
    private Map<String,List<Object>> dataTopicMap;

    /**
     * 患者事件数据
     * */
    private Map<String,List<PatientEventBO>> patientEventMap;







   public DataCleanProcessTwoContext(){
    }


    public void addPatientEvent(PatientEventType key, PatientEventBO patientEvent){
       if(this.patientEventMap==null){
           this.patientEventMap=new HashMap<>();
       }
       List<PatientEventBO> patientEventList= this.patientEventMap.get(key.getCode());
       if(patientEventList==null){
           patientEventList=new ArrayList<>();
           this.patientEventMap.put(key.getCode(),patientEventList);
       }
       patientEventList.add(patientEvent);
    }



    /**
     *
     * */
    public void addDataMqItem(String key, Object item){
        if(this.dataQueueMap==null){
            this.dataQueueMap=new HashMap<>();
        }
        List<Object> dataItemList= this.dataQueueMap.get(key);

        if(dataItemList==null){
            dataItemList=new ArrayList<>();
            this.dataQueueMap.put(key,dataItemList);
        }
        if(item!=null){
            dataItemList.add(item);
        }
    }

    /**
     *
     * */
    public void addPatientDataTime(String patientId,Date time,String deptCode,String deptName,String doctorCode,String doctorName){
        if(this.patientDataTimeList==null){
            this.patientDataTimeList=new ArrayList<>();
        }
        if(StrUtil.isEmpty(patientId) || time==null){
            return;
        }
        Map<String,Object> dataItem=new HashMap<>();
        dataItem.put("patient_id",patientId);
        dataItem.put("last_op_date",time);
        dataItem.put("data_day", DateUtil.format(time,"yyyyMMdd"));
        dataItem.put("data_num",1);

        if(StrUtil.isNotEmpty(deptName) && StrUtil.isNotEmpty(doctorName)){
            String dateStr= DateUtil.format(time, DatePattern.NORM_DATETIME_MS_FORMAT);
            Map<String,String> ddMap=new HashMap<>();
            ddMap.put("deptCode",deptCode);
            ddMap.put("deptName",deptName);
            ddMap.put("doctorCode",doctorCode);
            ddMap.put("doctorName",doctorName);
            dataItem.put("content",StrUtil.format("{}__{}",dateStr, JSON.toJSONString(ddMap)));
        }

        this.patientDataTimeList.add(dataItem);
    }

    /**
     *
     * */
    public void addPatientDataTime(Object patientId,Object time,Object deptCode,Object deptName,Object doctorCode,Object doctorName){
        if(patientId==null || time ==null){
            return;
        }
        //
        String _patientId=Convert.toStr(patientId);
        //

        Date _time=null;
        _time = Convert.toDate(time);

        if(_time==null){
            String _str=Convert.toStr(time);
            _time=DataCleanDateTimeUtils.toDate(_str);
            if(_time==null){
                logger.error("+++++++ 不支持的时间日期格式,{}",_str);
                return;
            }
        }

        this.addPatientDataTime(_patientId,_time,Convert.toStr(deptCode),Convert.toStr(deptName),Convert.toStr(doctorCode),Convert.toStr(doctorName));
    }





    /**
     * 新增错误日志
     * */
    public void addErrorLog(DataCleanErrorLogAddDTO item){
        if(this.errorLogs==null){
            this.errorLogs=new ArrayList<>();
        }
        if(item!=null){
            this.errorLogs.add(item);
        }
    }


    public void addErrorLog(String ruleCode,String ruleName,String table, String col,String objContent,String errorMsg){
        DataCleanErrorLogAddDTO item=new DataCleanErrorLogAddDTO();
        item.setName(table);
        item.setProp(col);
        item.setRuleCode(ruleCode);
        item.setRuleName(ruleName);
        item.setObjContent(objContent);
        item.setErrorInfo(errorMsg);
        item.setOpDate(new Date());
        this.addErrorLog(item);
    }
    //



    /**
     * 新增事件
     * */
    public void addEvent(MQPatientEventBean item){
        if(this.eventList==null){
            this.eventList=new ArrayList<>();
        }
        if(item!=null){
            this.eventList.add(item);
        }
    }


    /**
     * 新增计算患者
     * */
    public void addRuleDataObj(Object patientId,String sceneType,Object dataTime,Object dataId){
        if(patientId==null ||  StrUtil.isEmpty(sceneType)){
            return;
        }

        String date=null;
        if(dataTime!=null){
            if(dataTime instanceof String){
                date=(String) dataTime;
            }else {
                Date d=Convert.toDate(dataTime);
                date=DateUtil.format(d,"yyyy-MM-dd HH:mm:ss");
            }
        }

        MQRuleLabelBean ruleLabelBean=new MQRuleLabelBean(Convert.toStr(patientId),sceneType,date,Convert.toStr(dataId));
        this.addRuleData(ruleLabelBean);
    }


    /**
     * 新增计算患者
     * */
    public void addRuleData(String patientId,String sceneType,String dataTime,String dataId){
        if(StrUtil.isEmpty(patientId) || StrUtil.isEmpty(sceneType)){
            return;
        }
        MQRuleLabelBean ruleLabelBean=new MQRuleLabelBean(patientId,sceneType,dataTime,dataId);
        this.addRuleData(ruleLabelBean);
    }

    /**
     * 新增计算患者
     * */
    public void addRuleData(MQRuleLabelBean item){
        if(this.calDataList==null){
            this.calDataList=new ArrayList<>();
        }
        if(item!=null){
            this.calDataList.add(item);
        }
    }





    /**
     *  需要清洗
     * */
    public void addNeedItem(DataCleanConfigDetailSimpleSaveDTO item){
        if(this.needAddList==null){
            this.needAddList=new ArrayList<>();
        }
        if(item!=null){
            this.needAddList.add(item);
        }
    }


    public void addNeedItem( String ruleCode, String sourceCode,String sourceName){
        DataCleanConfigDetailSimpleSaveDTO  item=new DataCleanConfigDetailSimpleSaveDTO(ruleCode,sourceCode,sourceName);
        this.addNeedItem(item);
    }

    public void addLogSuccess(String ruleCode,String col){
        if(this.logMap==null){
            this.logMap=new HashMap<>();
        }
        String key= StrUtil.format("{}:{}",col,ruleCode);
        ExeDataCleanLogAddDTO item = this.logMap.get(key);
        if(item==null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"初始化日志不能为空");
        }
        item.addSuccessNum(1);
        item.addTotalNum(1);
        item.setEndDate(new Date());
    }


    public void addLogFail(String ruleCode, String col){
        if(this.logMap==null){
            this.logMap=new HashMap<>();
        }
        String key= StrUtil.format("{}:{}",col,ruleCode);
        ExeDataCleanLogAddDTO item = this.logMap.get(key);
        if(item==null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"初始化日志不能为空");
        }
        item.addFailNum(1);
        item.addTotalNum(1);
        item.setEndDate(new Date());
    }

    public void initLogMap(String table,String col,String ruleCode,String ruleName){
        if(this.logMap==null){
            this.logMap=new HashMap<>();
        }
        String key= StrUtil.format("{}:{}",col,ruleCode);
        ExeDataCleanLogAddDTO item= this.logMap.get(key);
        if(item!=null){
            return;
        }
        item=new ExeDataCleanLogAddDTO(table,col,ruleCode,ruleName);
        this.logMap.put(key,item);
    }



}
