package com.xxh.cloud.exeEtl.framework.modules.clean.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.etl.client.api.bean.bos.DataCleanConfigBO;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.DiagnoseBO;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.OperationDiagnoseBO;
import com.xxh.cloud.etl.common.framework.modules.write.bean.ods.ExamProjectBO;
import com.xxh.cloud.etl.common.framework.modules.write.bean.ods.LabProjectReportItemBO;
import com.xxh.cloud.exeEtl.framework.modules.clean.bean.dto.DataCleanProcessTwoContext;
import com.xxh.cloud.exeEtl.framework.modules.clean.bean.ros.DataCleanDiagnoseBean;
import com.xxh.cloud.exeEtl.framework.modules.clean.bean.ros.DataCleanLabProject;
import com.xxh.cloud.exeEtl.framework.modules.clean.service.DataCleanProcessTwoService;
import com.xxh.cloud.exeEtl.framework.modules.clean.utils.*;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Log4j2
@Service
public class DataCleanProcessTwoServiceImpl implements DataCleanProcessTwoService {


    @Override
    public CleanResultValueRO processMapping(DataCleanProcessTwoContext context,String table ,String col, String val, DataCleanConfigBO config) {
        context.initLogMap(table,col,config.getCode(),config.getName());

        CleanResultRO result = CleanMappingUtils.process(val,config);
        if(result.isResult()){
            context.addLogSuccess(config.getCode(),col);
            return new CleanResultValueRO(result.getStandardCode(),result.getStandardName(),result.getResultType());
        }

        context.addLogFail(config.getCode(),col);
        context.addNeedItem(config.getCode(),val,val);
        return null;
    }

    @Override
    public CleanResultValueRO processMapping(DataCleanProcessTwoContext context, String table, String col, String code, String name, DataCleanConfigBO config) {
        context.initLogMap(table,col,config.getCode(),config.getName());
        if(StrUtil.isEmpty(name)){
            name=code;
        }
        CleanResultRO result = CleanMappingUtils.process(code,name,config);
        if(result.isResult()){
            context.addLogSuccess(config.getCode(),col);
            return new CleanResultValueRO(result.getStandardCode(),result.getStandardName(),result.getResultType());
        }
        context.addLogFail(config.getCode(),col);
        context.addNeedItem(config.getCode(),code,name);
        return null;
    }

    @Override
    public List<CleanResultValueRO> processKeywords(DataCleanProcessTwoContext context,String table, String col, String val, DataCleanConfigBO config) {
        context.initLogMap(table,col,config.getCode(),config.getName());

        try {
            CleanResultRO result= CleanKeywordsUtils.process(context,val,config,table,col);
            if(result.isResult()){
                context.addLogSuccess(config.getCode(),col);
                return result.getKeywordsVals();
            }
            context.addLogFail(config.getCode(),col);
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public  List<DataCleanDiagnoseBean> processDiagnose(DataCleanProcessTwoContext context,String table, String col, List<DiagnoseBO> vals, DataCleanConfigBO config) {
        context.initLogMap(table,col,config.getCode(),config.getName());

        List<DataCleanDiagnoseBean> data=new ArrayList<>();

        List<CleanResultRO> resultList = CleanDiagnoseUtils.process(vals,config);
        if(CollUtil.isEmpty(resultList)){
            return data;
        }

        for(CleanResultRO result:resultList){
            if(result.isResult()){
                context.addLogSuccess(config.getCode(),col);
                data.add( result.getDiagnose());
            }else {
                context.addLogFail(config.getCode(),col);
                context.addNeedItem(config.getCode(),result.getSourceCode(),result.getSourceName());
            }
        }
        return data;

    }

    @Override
    public List<DataCleanDiagnoseBean> processDiagnoseOp(DataCleanProcessTwoContext context, String table, String col, List<OperationDiagnoseBO> vals, DataCleanConfigBO config) {
        context.initLogMap(table,col,config.getCode(),config.getName());

        List<DataCleanDiagnoseBean> data=new ArrayList<>();

        List<CleanResultRO> resultList = CleanDiagnoseUtils.processOp(vals,config);
        if(CollUtil.isEmpty(resultList)){
            return data;
        }

        for(CleanResultRO result:resultList){
            if(result.isResult()){
                context.addLogSuccess(config.getCode(),col);
                data.add( result.getDiagnose());
            }else {
                context.addLogFail(config.getCode(),col);
                context.addNeedItem(config.getCode(),result.getSourceCode(),result.getSourceName());
            }
        }
        return data;
    }

    @Override
    public List<CleanResultValueRO> processExamProject(DataCleanProcessTwoContext context, String table, String col, List<ExamProjectBO> vals, DataCleanConfigBO config) {
        context.initLogMap(table,col,config.getCode(),config.getName());

        List<CleanResultValueRO> data=new ArrayList<>();

        for(ExamProjectBO val:vals){
            CleanResultRO result = CleanMappingUtils.process(val.getCode(),val.getName(),config);
            if(result.isResult()){
                context.addLogSuccess(config.getCode(),col);
                data.add(new CleanResultValueRO(result.getStandardCode(),result.getStandardName()));
            }else {
                context.addLogFail(config.getCode(),col);
                context.addNeedItem(config.getCode(),result.getSourceCode(),result.getSourceName());
            }
        }
        return data;
    }

    @Override

    public DataCleanLabProject processLabProject(DataCleanProcessTwoContext context,String table, String col, LabProjectReportItemBO val,
                                                 String resultType, DataCleanConfigBO labTwoCategoryConfig){


        context.initLogMap(table,col,resultType,resultType);
        if(labTwoCategoryConfig!=null){
            context.initLogMap(table,col,labTwoCategoryConfig.getCode(),labTwoCategoryConfig.getName());
        }
        try {
            CleanResultRO result= CleanLabProjectUtils.process(val,resultType,labTwoCategoryConfig);

            if(result==null){
                return null;
            }

            if(result.isResult()){
                context.addLogSuccess(result.getRuleCode(),col);
                return result.getLabVal();
            }

            if(labTwoCategoryConfig.getCode().equals(result.getRuleCode())){
                context.addNeedItem(labTwoCategoryConfig.getCode(),result.getSourceCode(),result.getSourceCode());
            }

            context.addLogFail(result.getRuleCode(),col);

        }catch (Exception e){
            context.addErrorLog("LabReportItem","检验项目计算",table,col, JSON.toJSONString(val),e.getMessage());
            System.out.println("++++ 检验项目清洗 error:"+JSON.toJSONString(val));
            e.printStackTrace();
        }


        return null;
    }




}
