package com.xxh.cloud.exeEtl.framework.modules.clean.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.etl.client.api.bean.bos.DataCleanConfigBO;
import com.xxh.cloud.etl.common.framework.modules.write.bean.ods.LabProjectReportItemBO;
import com.xxh.cloud.exeEtl.framework.modules.clean.bean.ros.DataCleanLabProject;
import com.xxh.cloud.exeEtl.framework.modules.clean.labReportUtils.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.List;

public class CleanLabProjectUtils {


    private static final Logger logger = LoggerFactory.getLogger(CleanLabProjectUtils.class);




    public static CleanResultRO process(LabProjectReportItemBO val,String resultType, DataCleanConfigBO labTwoCategory) {

        //二分类结果[MSRT_ResultTwoCategory]
        if("result_two_category".equals(resultType)){
            if(labTwoCategory==null){
                return null;
            }
            CleanResultRO labTwoResult =  CleanMappingUtils.process(val.getInspect_result(),labTwoCategory);
            if(labTwoResult.isResult()){
                DataCleanLabProject data=new DataCleanLabProject();
                data.setResultType("result_two_category");
                data.setTwoCategoryCode(labTwoResult.getStandardCode());
                data.setTwoCategoryName(labTwoResult.getStandardName());
                labTwoResult.setLabVal(data);
            }
            return labTwoResult;
        }



        //叙述结果[MSRT_ResultNarrate]
        if("result_narrate".equals(resultType)){
            DataCleanLabProject data=new DataCleanLabProject();
            data.setResultType("result_narrate");
            data.setNarrate(val.getInspect_result());
            return CleanResultRO.ok(resultType,resultType,data);
        }


        //结果值[MSRT_ResultValue]
        if("result_value".equals(resultType)){
            //todo:1
            DataCleanLabProject data=new DataCleanLabProject();
            data.setResultType("result_value");
            CleanLabProjectUtils.resultValue(data,val);
            //todo:2
//          DataCleanLabProject data2=CleanLabProject_Value.calValue(val);
            return CleanResultRO.ok(resultType,resultType,data);
        }

        return null;
    }






    /**
     * 结果值
     * */
    public static void resultValue(DataCleanLabProject dataCleanLabProject,LabProjectReportItemBO val){
        boolean flag= NumberUtil.isNumber(val.getInspect_result());
        if(!flag){
            //todo:非数值
            String strLog1=StrUtil.format("+++++ 结果值非数值,项目编号:{} ,项目名称:{} ,结果值:{}",val.getCode(), val.getName(), val.getInspect_result());
            System.out.println(strLog1);
            logger.error(strLog1);
            return;
        }

        Number number= Convert.toNumber(val.getInspect_result());
        BigDecimal ddd= new BigDecimal(number.doubleValue());
//        System.out.println("number  = " + number.doubleValue());
//        System.out.println("number  = " + ddd.doubleValue());

        List<IntervalValueBean> ivbList= IntervalUtils.getInterval(val.getReference());
        if(CollUtil.isEmpty(ivbList)){
            //单值：
            String strLog1=StrUtil.format("+++++ 无参考区间,项目编号:{} ,项目名称:{} ,区间:{}",val.getCode(), val.getName(), val.getReference());
            System.out.println(strLog1);
//            logger.error(strLog1);
            dataCleanLabProject.setReportVal(new BigDecimal(number.doubleValue()));
            return;
        }

        IntervalValueBean intervalValueBean = IntervalUtils.getIntervalBy(ivbList,val.getAbnormal(),val.getAbnormal_indicator(),number.doubleValue());

        if(intervalValueBean==null){
            //todo:
            String strLog1=StrUtil.format("+++++ 无参考区间,项目编号:{} ,项目名称:{} ,结果值:{}",val.getCode(), val.getName(), val.getInspect_result());
            System.out.println(strLog1);
            return;
        }

        System.out.println("++++++ 获取的区间:"+ JSON.toJSONString(intervalValueBean));


        SoftMaxValue2Bean calInterval= IntervalValueBean.change(intervalValueBean);
        System.out.println("++++++ 计算区间:"+ JSON.toJSONString(calInterval));

        //计算规一
        BigDecimal resultVal= SoftMaxUtils.cal(calInterval,new BigDecimal(number.doubleValue()));
        dataCleanLabProject.setReportVal(resultVal);

        //
        CalStatusRateMBO catStatusRate= SoftMaxUtils.catStatusRate(calInterval,new BigDecimal(number.doubleValue()));
        dataCleanLabProject.setReportValFlag(catStatusRate.getStatus());
        dataCleanLabProject.setReportValRate(catStatusRate.getRate());

        dataCleanLabProject.setInterval(JSON.toJSONString(calInterval));

    }


    public static void main(String[] args) {

//        String str="<30.0";
//        String str="7.63-42.6";
//        String str="<1.000E+03";
//        String str="阴性(-)";
//        String str="正常值<256";
//        String str="86-324";
//        String str="阴性(<1:100)"; //todo:有bug

//        String str="绝经前期非孕妇女: <5.3;绝经后妇女:<8.3;孕妇:随孕周变化";
//        String str="上午:72.6--322.80;下午:32.4--150.00;";
//        String str="卧位: 3.11--41.20/立位: 4.20--45.6";
//        String str="||31--37周: 3.62--22.40";
//        String str="滤泡期: 3.50--9.20 排卵期: 4.70--25.9;黄体期: 1.70--5.60 绝经期:13.9--103.1"; //todo:
//        String str="女滤泡期:0.05--0.81; 女排卵期:0.05--0.81;女黄体期:0.05--.81 ;女绝经期:0.05--0.81";
//        String str="1.9--9.2 | 6.1--49.1 | 1.3--10.8 | 15.4--53.3 |";
//        String str="卧位:25--60||立位: 50--120||";
//        String str="31--37周: 3.62--22.40";
//        String str="1.9--9.2 ";
//        String str="绝经前期非孕妇女: <5.3";
//
//        String str="女滤泡期:132--498 ; 女排卵期:132--498 ; 女黄体期:132--498  ; 女绝经期:90--392 ;";
//        String str="上午:72.6--322.80;下午:32.4--150.00;";
//        String str="< 1.0E+003";
//        String str="1.9--9.2 | 6.4-49.1 |";
//        String str="绝经前期非孕妇女: <5.3;绝经后妇女:<8.3;孕妇:随孕周变化";
//        String str="<0.65ug/L为阴性，》0.85为阳性";



        DataCleanLabProject dataCleanLabProject=new DataCleanLabProject();
        LabProjectReportItemBO val=new LabProjectReportItemBO();
        val.setInspect_result("1.037E+05");
        //val.setReference("0--10.00");
        val.setReference("<1.000E+03");
        resultValue(dataCleanLabProject,val);

        System.out.println(JSON.toJSONString(dataCleanLabProject));

    }




}
