package com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.biz.impl;

import cn.hutool.core.thread.ThreadUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.bean.dto.DwdDataClearnTaskStartDTO;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.bean.dto.DwdDataClearnTaskStopDTO;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.biz.DwdDataClearnBizService;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.biz.DwdDataClearnTaskBizService;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.run.DwdHisDataCleanTaskRun;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.XXEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class DwdDataClearnTaskBizServiceImpl implements DwdDataClearnTaskBizService {

    @Autowired
    private DwdDataClearnBizService dwdDataClearnBizService;

    @Autowired
    private RedisUtils redisUtils;

    @Override
    public void start(DwdDataClearnTaskStartDTO params) {
        MQCleanTableKeyEnum key= XXEnumUtil.getEnum(MQCleanTableKeyEnum.class,params.getTableKey());
        if(key==null){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"不支持的清洗KEY");
        }
        String cacheKey= this.getCacheKey(params.getTableKey());
        if(redisUtils.hasKey(cacheKey)){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"任务已执行，不能重复执行。如要执行,请先关闭任务");
        }

        redisUtils.set(cacheKey, YN.YES.getCode());
        DwdHisDataCleanTaskRun run=new DwdHisDataCleanTaskRun(key,cacheKey,redisUtils,dwdDataClearnBizService,params.getWhereItems());
        //ThreadUtil.execAsync(run,true);
        new Thread(run).start();
    }

    @Override
    public void start2(DwdDataClearnTaskStartDTO params) {

        MQCleanTableKeyEnum key= XXEnumUtil.getEnum(MQCleanTableKeyEnum.class,params.getTableKey());
        if(key==null){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"不支持的清洗KEY");
        }
        String cacheKey= this.getCacheKey(params.getTableKey());
        if(redisUtils.hasKey(cacheKey)){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"任务已执行，不能重复执行。如要执行,请先关闭任务");
        }
        redisUtils.set(cacheKey, YN.YES.getCode());
        DwdHisDataCleanTaskRun run=new DwdHisDataCleanTaskRun(key,cacheKey,redisUtils,dwdDataClearnBizService,params.getWhereItems());
        new Thread(run).start();
    }


    @Override
    public void stop(DwdDataClearnTaskStopDTO params) {
        MQCleanTableKeyEnum key= XXEnumUtil.getEnum(MQCleanTableKeyEnum.class,params.getTableKey());
        if(key==null){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"不支持的清洗KEY");
        }
        String cacheKey= this.getCacheKey(params.getTableKey());
        redisUtils.delete(cacheKey);
        ThreadUtil.sleep(3000);
    }

    @Override
    public boolean getStatus(DwdDataClearnTaskStopDTO params) {
        MQCleanTableKeyEnum key= XXEnumUtil.getEnum(MQCleanTableKeyEnum.class,params.getTableKey());
        if(key==null){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"不支持的清洗KEY");
        }
        String cacheKey= this.getCacheKey(params.getTableKey());
        return redisUtils.hasKey(cacheKey);
    }

    private String getCacheKey(String tableKey){
        return "clean:h:t:"+tableKey;
    }
}
