package com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.bean.MQPatientEventBean;
import com.xxh.cloud.common.mq.bean.MQRuleLabelBean;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.etl.client.api.bean.bos.DataCleanConfigBO;
import com.xxh.cloud.etl.common.framework.constant.DwdCleanDataStatus;
import com.xxh.cloud.etl.common.framework.constant.PatientEventBO;
import com.xxh.cloud.etl.framework.modules.biz.bean.dto.DataCleanConfigDetailSimpleSaveDTO;
import com.xxh.cloud.etl.framework.modules.biz.service.DataCleanConfigDetailBizService;
import com.xxh.cloud.etl.framework.modules.cache.service.DataCleanConfigCacheService;
import com.xxh.cloud.exeEtl.framework.modules.biz.service.DataCleanErrorLogBizService;
import com.xxh.cloud.exeEtl.framework.modules.biz.service.DataCleanLogBizService;
import com.xxh.cloud.exeEtl.framework.modules.clean.bean.dto.DataCleanProcessTwoContext;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.bean.vos.Dwd2DwdCleanDataResultVO;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.config.Dwd2DwdCleanConfig;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.config.Dwd2DwdCleanConfigItem;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.bean.dto.Dwd2DwdCleanDataDTO;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.config.Dwd2DwdConfigUtils;
import com.xxh.cloud.exeEtl.framework.modules.read.bean.bos.DwdQueryDataBO;
import com.xxh.cloud.exeEtl.framework.modules.read.bean.dto.DwdDataReadByListDTO;
import com.xxh.cloud.exeEtl.framework.modules.read.bean.dto.DwdDataReadByPageDTO;
import com.xxh.cloud.exeEtl.framework.modules.read.service.DwdCleanDataReadService;
import com.xxh.cloud.exeEtl.framework.modules.read.service.DwdCleanDataWriteService;
import com.xxh.cloud.framework.activemq.constant.ActiveMQWayEnum;
import com.xxh.cloud.framework.activemq.utils.ActiveMQSendUtils;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.framework.common.write.bean.DbTaskResult;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.framework.common.write.doris.DorisWriter;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Log4j2
@Service
public  class Dwd2DwdCleanBaseService {



    @Autowired
    private DataCleanConfigCacheService dataCleanConfigCacheService;

    @Autowired
    private DataCleanLogBizService dataCleanLogBizService;

    @Autowired
    private DataCleanConfigDetailBizService dataCleanConfigDetailBizService;

    @Autowired
    private DwdCleanDataReadService dwdCleanDataReadService;

    @Autowired
    private DataCleanErrorLogBizService dataCleanErrorLogBizService;

    @Autowired
    private DwdCleanDataWriteService dwdCleanDataWriteService;


    /**
     * 获取配置
     * */
    protected DataCleanConfigBO getColumnCleanConfig(Dwd2DwdCleanConfig config, Map<String, DataCleanConfigBO> dataCleanConfigMap,String col){
        List<DataCleanConfigBO> list=this.getColumnCleanConfigList(config,dataCleanConfigMap,col);
        if(CollUtil.isEmpty(list)){
            return null;
        }else {
            return list.get(0);
        }
    }

    /**
     *
     * */
    protected DataCleanConfigBO getColumnCleanConfig(Dwd2DwdCleanConfig config, Map<String, DataCleanConfigBO> dataCleanConfigMap,String col,String cleanCode){
        Map<String,DataCleanConfigBO> map=this.getColumnCleanConfigMap(config,dataCleanConfigMap,col);
        return map.get(cleanCode);
    }


    /***/
    protected Map<String,DataCleanConfigBO> getColumnCleanConfigMap(Dwd2DwdCleanConfig config, Map<String, DataCleanConfigBO> dataCleanConfigMap,String col){
        Map<String,DataCleanConfigBO> data=new HashMap<>();

        Dwd2DwdCleanConfigItem configItem = config.getConfigMap().get(col);
        if(configItem==null){
            return data;
        }

        List<String> cleanCodes= StrJoinSplitUtils.split2List(configItem.getCleanCode());
        if(CollUtil.isEmpty(cleanCodes)){
            return data;
        }


        for(String cleanCode:cleanCodes){
            DataCleanConfigBO colConfig= dataCleanConfigMap.get(cleanCode);
            if(colConfig!=null){
                data.put(cleanCode,colConfig);
            }
        }

        return data;
    }

    /***/
    protected List<DataCleanConfigBO> getColumnCleanConfigList(Dwd2DwdCleanConfig config, Map<String, DataCleanConfigBO> dataCleanConfigMap,String col){
        List<DataCleanConfigBO> data=new ArrayList<>();

        Dwd2DwdCleanConfigItem configItem = config.getConfigMap().get(col);
        if(configItem==null){
            return data;
        }

        List<String> cleanCodes= StrJoinSplitUtils.split2List(configItem.getCleanCode());
        if(CollUtil.isEmpty(cleanCodes)){
            return data;
        }

        for(String cleanCode:cleanCodes){
            DataCleanConfigBO colConfig= dataCleanConfigMap.get(cleanCode);
            if(colConfig!=null){
                data.add(colConfig);
            }
        }

        return data;
    }






    protected DwdQueryDataBO onRead(Dwd2DwdCleanDataDTO params){
        System.out.println("++++++ onRead +++++++");
        if(params.isPageDataFlag()){ //分页查询
            DwdDataReadByPageDTO readParam=new DwdDataReadByPageDTO();
            readParam.setTableCode(params.getTableCode());
            readParam.setWhereItems(params.getWhereItems());
            readParam.setOrderByCode(params.getOrderByCode());
            readParam.setStart(params.getStart());
            readParam.setNum(params.getNum());
            DwdQueryDataBO readData= dwdCleanDataReadService.readDataByPage(readParam);
            return readData;
        }else {
            DwdDataReadByListDTO readParam=new DwdDataReadByListDTO();
            readParam.setTableCode(params.getTableCode());
            readParam.setWhereItems(params.getWhereItems());
            DwdQueryDataBO readData= dwdCleanDataReadService.readDataByList(readParam);
            return readData;
        }
    }

    protected Dwd2DwdCleanConfig onConfig(String tableCode){
        System.out.println("++++++ init onConfig ++++++");
        Dwd2DwdCleanConfig config= Dwd2DwdConfigUtils.getConfigBean(tableCode);
        return config;
    }

    protected Map<String, DataCleanConfigBO> onDataCleanConfig(Dwd2DwdCleanConfig config){
        System.out.println("++++++ init onDataCleanConfig ++++++");
        Set<String> DataCleanConfigCodes= Dwd2DwdConfigUtils.getCleanCodeList(config);
        Map<String, DataCleanConfigBO> dataCleanConfigMap =  dataCleanConfigCacheService.getDataCleanConfigMapByCache(DataCleanConfigCodes);
        return dataCleanConfigMap;
    }


    protected boolean onDataOne(String tableCode, DataCleanProcessTwoContext context , Dwd2DwdCleanConfig config, Map<String, DataCleanConfigBO> dataCleanConfigMap, Map<String,Object> dataItem){
        System.out.println("++++++ init onDataOne ++++++");
        return false;
    }





    /**
     *
     * */
    public Dwd2DwdCleanDataResultVO cleanData(Dwd2DwdCleanDataDTO params){
        ValidatorUtils.validateEntity(params);

        DwdQueryDataBO readData=this.onRead(params);
        if(readData==null || readData.getConfig()==null || CollUtil.isEmpty(readData.getData()) ){
            log.debug("++++++++ 无读取数据,{}",params.getTableCode());
            return new Dwd2DwdCleanDataResultVO(DwdCleanDataStatus.NotData);
        }

        Dwd2DwdCleanConfig config=null;
        Map<String, DataCleanConfigBO> dataCleanConfigMap=null;

        if(params.isCleanConfigFlag()){
            config= this.onConfig(params.getTableCode());
            if(config==null){
                log.debug("++++++++ 数据表无配置清洗关系,{}",params.getTableCode());
                return new Dwd2DwdCleanDataResultVO(DwdCleanDataStatus.NotConfig);
            }
            dataCleanConfigMap=this.onDataCleanConfig(config);
            if(MapUtil.isEmpty(dataCleanConfigMap)){
                log.debug("++++++++ 无清洗配置,{}", params.getTableCode());
                return new Dwd2DwdCleanDataResultVO(DwdCleanDataStatus.NotConfig);
            }
        }else {
            config=new Dwd2DwdCleanConfig();
            dataCleanConfigMap=new HashMap<>();
        }

        DataCleanProcessTwoContext context=new DataCleanProcessTwoContext();
        context.setCalFlag(params.isCalculateFlag());
        context.setEventFlag(params.isEventFlag());
        context.setDataMqFlag(params.isDataMQFlag());

        List<Map<String,Object>> processDataList= readData.getData();
        List<Map<String,Object>> removeDataList=new ArrayList<>();

        for (int i = 0; i < processDataList.size(); i++) {
            Map<String,Object> dataItem =processDataList.get(i);
            boolean flag= this.onDataOne(params.getTableCode(),context,config,dataCleanConfigMap,dataItem);
            if(flag==false){
                removeDataList.add(dataItem);
            }
        }

        if(removeDataList.size()>0){
            processDataList.removeAll(removeDataList);
        }

        //todo:
        //System.out.println("++++++ 1:");
        //System.out.println(JSON.toJSONString(processDataList));


        //日志
        if(MapUtil.isNotEmpty(context.getLogMap())){
            System.out.println("++++++ 2 日志 :");
            //System.out.println(JSON.toJSONString(context.getLogMap()));
            dataCleanLogBizService.saveBatch(context.getLogMap());
        }


        //记录需要处理的规则
        if(CollUtil.isNotEmpty(context.getNeedAddList())){
            System.out.println("++++++ 3 记录需要处理的规则:");
            //System.out.println(JSON.toJSONString(context.getNeedAddList()));

            List<DataCleanConfigDetailSimpleSaveDTO> dccSaveDto=new ArrayList<>();
            context.getNeedAddList().forEach(item->{
                dccSaveDto.add(new DataCleanConfigDetailSimpleSaveDTO(item.getRuleCode(),item.getSourceCode(),item.getSourceName()));
            });
            dataCleanConfigDetailBizService.saveBatch(dccSaveDto);
        }


        if(CollUtil.isEmpty(processDataList)){
            //System.out.println("++++++++++++ 清洗失败无数据 ++++++++++++++++++++++++++");
            return new Dwd2DwdCleanDataResultVO(DwdCleanDataStatus.NotCleanData);
        }

        try {
            dwdCleanDataWriteService.batchWrite(readData.getConfig(),processDataList);
        }catch (Exception e){
            //todo:记录日志
            e.printStackTrace();
            log.error("+++++数据清洗保存错误: KEY: {} , 错误信息:{}" ,readData.getConfig().getKey(),e.getMessage());
        }

        //诊断
//        log.debug("<<<<<< 记录诊断 ");
//        this.processDiagnosisReport(params.getDiagnosisReportConfig(),context.getDiagnosisList());


        //todo: 2024-05-08 同步历史数据，屏蔽代码
        //todo: 2024-05-13 计算引擎MQ，开始
        //统计患者最新数据时间
        log.debug("<<<<<< 统计患者最新数据时间 ");
        this.processPatientDataTime(params.getLastDateConfig(),params.getPatientDayConfig(), context.getPatientDataTimeList());


        //记录错误日志
        if(CollUtil.isNotEmpty(context.getErrorLogs())){
            log.debug("<<<<<< 记录错误日志 ");
            dataCleanErrorLogBizService.addBatch(context.getErrorLogs());
        }

        //发送事件
        if(params.isEventFlag()){
            log.debug("<<<<<< 发送事件-MQ ");
            this.sendMQEvent(context.getEventList());
        }

        //发送计算
        if(params.isCalculateFlag()){
            log.debug("<<<<<< 发送规则计算-MQ ");
            this.sendMQCalPatient(context.getCalDataList());
        }

        //发送数据事件
        if(params.isDataMQFlag()){
            log.debug("<<<<<< 发送数据事件-MQ");
            this.sendMQData2Topic(context.getDataTopicMap());
            this.sendMQData2Queue(context.getDataQueueMap());
        }

        log.debug("<<<<< 院中事件-MQ");
        this.sendEventMQ2(context.getPatientEventMap());


        return new Dwd2DwdCleanDataResultVO(DwdCleanDataStatus.Success);
    }





//    /**
//     * Diagnosis
//     * */
//    private void processDiagnosisReport(HwdsWriteDBConfigBean diagnosisReportConfig, List<Map<String,Object>> diagnosisList){
//        if(CollUtil.isEmpty(diagnosisList)){
//            log.debug("<<<<< 无数据,诊断报告数据");
//            return;
//        }
//        if(diagnosisReportConfig==null){
//            return;
//        }
//        DorisWriter dorisWriter=new DorisWriter();
//        dorisWriter.writer(diagnosisReportConfig,diagnosisList);
//    }

    /**
     * 患者数据日期处理
     * */
    private void processPatientDataTime(HwdsWriteDBConfigBean lastDateConfig,HwdsWriteDBConfigBean patientDayConfig,List<Map<String,Object>> patientDataTimeList){
        if(CollUtil.isEmpty(patientDataTimeList)){
            log.debug("<<<<< 无数据,患者数据日期处理");
            return;
        }
        if(lastDateConfig==null || patientDayConfig==null){
            return;
        }

        System.out.println("<<<<< 患者数据日期处理");

        dwdCleanDataWriteService.batchWrite(lastDateConfig,patientDataTimeList);
        dwdCleanDataWriteService.batchWrite(patientDayConfig,patientDataTimeList);

//        DorisWriter dorisWriter=new DorisWriter();
//        dorisWriter.writer(lastDateConfig,patientDataTimeList,false,"");
//        DorisWriter dorisWriter2=new DorisWriter();
//        dorisWriter2.writer(patientDayConfig,patientDataTimeList,false,"");

    }


    /**
     * mq rule
     * */
    private  void sendMQCalPatient(List<MQRuleLabelBean> ruleDataList){
        if(CollUtil.isEmpty(ruleDataList)){
            log.info("<<<<< 不需要无患者ID");
            return;
        }
        ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName.Que_Rule_Data_Save, JSON.toJSONString(ruleDataList),10);
    }


    /**
     * mq event
     * */
    private void sendMQEvent(List<MQPatientEventBean> eventList){
        if(CollUtil.isEmpty(eventList)){
            ConsoleUtils.log("++++++ 无事件MQ数据发送 ... ");
            return;
        }

        ConsoleUtils.log("2. ************************** ");
        ConsoleUtils.log("+++++++发送事件MQ:"+ JSON.toJSONString(eventList));


        ConsoleUtils.log("++++++ 开始发送事件MQ ...... ");
        List<List<MQPatientEventBean>> list= ListUtil.split(eventList,1000);
        for(List<MQPatientEventBean> itemList:list){
            ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName.Que_Event_Data_Sava, JSON.toJSONString(itemList));
        }
    }


    /**
     * mq data
     * */
    private void sendMQData2Topic(Map<String,List<Object>> dataTopicMap){
        if(MapUtil.isEmpty(dataTopicMap)){
            ConsoleUtils.log("+++++ 无数据，数据MQ广播发送");
            return;
        }
        ConsoleUtils.log("+++++ 数据MQ广播发送...");
        for(String key:dataTopicMap.keySet()){
            List<Object> dataList = dataTopicMap.get(key);
            List<List<Object>> list= ListUtil.split(dataList,1000);
            for(List<Object> itemList:list){
                ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Topic, key, JSON.toJSONString(itemList));
            }
        }
    }


    /**
     * mq data
     * */
    private void sendMQData2Queue(Map<String,List<Object>> dataQueueMap){
        if(MapUtil.isEmpty(dataQueueMap)){
            ConsoleUtils.log("+++++ 无数据，数据MQ队列发送");
            return;
        }
        ConsoleUtils.log("+++++ 数据MQ队列发送...");

        ConsoleUtils.log("3. ************************** ");
        ConsoleUtils.log("+++++++发送数据MQ:"+ JSON.toJSONString(dataQueueMap));


        for(String key:dataQueueMap.keySet()){
            List<Object> dataList = dataQueueMap.get(key);
            List<List<Object>> list= ListUtil.split(dataList,1000);
            for(List<Object> itemList:list){
                ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, key, JSON.toJSONString(itemList));
            }
        }
    }



    private void sendEventMQ2(Map<String,List<PatientEventBO>> patientEventMap){
        if(MapUtil.isEmpty(patientEventMap)){
            return;
        }

        for (String key:patientEventMap.keySet()){
            List<PatientEventBO> dataList = patientEventMap.get(key);
            List<List<PatientEventBO>> list= ListUtil.split(dataList,1000);
            for(List<PatientEventBO> itemList:list){
                ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, key, JSON.toJSONString(itemList));
            }
        }


    }




}
