package com.xxh.cloud.exeEtl.framework.modules.task.schedule;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.bean.dto.Dwd2DwdCleanByPageDTO;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.biz.DwdDataClearnBizService;
import com.xxh.cloud.exeEtl.framework.modules.dwd2dwd.cleansql.*;
import com.xxh.cloud.exeEtl.framework.modules.task.service.DwdEmrTGJBCleanService;
import com.xxh.cloud.exeEtl.framework.modules.task.service.DwdEmrZZCleanService;
import com.xxh.cloud.exeEtl.framework.modules.task.service.DwdMedicalContentCleanService;
import com.xxh.cloud.framework.common.utils.ConsoleUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;

@Log4j2
@Component
public class HisDataCleanTask {

    @Autowired
    private DwdDiagnosisCleanService dwdDiagnosisCleanService;
    @Autowired
    private DwdExamReserveCleanService dwdExamReserveCleanService;
    @Autowired
    private DwdExamReportCleanService dwdExamReportCleanService;

    @Autowired
    private DwdDataClearnBizService dwdDataClearnBizService;

    @Autowired
    private DwdOutCleanService dwdOutCleanService;



    @Autowired
    private DwdPatientMainCleanService dwdPatientMainCleanService;

    @Autowired
    private DwdExaminationPatientMainCleanService dwdExaminationPatientMainCleanService;

    @Autowired
    private ActivityExamineReportCleanService activityExamineReportCleanService;

    @Autowired
    private DwdInpatientChangeCleanService dwdInpatientChangeCleanService;

    @Autowired
    private DoctorAdviceCleanService doctorAdviceCleanService;

    /**
     * 历史数据清洗任务
     * */
    @XxlJob("hisDataClean")
    public void hisDataClean(){
        //todo: 测试
    }

    private Map<String,Object> getXxlParamMap(String xxlParamsJson){
        if(StrUtil.isEmpty(xxlParamsJson)){
            XxlJobHelper.log("+++++ 调用任务参数不能为空");
            ConsoleUtils.log("+++++ 调用任务参数不能为空");
            return null;
        }
        Map<String,Object> map = JSON.parseObject(xxlParamsJson, Map.class);
        return map;
    }

    private Boolean getXxlParamBool(Map<String,Object> map,String key){
        Object _flag =  map.get(key);
        if(_flag==null){
            XxlJobHelper.log("+++++ 调用任务参数配置错误["+key+":Boolean]");
            ConsoleUtils.log("+++++ 调用任务参数配置错误["+key+":Boolean]");
            return null;
        }
        return  Convert.toBool(_flag);
    }

    private Boolean getXxlParamBool_CleanFlag(Map<String,Object> map){
        return this.getXxlParamBool(map,"cleanFlag");
    }

    private Boolean getXxlParamBool_RuleFlag(Map<String,Object> map){
        return this.getXxlParamBool(map,"ruleFlag");
    }

    @XxlJob("diagnosisClean")
    public void diagnosisClean(){
        ConsoleUtils.log("++++++ ConsoleUtils ++清洗诊断报告开始");
        System.out.println("++++++ println ++清洗诊断报告开始");
        log.debug("++++++ log debug ++清洗诊断报告开始");

        String xxlParamsJson = XxlJobHelper.getJobParam();
        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);
        Boolean cleanFlag = this.getXxlParamBool_CleanFlag(configMap);
        if(cleanFlag==null){
            return;
        }
        dwdDiagnosisCleanService.clean(cleanFlag);
    }


    @XxlJob("inpatientChangeClean")
    public void inpatientChangeClean(){
        ConsoleUtils.log("++++++ ConsoleUtils : 住院变更清洗开始");
        System.out.println("++++++ println : 住院变更清洗开始");
        log.debug("++++++ log debug : 住院变更清洗开始");
        String xxlParamsJson = XxlJobHelper.getJobParam();
        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);

        Boolean ruleFlag = this.getXxlParamBool_RuleFlag(configMap);
        if(ruleFlag==null){
            return;
        }

        dwdInpatientChangeCleanService.clean(ruleFlag);
    }

    //死亡状态清洗
    @XxlJob("doctorAdviceClean")
    public void doctorAdviceClean(){
        ConsoleUtils.log("++++++ ConsoleUtils : 医嘱清洗开始");
        System.out.println("++++++ println : 医嘱清洗开始");
        log.debug("++++++ log debug : 医嘱清洗开始");

        String xxlParamsJson = XxlJobHelper.getJobParam();
        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);
        doctorAdviceCleanService.clean();
    }

    @XxlJob("examReserveClean")
    public void examReserveClean(){
        ConsoleUtils.log("++++++清洗检查预约开始");
        String xxlParamsJson = XxlJobHelper.getJobParam();
        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);
        Boolean cleanFlag = this.getXxlParamBool_CleanFlag(configMap);
        if(cleanFlag==null){
            return;
        }
        dwdExamReserveCleanService.clean(cleanFlag);
    }

    @XxlJob("examReportClean")
    public void examReportClean(){
        ConsoleUtils.log("++++++清洗检查报告开始");
        String xxlParamsJson = XxlJobHelper.getJobParam();
        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);

        Boolean clearnFlag = this.getXxlParamBool_CleanFlag(configMap);
        if(clearnFlag==null){
            return;
        }

        Boolean ruleFlag = this.getXxlParamBool_RuleFlag(configMap);
        if(ruleFlag==null){
            return;
        }

        dwdExamReportCleanService.clean(clearnFlag,ruleFlag);
    }


    @XxlJob("examReportActivityClean")
    public void examReportActivityClean(){
        ConsoleUtils.log("++++++清洗活动检查报告开始");
        System.out.println("++++++++++++++++清洗活动检查报告开始++++++++++++++++++");
        String xxlParamsJson = XxlJobHelper.getJobParam();
        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);

        Boolean clearnFlag = this.getXxlParamBool_CleanFlag(configMap);
        if(clearnFlag==null){
            return;
        }

        Boolean ruleFlag = this.getXxlParamBool_RuleFlag(configMap);
        if(ruleFlag==null){
            return;
        }
        activityExamineReportCleanService.clean(clearnFlag,ruleFlag);
    }



    @XxlJob("outContentSyncClean")
    public void outContentSyncClean(){
        ConsoleUtils.log("++++++清洗同步出院小结开始");
        String xxlParamsJson = XxlJobHelper.getJobParam();
        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);
        Boolean cleanFlag = this.getXxlParamBool_CleanFlag(configMap);
        if(cleanFlag==null){
            return;
        }
        dwdOutCleanService.clean(cleanFlag);
    }

    @XxlJob("outContentClean")
    public void outContentClean(){
        ConsoleUtils.log("++++++清洗出院小结解密开始");
        Dwd2DwdCleanByPageDTO pageDTO=new Dwd2DwdCleanByPageDTO();
        pageDTO.setKey(MQCleanTableKeyEnum.OutContent);
        pageDTO.addWhereItem(" content_standard is null ");
        pageDTO.addWhereItem(" content_raw is not null ");
        pageDTO.setStart(0);
        pageDTO.setNum(100);
        dwdDataClearnBizService.cleanByPage(pageDTO);
    }

    @XxlJob("operationContentClean")
    public void operationContentClean(){
        ConsoleUtils.log("++++++清洗手术内容解密开始");
        Dwd2DwdCleanByPageDTO pageDTO=new Dwd2DwdCleanByPageDTO();
        pageDTO.setKey(MQCleanTableKeyEnum.OperationContent);
        pageDTO.addWhereItem("content_standard is null");
        pageDTO.setStart(0);
        pageDTO.setNum(100);
        dwdDataClearnBizService.cleanByPage(pageDTO);

    }

    @XxlJob("medicalRecordContentClean")
    public void medicalRecordContentClean(){
        ConsoleUtils.log("++++++清洗电子病例内容解密开始");
        Dwd2DwdCleanByPageDTO pageDTO=new Dwd2DwdCleanByPageDTO();
        pageDTO.setKey(MQCleanTableKeyEnum.MedicalRecordContent);
        pageDTO.addWhereItem("content_standard is null");
        pageDTO.setStart(0);
        pageDTO.setNum(100);
        dwdDataClearnBizService.cleanByPage(pageDTO);
    }



    @XxlJob("dwdPatientMainClean")
    public void dwdPatientMainClean(){
        ConsoleUtils.log("++++++清洗普通患者关联主客户信息开始");
//        String xxlParamsJson = XxlJobHelper.getJobParam();
//        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);
//
//        Boolean clearnFlag = this.getXxlParamBool_CleanFlag(configMap);
//        if(clearnFlag==null){
//            return;
//        }
        //todo:待测试
        dwdPatientMainCleanService.relationPatient();
    }

    @XxlJob("dwdExaminationPatientMainClean")
    public void dwdExaminationPatientMainClean(){
        ConsoleUtils.log("++++++清洗体检患者关联主客户信息开始");
//        String xxlParamsJson = XxlJobHelper.getJobParam();
//        Map<String,Object> configMap = this.getXxlParamMap(xxlParamsJson);
//
//        Boolean clearnFlag = this.getXxlParamBool_CleanFlag(configMap);
//        if(clearnFlag==null){
//            return;
//        }
        //todo:待测试
        dwdExaminationPatientMainCleanService.relationExaminationPatient();

    }


    @Autowired
    private DwdMedicalContentCleanService dwdMedicalContentCleanService;

    @XxlJob("odsMedicalContentClean")
    public void odsMedicalContentClean(){
        ConsoleUtils.log("++++++清洗-电子病例内容NLP");
        dwdMedicalContentCleanService.nlp();
//        dwdMedicalContentCleanService.test();
    }


    @Autowired
    private DwdEmrZZCleanService dwdEmrZZCleanService;

    @Autowired
    private DwdEmrTGJBCleanService dwdEmrTGJBCleanService;

    @XxlJob("dwdEmrZZ")
    public void dwdEmrZZ(){
        ConsoleUtils.log("++++++清洗-电子病例内容转DWD");
        dwdEmrZZCleanService.clean();
    }

    @XxlJob("dwdEmrTGJB")
    public void dwdEmrTGJB(){
        ConsoleUtils.log("++++++清洗-电子病例内容转体格检查-基本信息");
        dwdEmrTGJBCleanService.clean();
    }








}
