package com.xxh.cloud.exeEtl.framework.modules.task.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.xxh.cloud.exeEtl.framework.modules.read.mapper.Dwd2DwdQueryMapper;
import com.xxh.cloud.exeEtl.framework.modules.task.bean.DwdEmrTGJBBO;
import com.xxh.cloud.exeEtl.framework.modules.task.bean.DwdEmrZZBO;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.utils.StrSqlUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@DS("wd")
@Log4j2
@Service
public class DwdEmrTGJBCleanService {

    @Autowired
    private Dwd2DwdQueryMapper dwd2DwdQueryMapper;


    private List<Map<String,Object>> onRead(){
        String sql="SELECT patient_id ,id ,type ,child_type,content  FROM ods_patient_medical_record_structural_info where content is not null and nlp_status='Y' and sync_status='N'  and type='tg' and child_type='tg-tzjc' ORDER BY op_date desc LIMIT 50";
        List<Map<String,Object>> data=  dwd2DwdQueryMapper.readListData(sql);
        return data;
    }



    public void clean(){
        List<Map<String,Object>> list= this.onRead();
        if(CollUtil.isEmpty(list)){
            return;
        }

        List<String> ids=new ArrayList<>();
        List<String> errorIds=new ArrayList<>();
        List<DwdEmrTGJBBO> zzList=new ArrayList<>();

        for(Map<String,Object> map:list){
            String id= Convert.toStr(map.get("id"));
            String patientId=Convert.toStr(map.get("patient_id"));
            String type=Convert.toStr(map.get("type"));
            String childType=Convert.toStr(map.get("child_type"));
            String content= Convert.toStr(map.get("content"));
            boolean flag  = this.getEMRTG_JC(zzList,id,patientId,type,childType,content);
            if(flag){
                ids.add(StrUtil.format("{}_{}_{}_{}",patientId,id,type,childType));
            }else {
                errorIds.add(StrUtil.format("{}_{}_{}_{}",patientId,id,type,childType));
            }
        }

        //concat(patient_id,'_',id,'_',type,'_',child_type) in ('0000039470_01728272183665112304_tg_tg-bks','0000113706_01727654792427232652_tg_tg-fz')

        if(CollUtil.isNotEmpty(ids)){
            String updateSQL1= StrUtil.format("UPDATE ods_patient_medical_record_structural_info set sync_status='Y' where concat(patient_id,'_',id,'_',type,'_',child_type) in {}", StrSqlUtils.in(ids));
            dwd2DwdQueryMapper.update(updateSQL1);
        }

        if(CollUtil.isNotEmpty(errorIds)){
            String updateSQL1= StrUtil.format("UPDATE ods_patient_medical_record_structural_info set sync_status='ERR' where concat(patient_id,'_',id,'_',type,'_',child_type) in {}", StrSqlUtils.in(errorIds));
            dwd2DwdQueryMapper.update(updateSQL1);
        }

        String sql = this.getWriteSQL(zzList);
        dwd2DwdQueryMapper.insert(sql);
    }



    private static boolean getEMRTG_JC(List<DwdEmrTGJBBO> zzList,String emrId,String patientId,String type,String childType,String content){
        if(StrUtil.isEmpty(content)){
            return false;
        }
        try {
            JSONObject jsonObject=JSON.parseObject(content);
            String twjg = jsonObject.getString("体温结果");
            String xl= jsonObject.getString("心率");
            String xy= jsonObject.getString("血压");
            String tz= jsonObject.getString("体重");
            String sg= jsonObject.getString("身高");
            String bmi= jsonObject.getString("BMI");
            DwdEmrTGJBBO dwdEmrZZBO=new DwdEmrTGJBBO();
            dwdEmrZZBO.setId(IdUtil.getSnowflake(1,1).nextIdStr());
            dwdEmrZZBO.setEmrId(emrId);
            dwdEmrZZBO.setPatientId(patientId);
            dwdEmrZZBO.setHeight(sg);
            dwdEmrZZBO.setWeight(tz);
            dwdEmrZZBO.setBloodPressure(xy);
            dwdEmrZZBO.setHeartRate(xl);
            dwdEmrZZBO.setTemperature(twjg);
            dwdEmrZZBO.setBmi(bmi);
            zzList.add(dwdEmrZZBO);
            return true;
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }
    }



    public  String getWriteSQL(List<DwdEmrTGJBBO> list){
        if(CollUtil.isEmpty(list)){
            return null;
        }
        StringBuffer sql=new StringBuffer();
        sql.append("INSERT INTO dwd_patient_emr_body_check_basic_info (id,emr_id,patient_id,height,weight,blood_pressure,heart_rate,temperature,bmi,op_user,op_date) VALUES ");
        List<String> sqlList=new ArrayList<>();
        for(DwdEmrTGJBBO item:list){
            sqlList.add(getWriteOneSQL(item));
        }
        sql.append(StrJoinSplitUtils.join(sqlList));
        return sql.toString();
    }



    private  String getWriteOneSQL(DwdEmrTGJBBO data){
        String sql= StrUtil.format("('{}','{}','{}','{}','{}','{}','{}','{}','{}','{}','{}')",
                data.getId(),
                data.getEmrId(),
                data.getPatientId(),
                data.getHeight(),
                data.getWeight(),
                data.getBloodPressure(),
                data.getHeartRate(),
                data.getTemperature(),
                data.getBmi(),
                "sys",
                DateUtil.now()
        );
        return sql;
    }











}
