package com.xxh.cloud.exeEtl.framework.modules.task.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.xxh.cloud.exeEtl.framework.modules.read.mapper.Dwd2DwdQueryMapper;
import com.xxh.cloud.exeEtl.framework.modules.task.bean.DwdEmrZZBO;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.utils.StrSqlUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@DS("wd")
@Log4j2
@Service
public class DwdEmrZZCleanService {


    @Autowired
    private Dwd2DwdQueryMapper dwd2DwdQueryMapper;



    private List<Map<String,Object>> onRead(){
        String sql="SELECT patient_id ,id ,type ,child_type,content  FROM ods_patient_medical_record_structural_info where content is not null and nlp_status='Y' and sync_status='N'  and type='zz' ORDER BY op_date desc LIMIT 50";
        List<Map<String,Object>> data=  dwd2DwdQueryMapper.readListData(sql);
        return data;
    }

    public void clean(){
        List<Map<String,Object>> list= this.onRead();
        if(CollUtil.isEmpty(list)){
            return;
        }

        List<String> ids=new ArrayList<>();
        List<String> errorIds=new ArrayList<>();
        List<DwdEmrZZBO> zzList=new ArrayList<>();

        for(Map<String,Object> map:list){
            String id= Convert.toStr(map.get("id"));
            String patientId=Convert.toStr(map.get("patient_id"));
            String type=Convert.toStr(map.get("type"));
            String childType=Convert.toStr(map.get("child_type"));
            String content= Convert.toStr(map.get("content"));
            boolean flag =  this.getEMRZZ(zzList,id,patientId,type,childType,content);
            if(flag){
                ids.add(StrUtil.format("{}_{}_{}_{}",patientId,id,type,childType));
            }else {
                errorIds.add(StrUtil.format("{}_{}_{}_{}",patientId,id,type,childType));
            }
        }

        //concat(patient_id,'_',id,'_',type,'_',child_type) in ('0000039470_01728272183665112304_tg_tg-bks','0000113706_01727654792427232652_tg_tg-fz')

        if(CollUtil.isNotEmpty(ids)){
            String updateSQL1= StrUtil.format("UPDATE ods_patient_medical_record_structural_info set sync_status='Y' where concat(patient_id,'_',id,'_',type,'_',child_type) in {}", StrSqlUtils.in(ids));
            dwd2DwdQueryMapper.update(updateSQL1);
        }

        if(CollUtil.isNotEmpty(errorIds)){
            String updateSQL1= StrUtil.format("UPDATE ods_patient_medical_record_structural_info set sync_status='ERR' where concat(patient_id,'_',id,'_',type,'_',child_type) in {}", StrSqlUtils.in(errorIds));
            dwd2DwdQueryMapper.update(updateSQL1);
        }

        String sql = this.getWriteSQL(zzList);
        dwd2DwdQueryMapper.insert(sql);

    }



    public  String getWriteSQL(List<DwdEmrZZBO> list){
        if(CollUtil.isEmpty(list)){
            return null;
        }
        StringBuffer sql=new StringBuffer();
        sql.append("INSERT INTO dwd_patient_emr_symptom_info (id,emr_id,patient_id,type,name,body_part,degree,duration_time,one_time,op_user,op_date) VALUES ");
        List<String> sqlList=new ArrayList<>();
        for(DwdEmrZZBO item:list){
            sqlList.add(getWriteOneSQL(item));
        }
        sql.append(StrJoinSplitUtils.join(sqlList));
        return sql.toString();
    }

    private  String getWriteOneSQL(DwdEmrZZBO data){

        String sql= StrUtil.format("('{}','{}','{}','{}','{}','{}','{}','{}','{}','{}','{}')",
                data.getId(),
                data.getEmrId(),
                data.getPatientId(),
                data.getType(),
                data.getName(),
                data.getBodyPart(),
                data.getDegree(),
                data.getDurationTime(),
                data.getOneTime(),
                "sys",
                DateUtil.now()
        );
        return sql;
    }




    private static boolean getEMRZZ(List<DwdEmrZZBO> zzList,String emrId,String patientId,String type,String childType,String content){
        if(StrUtil.isEmpty(content)){
            return false;
        }

        try {
            JSONArray jsonArray= JSON.parseArray(content);
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject= jsonArray.getJSONObject(i);
                String name = jsonObject.getString("症状名称");
                String part= jsonObject.getString("发生部位");
                String degree= jsonObject.getString("严重程度");
                String cxsj= jsonObject.getString("持续时间");
                String dcsj= jsonObject.getString("单次时长");

                DwdEmrZZBO dwdEmrZZBO=new DwdEmrZZBO();
                dwdEmrZZBO.setId(IdUtil.getSnowflake(1,1).nextIdStr());
                dwdEmrZZBO.setEmrId(emrId);
                dwdEmrZZBO.setPatientId(patientId);
                dwdEmrZZBO.setType(childType);
                dwdEmrZZBO.setName(name);
                dwdEmrZZBO.setBodyPart(part);
                dwdEmrZZBO.setDegree(degree);
                dwdEmrZZBO.setDurationTime(cxsj);
                dwdEmrZZBO.setOneTime(dcsj);
                zzList.add(dwdEmrZZBO);
            }
            return true;
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }

    }




}
