package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.bean.pe.SlifeTjSuggest;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.ExamineReportsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import com.xxh.cloud.sync.framework.modules.read.api.service.PhysicalExaminationSyncService;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class SyncActivityExamReportService {



    public List<ExamineReportsHBO> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                            DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                            boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<ExamineReportsHBO> readData = BwwbSyncDataService.getActivityCheckReports(readConfig,startDate,endDate,logFlag,logBatchNo);

        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }
        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }



    private List<Map<String,Object>> getDataList(HdsContext context,List<ExamineReportsHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_activity_examine_report_info".equals(config.getKey())){
            for(ExamineReportsHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);

                //todo:清洗
//                context.addCleanObjId(MQCleanTableKeyEnum.ActivityExamineReport.getCode(),readData.getPatientId());
            }
        }

        return data;
    }


    private  Map<String,Object> getWriteMap_Ods(ExamineReportsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }


    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic) {

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","id"); //ID
        //map.put("apply_id","recipeId"); //申请流水号
        map.put("patient_id","pid"); //
        map.put("visit_id","ipid"); //患者就诊卡号
        map.put("patient_name","patientName"); // 患者姓名
        map.put("patient_id_no","idCard");//身份证号
        map.put("patient_gender","sex");//性别
        map.put("patient_age","age");//年龄
        map.put("patient_phone","patientPhone");// 患者电话

        map.put("department_name","deptName");//申请科室名称
        map.put("department_code","deptCode");//申请科室代码
        map.put("apply_doctor_code","docCode");//申请医生代码
        map.put("apply_doctor_name","docName");//申请医生姓名

        map.put("in_patient_No","ipid"); //住院流水号

        map.put("scene_type","patientType"); //  患者类型【场景分类】

        map.put("device_type_code","classId");//设备类型id
        map.put("device_type_name","className");//设备类型名称

        map.put("report_description","reportResult");//报告结果
        map.put("report_result","reportDiagnose");//报告诊断

        map.put("reports_time","reportDate");//报告日期
        map.put("report_name","serviceName");//报告名称

        map.put("data_update_date","updateTime");

        map.put("report_path","filePath");
        map.put("report_no","checkSerialnum");

        map.put("is_sync",SyncDataKeyEnum.SDKCV_SyncFlag.getCode());
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("ods_activity_examine_report_info",dbUrl,dbUserName,dbUserPassword,"ods_activity_examine_report_info",map));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }


}
