package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.DiagnoseBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.RecipeMedicineHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.bo1.MedicineOrderHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 医嘱用药数据
 * */
@Service
public class SyncDataAdviceMedicationService {


    public List<RecipeMedicineHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                               DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                               boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<RecipeMedicineHBO> readData = BwwbSyncDataService.getRecipeMedicine(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","opid"); //就诊记录号
        map1.put("visit_id","visitId"); //就诊记录号
        map1.put("in_patient_no","inPatientNo"); //住院号
        map1.put("patient_id","patientid"); //患者编号
        map1.put("patient_name","patientName"); //患者姓名
        map1.put("scene_type","patientType"); //
        map1.put("diagnosis","__diagnoseList"); //诊断
        map1.put("data_update_date","updateTime"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("ods_recipe_medicine_info",dbUrl,dbUserName,dbUserPassword,"ods_recipe_medicine_info",map1));

        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("id","orderSn");
        map2.put("recipe_medicine_id","__MainId"); //主ID
        map2.put("visit_id","visitId"); //就诊记录号
        map2.put("reg_in_id","visitId"); //住院 挂号
        map2.put("recipe_id","orderSn"); //医嘱处方号
        map2.put("department_code","departmentCode"); //科室代码
        map2.put("department_name","departmentName"); //科室名称
        map2.put("pay_time","payTime"); //时间
        map2.put("recipe_time","recipeTime"); //提交时间
        map2.put("fee_status","feeStatus"); //收费状态:CC:已缴费；RF；已退费；RFG:退费
        map2.put("name","drugName"); //药品名称
        map2.put("code","drugCode"); //药品代码
        map2.put("medicine_spec","medicineSpec"); //规格
        map2.put("dose_unit","doseUnit"); //单次用药规格
        map2.put("each_quantity","eachQuantity"); //单次用药数量
        map2.put("total","total"); //总量
        map2.put("total_unit","totalUnit"); //总量单位
        map2.put("usage_code","usageCode"); //用药方式编码
        map2.put("usage_content","usageContent"); //用药方式名称
        map2.put("frequency_code","frequencyCode"); //用药频次代码
        map2.put("frequency_name","frequencyName"); //用药频次
        map2.put("data_update_date","updateTime"); //
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map2.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_recipe_medicine_item_info",dbUrl,dbUserName,dbUserPassword,"ods_recipe_medicine_item_info",map2));


        //dwd
        Map<String,String> map3=new LinkedHashMap<>();
        map3.put("patient_id","patientid"); //患者编号
        map3.put("id","opid"); //就诊记录号
        map3.put("visit_id","visitId"); //就诊记录号
        map3.put("in_patient_no","inPatientNo"); //住院号
        map3.put("patient_name","patientName"); //患者姓名
        map3.put("scene_type_raw","patientType"); //
        map3.put("department_name_raw","__deptName");
        map3.put("department_name_standard","__deptName");
        map3.put("department_code_raw","__deptCode");
        map3.put("department_code_standard","__deptCode");

        map3.put("pay_time","__pay_time");
        map3.put("recipe_time","__recipe_time");

        map3.put("diagnosis_raw","__diagnoseList"); //诊断
        map3.put("data_update_date","updateTime"); //
        map3.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map3.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_recipe_medicine_info",dbUrl,dbUserName,dbUserPassword,"dwd_recipe_medicine_info",map3));

        Map<String,String> map4=new LinkedHashMap<>();
        map4.put("patient_id","__patientid"); //
        map4.put("recipe_medicine_id","__MainId"); //主ID
        map4.put("id","orderSn");
        map4.put("visit_id","visitId"); //就诊记录号
        map4.put("in_patient_no","visitId"); //住院 挂号
        map4.put("recipe_id","parentOrderSn"); //医嘱处方号

        map4.put("department_name_raw","departmentName"); //科室名称
        map4.put("department_name_standard","departmentName"); //科室名称
        map4.put("department_code_raw","departmentCode"); //科室代码
        map4.put("department_code_standard","departmentCode"); //科室代码

        map4.put("pay_time","payTime"); //时间
        map4.put("recipe_time","recipeTime"); //提交时间

        //map2.put("fee_status","feeStatus"); //收费状态:CC:已缴费；RF；已退费；RFG:退费

        map4.put("name_raw","drugName"); //药品名称
        map4.put("name_standard","drugName"); //药品名称

        map4.put("code_raw","drugCode"); //药品代码
        map4.put("code_standard","drugCode"); //药品代码
        map4.put("medicine_spec","medicineSpec"); //规格

        //
        map4.put("dose_unit_raw","doseUnit"); //单次用药规格
        map4.put("dose_unit_standard","doseUnit"); //单次用药单位
        map4.put("each_quantity","eachQuantity"); //单次用药数量

        //
        map4.put("frequency_code_raw","frequencyCode"); //用药频次代码
        map4.put("frequency_code_standard","frequencyCode"); //用药频次代码
        map4.put("frequency_name_raw","frequencyName"); //用药频次
        map4.put("frequency_name_raw","frequencyName"); //用药频次

        //
        map4.put("medication_amount","total"); //总量
        map4.put("medication_amount_unit","totalUnit"); //总量单位
        //  //


        //用药方式
        map4.put("usage_code_raw","usageCode"); //用药方式编码
        map4.put("usage_code_standard","usageCode"); //用药方式编码
        map4.put("usage_content_raw","usageContent"); //用药方式名称
        map4.put("usage_content_standard","usageContent"); //用药方式名称



        map4.put("status_raw","feeStatus"); //
        map4.put("data_update_date","updateTime"); //
        map4.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map4.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_recipe_medicine_item_info",dbUrl,dbUserName,dbUserPassword,"dwd_recipe_medicine_item_info",map4));



        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);

        return config;
    }


    private List<Map<String,Object>> getDataList(HdsContext context,List<RecipeMedicineHBO> dataList , HwdsWriteDBConfigBean config){

        List<Map<String,Object>> data=new ArrayList<>();

        if("ods_recipe_medicine_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeMedicineHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getMainWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("ods_recipe_medicine_item_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeMedicineHBO dataItem:dataList){
                if(CollUtil.isNotEmpty(dataItem.getOrderList())){
                    dataItem.getOrderList().forEach(item->{
                        Map<String,Object> writeItemMap=this.getItemWriteMap_Ods(dataItem.getOpid(), item,mapping);
                        data.add(writeItemMap);
                    });
                }
            }
        }else if("dwd_recipe_medicine_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeMedicineHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getMainWriteMap_dwd(dataItem,mapping);
                if(writeItemMap!=null){
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.Medicine.getCode(),dataItem.getOpid());
                    }
                    data.add(writeItemMap);
                }
                data.add(writeItemMap);
            }

        }else if("dwd_recipe_medicine_item_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeMedicineHBO dataItem:dataList){
                if(CollUtil.isNotEmpty(dataItem.getOrderList())){
                    dataItem.getOrderList().forEach(item->{
                        Map<String,Object> writeItemMap=this.getItemWriteMap_Dwd(dataItem.getPatientid(), dataItem.getOpid(), item,mapping);
                        data.add(writeItemMap);
                    });
                }
            }
        }


        return data;
    }

    private  Map<String,Object> getMainWriteMap_Ods(RecipeMedicineHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });


        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis", JSON.toJSONString(diagnoseList));
        }

        return writeMap;
    }


    private  Map<String,Object> getMainWriteMap_dwd(RecipeMedicineHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });


        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis_raw", JSON.toJSONString(diagnoseList));
        }

        String deptCode=null;
        String deptName=null;
        Date recipeTime=null;
        if(CollUtil.isNotEmpty(dataItem.getOrderList())){
            for(MedicineOrderHBO orderItem : dataItem.getOrderList()){
                if(orderItem.getRecipeTime()!=null){
                    recipeTime=orderItem.getRecipeTime();
                }
                if(StrUtil.isNotEmpty(orderItem.getDepartmentCode()) && StrUtil.isNotEmpty(orderItem.getDepartmentName())){
                    deptCode=orderItem.getDepartmentCode();
                    deptName=orderItem.getDepartmentName();
                    break;
                }
            }
        }


        writeMap.put("pay_time",recipeTime);
        writeMap.put("recipe_time",recipeTime);

        if(StrUtil.isNotEmpty(deptCode)){
            writeMap.put("department_code_raw",deptCode);
            writeMap.put("department_code_standard",deptCode);
        }

        if(StrUtil.isNotEmpty(deptName)){
            writeMap.put("department_name_raw",deptName);
            writeMap.put("department_name_standard",deptName);
        }
        return writeMap;
    }



    private  Map<String,Object> getItemWriteMap_Ods(String id, MedicineOrderHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });
        writeMap.put("recipe_medicine_id",id);
        return writeMap;
    }


    private  Map<String,Object> getItemWriteMap_Dwd(String patientid, String id, MedicineOrderHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();

        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });

        writeMap.put("patient_id",patientid);
        writeMap.put("recipe_medicine_id",id);
        return writeMap;
    }
    //

}
