package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.DiagnoseBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.RecipeTherapyHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.bo1.TherapyOrderHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 医嘱治疗数据
 * */
@Service
public class SyncDataAdviceTherapyService {


    public List<RecipeTherapyHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                              DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                              boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<RecipeTherapyHBO> readData = BwwbSyncDataService.getRecipeTherapy(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }



    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){


        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","opid"); //就诊记录号
        map1.put("visit_id","visitId"); //就诊记录号
        map1.put("scene_type","patientType"); //
        map1.put("in_patient_no","inPatientNo"); //
        map1.put("patient_name","patientName"); //
        map1.put("patient_id","patientId"); //
        map1.put("diagnosis","__diagnoseList"); //
        map1.put("data_update_date","updateTime"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_recipe_therapy_info",dbUrl,dbUserName,dbUserPassword,"ods_recipe_therapy_info",map1));

        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("id","orderSn");
        map2.put("therapy_id","opid");
        map2.put("reg_in_id","visitId"); //就诊记录号
        map2.put("visit_id","opid"); //就诊记录号
        map2.put("recipe_id","orderSn"); //医嘱处方号
        map2.put("in_patient_no","inPatientNo"); //住院流水号
        map2.put("patient_name","patientName"); //患者姓名
        map2.put("department_code","departmentCode"); //申请科室代码
        map2.put("department_name","departmentName"); //申请科室名称
        map2.put("pay_time","payTime"); //时间

        map2.put("recipe_time","recipeTime"); //处方时间
        map2.put("doctor_code","doctorCode"); //申请医生编号
        map2.put("doctor_name","doctorName"); //申请医生姓名
        map2.put("code","itemCode"); //项目代码
        map2.put("name","itemName"); //项目名称
        map2.put("status","orderStatus"); //申请状态
        map2.put("parent_recipe_id","parentOrderSn"); //父处方号:可通过此字段关联分组
        map2.put("data_update_date","updateTime"); //
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_recipe_therapy_item_info",dbUrl,dbUserName,dbUserPassword,"ods_recipe_therapy_item_info",map2));



        //dwd
        Map<String,String> map3=new LinkedHashMap<>();
        map3.put("patient_id","patientId"); //患者编号
        map3.put("id","opid"); //就诊记录号
        map3.put("visit_id","inPatientNo"); //就诊记录号
        map3.put("patient_name","patientName"); //
        map3.put("in_patient_no","inPatientNo"); //
        map3.put("department_name_raw","__deptName");
        map3.put("department_name_standard","__deptName");
        map3.put("department_code_raw","__deptCode");
        map3.put("department_code_standard","__deptCode");
        map3.put("doctor_code","__doctorCode");
        map3.put("doctor_name","__doctorName");
        map3.put("pay_time","__pay_time");
        map3.put("recipe_time","__recipe_time");
        map3.put("scene_type_raw","patientType"); //
        map3.put("diagnosis_raw","__diagnoseList"); //
        map3.put("data_update_date","updateTime"); //
        map3.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map3.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_recipe_therapy_info",dbUrl,dbUserName,dbUserPassword,"dwd_recipe_therapy_info",map3));

        Map<String,String> map4=new LinkedHashMap<>();
        map4.put("patient_id","__patientId");
        map4.put("id","orderSn");
        map4.put("therapy_id","opid");
        map4.put("visit_id","opid"); //就诊记录号
        map4.put("recipe_id","orderSn"); //医嘱处方号
        map4.put("in_patient_no","inPatientNo"); //住院流水号
        map4.put("patient_name","patientName"); //患者姓名
        map4.put("department_code_raw","departmentCode"); //申请科室代码
        map4.put("department_code_standard","departmentCode"); //申请科室代码
        map4.put("department_name_raw","departmentName"); //申请科室名称
        map4.put("department_name_standard","departmentName"); //申请科室名称

        map4.put("doctor_code","doctorCode"); //申请医生编号
        map4.put("doctor_name","doctorName"); //申请医生姓名

        map4.put("pay_time","payTime"); //时间
        map4.put("recipe_time","recipeTime"); //处方时间


        map4.put("code_raw","itemCode"); //项目代码
        map4.put("code_standard","itemCode"); //项目代码

        map4.put("name_raw","itemName"); //项目名称
        map4.put("name_standard","itemName"); //项目名称

        map4.put("status_raw","orderStatus"); //申请状态
        map4.put("parent_recipe_id","parentOrderSn"); //父处方号:可通过此字段关联分组

        map4.put("data_update_date","updateTime"); //
        map4.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map4.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_recipe_therapy_item_info",dbUrl,dbUserName,dbUserPassword,"dwd_recipe_therapy_item_info",map4));


        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }




    private List<Map<String,Object>> getDataList(HdsContext context,List<RecipeTherapyHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();

        if("ods_recipe_therapy_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeTherapyHBO dataItem:dataList){
                Map<String,Object> writeItemMap=this.getMainWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("ods_recipe_therapy_item_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeTherapyHBO dataItem:dataList){
                if(CollUtil.isNotEmpty(dataItem.getOrderList())){
                    dataItem.getOrderList().forEach(item->{
                        Map<String,Object>  writeItemMap=this.getItemWriteMap_ods(item,mapping);
                        data.add(writeItemMap);
                    });
                }
            }
        }else if("dwd_recipe_therapy_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeTherapyHBO dataItem:dataList){
                Map<String,Object> writeItemMap=this.getMainWriteMap_dwd(dataItem,mapping);
                if(writeItemMap!=null){
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.Therapy.getCode(),dataItem.getOpid());
                    }
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_recipe_therapy_item_info".equals(config.getKey())){
            Map<String,String> mapping= config.columnEntityMapping();
            for(RecipeTherapyHBO dataItem:dataList){
                if(CollUtil.isNotEmpty(dataItem.getOrderList())){
                    dataItem.getOrderList().forEach(item->{
                        Map<String,Object>  writeItemMap=this.getItemWriteMap_dwd(dataItem.getPatientId(), item,mapping);
                        data.add(writeItemMap);
                    });
                }
            }
        }

        return data;
    }

    private  Map<String,Object> getMainWriteMap_Ods(RecipeTherapyHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis", JSON.toJSONString(diagnoseList));
        }

        return writeMap;
    }


    private  Map<String,Object> getMainWriteMap_dwd(RecipeTherapyHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });


        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis_raw", JSON.toJSONString(diagnoseList));
        }


        if(CollUtil.isNotEmpty(dataItem.getOrderList())){
            TherapyOrderHBO therapyOrderHBO = dataItem.getOrderList().get(0);
            if(therapyOrderHBO!=null){
                writeMap.put("pay_time",therapyOrderHBO.getRecipeTime());
                writeMap.put("recipe_time",therapyOrderHBO.getRecipeTime());
            }
        }


        String deptCode=null;
        String deptName=null;
        String doctorCode=null;
        String doctorName=null;
        if(CollUtil.isNotEmpty(dataItem.getOrderList())){
            for(TherapyOrderHBO orderHBO: dataItem.getOrderList()){
                if(StrUtil.isNotEmpty(orderHBO.getDepartmentCode()) && StrUtil.isNotEmpty(orderHBO.getDepartmentName())){
                    deptCode=orderHBO.getDepartmentCode();
                    deptName=orderHBO.getDepartmentName();
                }
                if(StrUtil.isNotEmpty(orderHBO.getDoctorCode()) && StrUtil.isNotEmpty(orderHBO.getDoctorName())){
                    doctorCode=orderHBO.getDoctorCode();
                    doctorName=orderHBO.getDoctorName();
                }
            }
        }
        if(StrUtil.isNotEmpty(deptCode)){
            writeMap.put("department_code_raw",deptCode);
            writeMap.put("department_code_standard",deptCode);
        }
        if(StrUtil.isNotEmpty(deptName)){
            writeMap.put("department_name_raw",deptName);
            writeMap.put("department_name_standard",deptName);
        }
        if(StrUtil.isNotEmpty(doctorCode)){
            writeMap.put("doctor_code",doctorCode);
        }
        if(StrUtil.isNotEmpty(doctorName)){
            writeMap.put("doctor_name",doctorName);
        }

        return writeMap;
    }



    private  Map<String,Object> getItemWriteMap_ods(TherapyOrderHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }

    private  Map<String,Object> getItemWriteMap_dwd(String patientId, TherapyOrderHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        writeObjMap.put("patient_id",patientId);
        return writeObjMap;
    }


}
