package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.*;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class SyncDataDiagnosisService {




    public List<DiagnosisHBO> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                          DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                          boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<DiagnosisHBO> readData = BwwbSyncDataService.getDiagnosis(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }



    private List<Map<String,Object>> getDataList(List<DiagnosisHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();
        for(DiagnosisHBO dataItem:dataList){
            Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
            data.add(writeItemMap);
        }
        return data;
    }


    private Map<String,Object> getWriteMapOds(DiagnosisHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","diagSn"); //就诊记录号
        map1.put("patient_id","patientId"); //
        map1.put("visit_id","visitId"); //
        map1.put("in_patient_no","inPatientNo"); //
        map1.put("patient_name","patientName"); //
        map1.put("department_code","departmentCode"); //
        map1.put("department_name","departmentName"); //
        map1.put("doctor_code","doctorCode"); //
        map1.put("doctor_name","doctorName"); //
        map1.put("diagnosis_time","diagnosisTime"); //
        map1.put("type","type"); //
        map1.put("code","icdCode"); //

        map1.put("diag_code","diagCode"); //
        map1.put("diag_name","diagName"); //
        map1.put("diag_type_code","diagTypeCode"); //

        map1.put("is_main","mainFlag"); //
        map1.put("is_define","defineFlag"); //
        map1.put("sort_idx","sortIndex"); //
        map1.put("data_update_date","updateTime"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map1.put("is_sync",SyncDataKeyEnum.SDKCV_SyncFlag.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_diagnosis_report_info",dbUrl,dbUserName,dbUserPassword,"ods_diagnosis_report_info",map1));
        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }







}
