package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncConfigTypeEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.RecipeItemsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 医嘱数据
 * */
@Service
public class SyncDataDoctorAdviceService {




    public List<RecipeItemsHBO> getDataList( HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                             DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                             boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<RecipeItemsHBO> readData = BwwbSyncDataService.getRecipeItems(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }


        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }
        return readData.getData();
    }



    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("order_sn","orderSn"); //
        map1.put("dept_code","departmentCode"); //
        map1.put("dept_name","departmentName"); //
        map1.put("status_code","status"); //
        map1.put("status_name","statusName"); //
        map1.put("fee_status","feeStatus"); //
        map1.put("patient_type","patientType"); //
        map1.put("parent_order_sn","parentOrderSn"); //
        map1.put("order_type","orderType"); //
        map1.put("code","code"); //
        map1.put("name","name"); //
        map1.put("opid","opid"); //
        map1.put("data_update_date","updateTimeStr"); //
        map1.put("is_sync",SyncDataKeyEnum.SDKCV_SyncFlag.getCode()); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_doctor_advice_info",dbUrl,dbUserName,dbUserPassword,"ods_doctor_advice_info",map1));
        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }



    private List<Map<String,Object>> getDataList(List<RecipeItemsHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();
        for(RecipeItemsHBO dataItem:dataList){
            Map<String,Object>  writeItemMap=this.getWriteMap(dataItem,mapping);
            data.add(writeItemMap);
        }
        return data;
    }

    private  Map<String,Object> getWriteMap(RecipeItemsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);

        Map<String,Object> writeObjMap=new HashMap<>();

        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }



}
