package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.DiagnoseBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.ods.ExamProjectBO;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.CheckApplyHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.bo1.ApplyCheckOrderHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 检查申请
 * */
@Service
public class SyncDataExamApplyService {


    /**
     *
     * */
    public List<CheckApplyHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                           DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                           boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<CheckApplyHBO> readData = BwwbSyncDataService.getCheckApply(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context, readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }


        return readData.getData();
    }




    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        //ods
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","applyId"); //申请单号
        map1.put("visit_id","visitId"); //就诊记录号
        map1.put("patient_id","patientId"); //患者编号
        map1.put("in_patient_No","inPatientNo"); // 住院号

        map1.put("status","status"); //状态
        map1.put("status_name","statusName"); //状态名称

        map1.put("patient_type","patientType"); //1.门诊；2，急诊；4.住院
        map1.put("patient_name","patientName"); //患者名称
        map1.put("department_name","departmentName"); // 申请科室名称
        map1.put("department_code","departmentCode"); // 申请科室编码
        map1.put("apply_date","applyDate"); // 申请时间
        map1.put("scene_type","type"); // 1.门诊 2.急诊 4.住院
        map1.put("doctor_code","doctorCode"); //申请医生编号
        map1.put("doctor_name","doctorName"); // 申请医生姓名

        map1.put("items","__orderList");// 医嘱列表[检查项目] 特殊
        map1.put("diagnosis","__diagnoseList"); //诊断列表[] 特殊

        map1.put("data_update_date","updateTime");
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_examine_apply_info",dbUrl,dbUserName,dbUserPassword,"ods_examine_apply_info",map1));

        //dwd
        Map<String, String> map2 = new LinkedHashMap<>();
        map2.put("patient_id", "patientId");
        map2.put("id", "applyId");
        map2.put("visit_id", "visitId");
        map2.put("in_patient_no", "inPatientNo");

        map2.put("patient_name", "patientName");

        map2.put("department_name_raw", "departmentName");
        map2.put("department_name_standard", "departmentName");
        map2.put("department_code_raw", "departmentCode");
        map2.put("department_code_standard", "departmentCode");
        map2.put("doctor_code", "doctorCode");
        map2.put("doctor_name", "doctorName");

        map2.put("apply_date", "applyDate");
        map2.put("scene_type_raw", "patientType");//场景分类

        map2.put("diagnosis_raw", "__diagnoseList");//医嘱诊断

        map2.put("status_raw", "status");
        map2.put("status_name_raw", "statusName");
        map2.put("data_update_date", "updateTime");

        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_examine_apply_info",dbUrl,dbUserName,dbUserPassword,"dwd_examine_apply_info",map2));


        Map<String, String> map3 = new LinkedHashMap<>();
        map3.put("patient_id", "__patientId");
        map3.put("apply_id", "__apply_id");
        map3.put("id", "__id");
        map3.put("type_raw", "__type_raw");
        map3.put("type_standard", "__type_standard");
        map3.put("code_raw", "__code_raw");
        map3.put("code_standard", "__code_standard");
        map3.put("name_raw", "__name_raw");
        map3.put("name_standard","__name_standard");
        map3.put("part_code_raw","__part_code_raw");
        map3.put("part_code_standard","__part_code_standard");
        map3.put("part_name_raw","__part_name_raw");
        map3.put("part_name_standard","__part_name_standard");
        map3.put("data_update_date", "updateTime");
        map3.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map3.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_examine_apply_item_info",dbUrl,dbUserName,dbUserPassword,"dwd_examine_apply_item_info",map3));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }


    private List<Map<String,Object>> getDataList(HdsContext context,List<CheckApplyHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_examine_apply_info".equals(config.getKey())){
            for(CheckApplyHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_examine_apply_info".equals(config.getKey())){
            for(CheckApplyHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapDwd(dataItem,mapping);
                if(writeItemMap!=null){
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.ExamineApply.getCode(),dataItem.getApplyId());
                    }
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_examine_apply_item_info".equals(config.getKey())){
            for(CheckApplyHBO dataItem:dataList){
                if(CollUtil.isNotEmpty(dataItem.getOrderList())){
                    dataItem.getOrderList().forEach(item->{
                        Map<String,Object>  writeItemMap= getWriteMapItemDwd(dataItem.getPatientId(), dataItem.getApplyId(),dataItem.getUpdateTime(),item);
                        if(writeItemMap!=null){
                            data.add(writeItemMap);
                        }
                    });
                }
            }
        }



        return data;
    }

    private  Map<String,Object> getWriteMapOds(CheckApplyHBO dataItem, Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();

        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        mapping.forEach((dp,ep)->{
            writeMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getOrderList())){
            List<ExamProjectBO> examProjectList=new ArrayList();
            dataItem.getOrderList().forEach(item->{
                examProjectList.add(HospitalSyncDateConvert.getExamProjectBO(item));
            });
            writeMap.put("items",JSON.toJSONString(examProjectList));
        }

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis", JSON.toJSONString(diagnoseList));
        }

        return writeMap;
    }


    private  Map<String,Object> getWriteMapDwd(CheckApplyHBO dataItem, Map<String,String> mapping){
        if(StrUtil.isEmpty(dataItem.getPatientId())){
            return null;
        }

        Map<String,Object> writeMap=new HashMap<>();

        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        mapping.forEach((dp,ep)->{
            writeMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis_raw", JSON.toJSONString(diagnoseList));
        }

        return writeMap;
    }




    private  Map<String,Object> getWriteMapItemDwd(String patientId, String applyId, Date updateTime,ApplyCheckOrderHBO dataItem){

        if(StrUtil.isEmpty(patientId)){
            return null;
        }



        Map<String,Object> writeMap=new HashMap<>();

        writeMap.put("patient_id", patientId);
        writeMap.put("apply_id", applyId);
        writeMap.put("id", SecureUtil.md5(StrUtil.format("{}_{}_{}",patientId,applyId,dataItem.getExamOrderCode())));
        writeMap.put("type_raw", dataItem.getExamName());
        writeMap.put("type_standard", dataItem.getExamName());
        writeMap.put("code_raw", dataItem.getExamOrderCode());
        writeMap.put("code_standard",  dataItem.getExamOrderCode());
        writeMap.put("name_raw",  dataItem.getExamOrderName());
        writeMap.put("name_standard", dataItem.getExamOrderName());
        writeMap.put("part_code_raw",dataItem.getExamItems());
        writeMap.put("part_code_standard",dataItem.getExamItems());
        writeMap.put("part_name_raw",dataItem.getExamItemClassName());
        writeMap.put("part_name_standard",dataItem.getExamItemClassName());
        writeMap.put("data_update_date", updateTime);

        return writeMap;
    }


}
