package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.service.OdsExamineApplyCacheService;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.CheckAppointmentHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 检查预约
 * */
@Service
public class SyncDataExamAppointmentService {


    @Autowired
    private OdsExamineApplyCacheService odsExamineApplyCacheService;

    /**
     *
     * */
    public List<CheckAppointmentHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                 DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                 boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<CheckAppointmentHBO> readData = BwwbSyncDataService.getCheckAppointment(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        Set<String> applyIds=new HashSet<>();
        readData.getData().forEach(item->{
           if(StrUtil.isNotEmpty(item.getApplyId())){
               if(StrUtil.isNotEmpty(item.getApplyId())){
                   applyIds.add(item.getApplyId());
               }
           }
        });

//        Map<String, String> apply2patientMap= odsExamineApplyCacheService.getMapByCache(applyIds);


        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            if(CollUtil.isNotEmpty(writeDataList)){
                writeDataMap.put(writeDBConfig.getKey(),writeDataList);
            }
        }


        return readData.getData();
    }

    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        //ods
        Map<String,String> map=new LinkedHashMap<>();
        //todo:接口返回未空值，暂时调整为 applyId
        //map.put("id","appNo"); //预约流水号
        map.put("id","applyId"); //预约流水号

        map.put("apply_id","applyId"); //申请流水号
        map.put("patient_id","__patient_id"); //
        map.put("book_date","examDate"); //预约时间
        map.put("start_date","beginTime"); //开始时间
        map.put("end_date","endTime"); //结束时间
        map.put("status","appStatus"); //预约状态
        map.put("status_name","appStatusName"); //预约状态名称

        map.put("data_update_date","updateTime");
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map.put("is_sync",SyncDataKeyEnum.SDKCV_SyncFlag.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_examine_reserve_info",dbUrl,dbUserName,dbUserPassword,"ods_examine_reserve_info",map));

//        //dwd
//        Map<String,String> map2=new LinkedHashMap<>();
//        //apply
//        map2.put("patient_id", "__patient_id");
//        //预约
//        //todo:接口返回未空值，暂时调整为 applyId
//        // map2.put("id", "appNo"); //预约流水号
//
//        map2.put("id", "applyId");
//        map2.put("apply_id", "applyId");
//        map2.put("book_date", "examDate");
//        map2.put("status_raw", "appStatus");
//        map2.put("data_update_date", "updateTime");
//        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
//        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
//        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_examine_reserve_info",dbUrl,dbUserName,dbUserPassword,"dwd_examine_reserve_info",map2));


        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }




    private List<Map<String,Object>> getDataList(HdsContext context, List<CheckAppointmentHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();
        if("ods_examine_reserve_info".equals(config.getKey())){
            for(CheckAppointmentHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_examine_reserve_info".equals(config.getKey())){
            for(CheckAppointmentHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapDwd(dataItem,mapping);
                if(writeItemMap!=null){
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.ExamineReserve.getCode(),dataItem.getApplyId());
                    }
                    data.add(writeItemMap);
                }
            }
        }

        return data;
    }

    private  Map<String,Object> getWriteMapOds(CheckAppointmentHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });

//        if(StrUtil.isNotEmpty(dataItem.getApplyId())){
//           String patient_id = apply2patientMap.get(dataItem.getApplyId());
//            writeObjMap.put("patient_id",patient_id);
//        }

        return writeObjMap;
    }


    private  Map<String,Object> getWriteMapDwd(CheckAppointmentHBO dataItem, Map<String,String> mappingp){
        System.out.println("+++++++++++++++++++ dwd start");
        if(StrUtil.isEmpty(dataItem.getApplyId())){
            return null;
        }
//        String patient_id = apply2patientMap.get(dataItem.getApplyId());
//        if(StrUtil.isEmpty(patient_id)){
//            return null;
//        }
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
//        mapping.forEach((dp,ep)->{
//            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
//        });
//        writeObjMap.put("patient_id",patient_id);
        System.out.println("+++++++++++++++++++ dwd end");
        return writeObjMap;
    }


}
