package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.service.OdsExamineApplyCacheService;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.ExamineReportConsultHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.ExamineReportsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 检查报告单数据
 * */
@Service
public class SyncDataExamReportService {


//    @Autowired
//    private OdsExamineApplyCacheService odsExamineApplyCacheService;
//


    /**
     *
     * */
    public List<ExamineReportsHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                               DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                               boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<ExamineReportsHBO> readData = BwwbSyncDataService.getCheckReports(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        Set<String> applyIds=new HashSet<>();
        readData.getData().forEach(item->{
            if(StrUtil.isNotEmpty(item.getRecipeId())){
                applyIds.add(item.getRecipeId());
            }
        });
//        Map<String, String> apply2patientMap= odsExamineApplyCacheService.getMapByCache(applyIds);


        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }


        return readData.getData();
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","id"); //ID
        map.put("apply_id","recipeId"); //申请流水号
        map.put("patient_id","__opid"); //
        map.put("visit_id","ipid"); //患者就诊卡号
        map.put("patient_name","patientName"); // 患者姓名
        map.put("patient_id_no","idCard");//身份证号
        map.put("patient_gender","sex");//性别
        map.put("patient_age","age");//年龄
        map.put("patient_phone","patientPhone");// 患者电话

        map.put("department_name","deptName");//申请科室名称
        map.put("department_code","deptCode");//申请科室代码
        map.put("apply_doctor_code","docCode");//申请医生代码
        map.put("apply_doctor_name","docName");//申请医生姓名

        map.put("in_patient_No","ipid"); //住院流水号

        map.put("scene_type","patientType"); //  患者类型【场景分类】

        map.put("device_type_code","classId");//设备类型id
        map.put("device_type_name","className");//设备类型名称

        map.put("report_description","reportResult");//报告结果
        map.put("report_result","reportDiagnose");//报告诊断

        map.put("reports_time","reportDate");//报告日期
        map.put("report_name","serviceName");//报告名称

        map.put("data_update_date","updateTime");

        map.put("report_path","filePath");
        map.put("report_no","checkSerialnum");

        map.put("status","status");

        map.put("is_sync",SyncDataKeyEnum.SDKCV_SyncFlag.getCode());
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("ods_examine_report_info",dbUrl,dbUserName,dbUserPassword,"ods_examine_report_info",map));

        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id", "__patientId");
        map2.put("id", "id");
        map2.put("apply_id", "recipeId");
        map2.put("visit_id", "ipid");
        map2.put("in_patient_no", "ipid");
        map2.put("patient_name", "patientName");
        map2.put("patient_phone", "patientPhone");
        map2.put("patient_gender_raw", "sex");//患者性别
        map2.put("patient_card_no", "idCard");
        map2.put("patient_age", "age");
        map2.put("department_name_raw", "deptName");
        map2.put("department_name_standard", "deptName");
        map2.put("department_code_raw", "deptCode");
        map2.put("department_code_standard", "deptCode");
        map2.put("reports_time", "reportDate");
        //map2.put("apply_time", "");
        //map2.put("inspect_time", "inspect_time");
        map2.put("scene_type_raw", "patientType"); //场景分类

        map2.put("apply_doctor_code", "docCode");
        map2.put("apply_doctor_name", "docName");

        map2.put("examine_type_raw", "__examine_type_raw");//项目编号
        map2.put("examine_type_name_raw", "__examine_type_name_raw"); //项目名称
        map2.put("body_part_name_raw", "__body_part_name_raw"); //检查部位名称
        map2.put("body_part_code_raw", "__body_part_code_raw"); //检查部位编号

        map2.put("report_name_raw", "serviceName");//报告名称
        map2.put("report_description_raw", "reportResult"); //检查所见
        map2.put("report_result_raw", "reportDiagnose");//检查结果

        map2.put("data_update_date", "updateTime");
        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
//        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_examine_report_info",dbUrl,dbUserName,dbUserPassword,"dwd_examine_report_info",map2));


        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }





    private List<Map<String,Object>> getDataList(HdsContext context, List<ExamineReportsHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_examine_report_info".equals(config.getKey())){
            for(ExamineReportsHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_examine_report_info".equals(config.getKey())){
//            for(ExamineReportsHBO dataItem:dataList){
//                Map<String,Object> writeItemMap=this.getWriteMapDwd(dataItem,mapping);
//                if(writeItemMap!=null){
//                    if(context.isCleanMqFlag()){
//                        context.addCleanObjId(MQCleanTableKeyEnum.ExamineReport.getCode(),dataItem.getId());
//                    }
//                    data.add(writeItemMap);
//                }
//            }
        }

        return data;
    }

    private  Map<String,Object> getWriteMapOds(ExamineReportsHBO dataItem, Map<String,String> mapping){
        if(dataItem==null || StrUtil.isEmpty(dataItem.getId())){
            return null;
        }


        String consults = dataItem.toConsults();
        if(StrUtil.isNotEmpty(consults)){
            if(StrUtil.isNotEmpty(dataItem.getReportResult())){
                dataItem.setReportResult(consults + dataItem.getReportResult());
            }else {
                dataItem.setReportResult(consults);
            }
        }


        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);


        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }


//    private  Map<String,Object> getWriteMapDwd(ExamineReportsHBO dataItem, Map<String,String> mapping){
//
//        if(dataItem==null || StrUtil.isEmpty(dataItem.getId()) ){
//            return null;
//        }
//        if(StrUtil.isEmpty(dataItem.getRecipeId())){
//            return null;
//        }
////        String patient_id = apply2patientMap.get(dataItem.getRecipeId());
////        if(StrUtil.isEmpty(patient_id)){
////            return null;
////        }
//
//        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
//        Map<String,Object> writeObjMap=new HashMap<>();
//
//
//        mapping.forEach((dp,ep)->{
//            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
//        });
//
//        //patient_id
////        writeObjMap.put("patient_id",patient_id);
//
//        return writeObjMap;
//    }

}
