package com.xxh.cloud.sync.framework.modules.biz.data;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.SlifeDocumentOutPatientHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.SlifeDocumentProgressNoteHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class SyncDataFirstCourseIllnessService {


    public List<SlifeDocumentProgressNoteHBO> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                        DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                        boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<SlifeDocumentProgressNoteHBO> readData = BwwbSyncDataService.getSlifeDocumentProgressNot(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }


    private List<Map<String,Object>> getDataList(HdsContext context, List<SlifeDocumentProgressNoteHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();
        if("ods_first_course_illness_info".equals(config.getKey())){
            for(SlifeDocumentProgressNoteHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }
        return data;
    }


    private  Map<String,Object> getWriteMap_Ods(SlifeDocumentProgressNoteHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }




    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","id"); //id
        map1.put("ip_id","ipid"); //
        map1.put("pid","pid"); //
        map1.put("node_date","nodeDate"); //节点时间
        map1.put("node_type","nodeType"); //节点分类编号
        map1.put("node_type_name","nodeTypeName"); //节点分类名称
        map1.put("high_doc_code","highDocCode"); //高等级医生编号
        map1.put("high_doc_name","highDocName"); //
        map1.put("content","xmlContent"); //病例内容
        map1.put("submit_time","submitTime"); //提交时间
        map1.put("doc_code","docCode"); //医生编号
        map1.put("doc_name","docName"); //医生名称
        map1.put("dept_code","deptCode"); //科室编号
        map1.put("dept_name","deptName"); //科室名称
        map1.put("ward_code","wardCode"); //病区号
        map1.put("ward_name","wardName"); //病区名称
        map1.put("modify_code","modifyCode"); //修改人
        map1.put("modify_name","modifyName"); //修改名称
        map1.put("template_id","templateId"); //模板ID
        map1.put("template_version","templateVersion"); //模板版本号

        map1.put("data_update_date","updateTimeStr"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("ods_first_course_illness_info",dbUrl,dbUserName,dbUserPassword,"ods_first_course_illness_info",map1));
        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }





}
