package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.RecipeInpatientHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 预约住院信息记录
 * */
@Log4j2
@Service
public class SyncDataInAppointmentService {


    public List<RecipeInpatientHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<RecipeInpatientHBO> readData = BwwbSyncDataService.getRecipeInPatient(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }
        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }

    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","id"); //ID
        map.put("visit_id","opid"); //就诊号

        map.put("register_id","visitId"); // 挂号流水号
        map.put("in_patient_no","ipid"); //  住院号

        map.put("patient_id","patientId"); //患者编号
        map.put("patient_name","patientName"); //患者名称

        map.put("apply_department_name","admitDepartmentName"); //申请科室名称
        map.put("apply_department_code","admitDepartmentCode"); //申请科室编码

        map.put("admit_department_name","admitDepartmentName"); //入院科室名称
        map.put("apply_department_code","admitDepartmentCode"); //入院科室编码
        map.put("apply_date","applyDate"); //申请时间
        map.put("book_date","bookDate"); //预约时间
        map.put("doctor_code","doctorCode"); //医生编号
        map.put("doctor_name","doctorName"); //医生姓名
        map.put("status","status"); //预约状态
        map.put("status_name","statusName"); //预约状态描述

        map.put("data_update_date","updateTime");
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_inpatient_reserve_info",dbUrl,dbUserName,dbUserPassword,"ods_inpatient_reserve_info",map));

        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id", "patientId");
        map2.put("id", "id");
        map2.put("visit_id", "visitId"); //opid
        map2.put("patient_name", "patientName");
        //map2.put("in_patient_no", "in_patient_no");
        //map2.put("company_code", "company_code");
        //map2.put("company_name", "company_name");
        map2.put("apply_department_name_raw", "admitDepartmentName");
        map2.put("apply_department_name_standard", "admitDepartmentName");
        map2.put("apply_department_code_raw", "admitDepartmentCode");
        map2.put("apply_department_code_standard", "admitDepartmentCode");

        map2.put("admit_department_name_raw", "admitDepartmentName");
        map2.put("admit_department_name_standard", "admitDepartmentName");
        map2.put("admit_department_code_raw", "admitDepartmentCode");
        map2.put("admit_department_code_standard", "admitDepartmentCode");

        map2.put("apply_date", "applyDate");
        map2.put("book_date", "bookDate");
        map2.put("doctor_code", "doctorCode");
        map2.put("doctor_name", "doctorName");
        map2.put("status_raw", "status");
        map2.put("data_update_date", "updateTime");
        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_inpatient_reserve_info",dbUrl,dbUserName,dbUserPassword,"dwd_inpatient_reserve_info",map2));


        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }



    private List<Map<String,Object>> getDataList(HdsContext context,List<RecipeInpatientHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

//        HdsContextMqQueue queueEvent=new HdsContextMqQueue(HMQQueueName.Que_PatientEvent);
//        if(context.isMqSendFlag()){
//            context.addMQQueueItem(queueEvent);
//        }

        if("ods_inpatient_reserve_info".equals(config.getKey())){
            for(RecipeInpatientHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("dwd_inpatient_reserve_info".equals(config.getKey())){
            for(RecipeInpatientHBO dataItem:dataList){
                //
                if(context.isCleanMqFlag()){
                    context.addCleanObjId(MQCleanTableKeyEnum.In_Reserve.getCode(),dataItem.getId());
                }

//                if(context.isMqSendFlag()){
//                    MQPatientEventBean peb=new MQPatientEventBean();
//                    peb.setPatientId(dataItem.getPatientId());
//                    peb.setEventType(PatientEventEnum.PE_E1040.getCode());
//                    peb.addEventDate(dataItem.getApplyDate());
//                    //peb.setEventData(writeObjMap);
//                    queueEvent.addItem(peb);
//                }

                Map<String,Object>  writeItemMap=this.getWriteMapDwd(dataItem,mapping);
                data.add(writeItemMap);
            }
        }
        return data;
    }

    private  Map<String,Object> getWriteMapOds(RecipeInpatientHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }


    private  Map<String,Object> getWriteMapDwd(RecipeInpatientHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }


}
