package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.InPatientRecordsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.RecipeItemsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.SlifePatientChangeHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.util.*;

@Log4j2
@Service
public class SyncDataInChangeService {




    public List<SlifePatientChangeHBO> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                 DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                 boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<SlifePatientChangeHBO> readData = BwwbSyncDataService.getSlifePatientChange(readConfig,startDate,endDate,logFlag,logBatchNo);

        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }





    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","id");//
        map.put("inpatient_id","ipid"); //住院号
        //map.put("patient_id",""); //

        map.put("change_time","changeTime"); //
        map.put("change_type","changeType"); //

        map.put("change_type_name","changeTypeName"); //
        map.put("code","code"); //
        map.put("name","name"); //

        map.put("source_code","sourceCode"); //
        map.put("source_name","sourceDesc"); //
        map.put("target_code","targetCode"); //
        map.put("target_name","targetDesc"); //
        map.put("data_update_date","updateTime");

        map.put("status", SyncDataKeyEnum.SDKCV_SyncFlag.getCode());
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_inpatient_change_info",dbUrl,dbUserName,dbUserPassword,"ods_inpatient_change_info",map));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);

        return config;
    }


    private List<Map<String,Object>> getDataList(HdsContext context,List<SlifePatientChangeHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();
        for(SlifePatientChangeHBO dataItem:dataList){
            Map<String,Object>  writeItemMap=this.getWriteMap(dataItem,mapping);
            data.add(writeItemMap);
        }
        return data;
    }

    private  Map<String,Object> getWriteMap(SlifePatientChangeHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);

        Map<String,Object> writeObjMap=new HashMap<>();

        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }




}
