package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.InPatientRecordsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 住院记录同步
 * */
@Service
public class SyncDataInRecordService {



    public List<InPatientRecordsHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                 DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                 boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<InPatientRecordsHBO> readData = BwwbSyncDataService.getInPatientRecord(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }





    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","visitId");//住院流水号
        map.put("in_patient_no","inPatientNo");//住院号
        map.put("patient_id","patientId");//患者编号
        map.put("patient_name","patientName");//患者名称
        map.put("patient_id_no","idCardNo");//身份证号

        map.put("company_code","companyNo");//医院编号
        map.put("company_name","companyName");//医院名称

        map.put("admit_department_name","departmentName");//住院科室名称
        map.put("admit_department_code","departmentCode");//住院科室编码

        map.put("inpatient_ward_code","inPatientWardCode");// 病区编号
        map.put("inpatient_ward_name","inPatientWardName");//病区名称
        map.put("bed_number","bedNo");//床号

        map.put("inpatient_date","date");//入院时间
        map.put("status","state");//住院状态

        map.put("doctor_code","doctorCode");//医生编号
        map.put("doctor_name","doctorName");//医生姓名

        map.put("inpatient_type","type");//类型

        map.put("data_update_date","updateTime");
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_inpatient_record_info",dbUrl,dbUserName,dbUserPassword,"ods_inpatient_record_info",map));
        //HwdsWriteConfigBean config =  HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,dbUrl,dbUserName,dbUserPassword,mqType,mqTopic,"ods_inpatient_record_info",map);


        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id", "patientId");
        map2.put("id", "visitId");
        map2.put("visit_id", "visitId");
        map2.put("in_patient_no", "inPatientNo");
        map2.put("patient_name", "patientName");

        map2.put("company_code", "companyNo");
        map2.put("company_name", "companyName");

        map2.put("admit_department_name_raw", "departmentName");
        map2.put("admit_department_name_standard", "departmentName");
        map2.put("admit_department_code_raw", "departmentCode");
        map2.put("admit_department_code_standard", "departmentCode");

        map2.put("inpatient_date", "date");
        map2.put("inpatient_ward", "inPatientWardCode");
        map2.put("bed_number", "bedNo");
        map2.put("enter_way_raw", "type");
        map2.put("status_raw","state");

        map2.put("doctor_code","doctorCode");
        map2.put("doctor_name","doctorName");

        map2.put("data_update_date", "updateTime");
        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_inpatient_record_info",dbUrl,dbUserName,dbUserPassword,"dwd_inpatient_record_info",map2));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);

        return config;
    }

    private List<Map<String,Object>> getDataList(HdsContext context,List<InPatientRecordsHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_inpatient_record_info".equals(config.getKey())){
            for(InPatientRecordsHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("dwd_inpatient_record_info".equals(config.getKey())){

//            HdsContextMqQueue queueEvent=new HdsContextMqQueue(HMQQueueName.Que_PatientEvent);
//            if(context.isMqSendFlag()){
//                context.addMQQueueItem(queueEvent);
//            }

            for(InPatientRecordsHBO dataItem:dataList){
                if(context.isCleanMqFlag()){
                    context.addCleanObjId(MQCleanTableKeyEnum.Inpatient.getCode(),dataItem.getVisitId());
                }

//                if(context.isMqSendFlag()){
//                    MQPatientEventBean peb=new MQPatientEventBean();
//                    peb.setPatientId(dataItem.getPatientId());
//                    peb.setEventType(PatientEventEnum.PE_E1080.getCode());
//                    peb.addEventDate(dataItem.getDate());
//                    queueEvent.addItem(peb);
//                }

                Map<String,Object>  writeItemMap=this.getWriteMapDwd(dataItem,mapping);
                data.add(writeItemMap);
            }
        }

        return data;
    }

    private  Map<String,Object> getWriteMapOds(InPatientRecordsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }


    private  Map<String,Object> getWriteMapDwd(InPatientRecordsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }



}
