package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.DiagnoseBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.ods.LaboratoryProjectBO;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.LaboratoryApplyBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.bo1.ApplyLabOrderHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 检验申请
 * */
@Service
public class SyncDataLaboratoryApplyService {


    /**
     *
     * */
    public List<LaboratoryApplyBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                               DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                               boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<LaboratoryApplyBO> readData = BwwbSyncDataService.getLaboratoryApply(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }


        return readData.getData();
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","applyId"); //检验申请流水号

        map.put("visit_id","visitId"); //就诊记录号opid
        map.put("recipe_id","visitId"); //关联处方id (作废)


        map.put("in_patient_no","inPatientNo");// 住院号

        map.put("patient_id","patientId");//患者编号

        map.put("patient_name","patientName");//患者姓名

        map.put("department_name","departmentName");//申请科室名称
        map.put("department_code","departmentCode");//申请科室代码

        map.put("apply_date","applyDate");//申请时间
        map.put("pay_time","payTime");//时间

        map.put("scene_type","type");//类型

        map.put("doctor_code","doctorCode");//申请医生编号
        map.put("doctor_name","doctorName");//申请医生姓名
        map.put("status","status");//申请状态

        map.put("items","__orderList");//项目
        map.put("diagnosis","__diagnoseList");//诊断

        map.put("data_update_date","updateTime"); //
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("ods_laboratory_apply_info",dbUrl,dbUserName,dbUserPassword,"ods_laboratory_apply_info",map));


        //dwd
        //DWD检验申请单信息表
        Map<String, String> map1 = new LinkedHashMap<>();
        map1.put("patient_id", "patientId");
        map1.put("id", "applyId");
        map1.put("visit_id", "visitId"); //opid
        map1.put("recipe_id", "visitId");
        map1.put("in_patient_no", "inPatientNo");
        map1.put("patient_name", "patientName");
        map1.put("department_name_raw", "departmentName");
        map1.put("department_name_standard", "departmentName");
        map1.put("department_code_raw", "departmentCode");
        map1.put("department_code_standard", "departmentCode");
        map1.put("pay_time", "payTime");
        map1.put("apply_date", "applyDate");
        map1.put("scene_type_raw", "type");
        map1.put("doctor_code", "doctorCode");
        map1.put("doctor_name", "doctorName");
        map1.put("status_raw","status");
        map1.put("diagnosis_raw","__diagnoseList");
        map1.put("data_update_date", "updateTime");
        map1.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_laboratory_apply_info",dbUrl,dbUserName,dbUserPassword,"dwd_laboratory_apply_info",map1));


        //DWD检验申请单项目信息表
        Map<String, String> map2 = new LinkedHashMap<>();
        map2.put("patient_id", "patientId");
        map2.put("laboratory_apply_id", "applyId");
        map2.put("id", "__itemId");
        map2.put("item_group_code_raw", "__item_group_code_raw");
        map2.put("item_group_code_standard","__item_group_code_standard");
        map2.put("item_group_name_raw", "__item_group_name_raw");
        map2.put("item_group_name_standard","__item_group_name_standard");
        map2.put("data_update_date", "updateTime");
        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_laboratory_apply_project_info",dbUrl,dbUserName,dbUserPassword,"dwd_laboratory_apply_project_info",map2));


        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }


    private List<Map<String,Object>> getDataList(HdsContext context,List<LaboratoryApplyBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_laboratory_apply_info".equals(config.getKey())){
            for(LaboratoryApplyBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("dwd_laboratory_apply_info".equals(config.getKey())){
            for(LaboratoryApplyBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapDwd(dataItem,mapping);
                if(writeItemMap!=null){
                    //
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.LaboratoryApply.getCode(),dataItem.getApplyId());
                    }
                    data.add(writeItemMap);
                }
            }


        }else if("dwd_laboratory_apply_project_info".equals(config.getKey())){
            for(LaboratoryApplyBO dataItem:dataList){
                if(CollUtil.isNotEmpty(dataItem.getOrderList())){
                    for(ApplyLabOrderHBO orderItem : dataItem.getOrderList()){
                        Map<String,Object>  writeItemMap=this.getWriteMapDwdItem(dataItem,orderItem,mapping);
                        data.add(writeItemMap);
                    }
                }
            }
        }

        return data;
    }

    private  Map<String,Object> getWriteMapOds(LaboratoryApplyBO dataItem, Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();

        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);

        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });


        if(CollUtil.isNotEmpty(dataItem.getOrderList())){
            List<LaboratoryProjectBO> examProjectList=new ArrayList();
            dataItem.getOrderList().forEach(item->{
                examProjectList.add(HospitalSyncDateConvert.getLaboratoryProjectBO(item));
            });
            writeMap.put("items", JSON.toJSONString(examProjectList));
        }

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis", JSON.toJSONString(diagnoseList));
        }

        return writeMap;
    }


    private  Map<String,Object> getWriteMapDwd(LaboratoryApplyBO dataItem, Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();

        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);

        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis_raw", JSON.toJSONString(diagnoseList));
        }

        return writeMap;
    }




    private  Map<String,Object> getWriteMapDwdItem(LaboratoryApplyBO dataItem,ApplyLabOrderHBO orderItem , Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();
        writeMap.put("patient_id", dataItem.getPatientId());
        writeMap.put("laboratory_apply_id", dataItem.getApplyId());
        writeMap.put("id", StrUtil.format("{}_{}",dataItem.getApplyId(),orderItem.getItemCode()));
        writeMap.put("item_group_code_raw", orderItem.getItemCode());
        writeMap.put("item_group_code_standard", orderItem.getItemCode());
        writeMap.put("item_group_name_raw", orderItem.getItemName());
        writeMap.put("item_group_name_standard", orderItem.getItemName());
        writeMap.put("data_update_date", dataItem.getUpdateTime());
        return writeMap;
    }


}
