package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.LaboratoryReportHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.bo1.LaboratoryReportsItemHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 检验报告单
 * */
@Service
@Log4j2
public class SyncDataLaboratoryReportService {


    /**
     *
     * */
    public List<LaboratoryReportHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                 DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                 boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<LaboratoryReportHBO> readData = BwwbSyncDataService.getLaboratoryReport(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context, readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }


        return readData.getData();
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){


        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        //ods
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","id"); //
        map1.put("apply_id","applyId"); //申请单号
        //map1.put("visit_id","opid"); //就诊记录号
        map1.put("visit_id","visitId"); //就诊记录号
        map1.put("report_name","itemName"); //项目名称
        map1.put("recipe_id","recipeId"); //关联处方id
        map1.put("in_patient_no","inPatientNo"); //住院号
        map1.put("patient_id","patientId"); //患者编号
        map1.put("patient_name","patientName"); //患者姓名
        map1.put("department_name","departmentName"); //申请科室姓名
        map1.put("department_code","departmentCode"); //申请科室编码
        map1.put("apply_time","applyTime"); //申请时间
        map1.put("inspect_time","inspectTime"); //采样时间
        map1.put("reports_time","reportsTime"); //报告时间
        map1.put("check_time","checkTime"); //复核时间
        //map1.put("scene_type","type"); //类型
        map1.put("scene_type","patientType"); //患者类型
        map1.put("apply_doctor_code","applyDoctorCode"); //申请医生编号
        map1.put("apply_doctor_name","applyDoctorName"); //申请医生姓名
        map1.put("sample_type","sampleType"); //样本类别
        map1.put("status","status"); //状态
        map1.put("data_update_date","updateTime"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_laboratory_report_info",dbUrl,dbUserName,dbUserPassword,"ods_laboratory_report_info",map1));

        //ods
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("id","__id");
        map2.put("laboratory_report_id","__mainId"); //id
        map2.put("sample_type","__sample_type"); //检验类型
        map2.put("test_date","testDate"); //检验时间
        map2.put("machine_code","machineCode"); //机器号
        map2.put("prd_id","hisItemCode"); //his项目编号
        map2.put("code","code"); //项目编码
        map2.put("name","name"); //项目名称
        map2.put("inspect_result","inspectResult"); //结果
        map2.put("unit","unit"); //单位
        map2.put("abnormal_indicator","abnormalIndicator"); //异常标记
        map2.put("abnormal","abnormal"); //是否异常:0:否；1.是
        map2.put("reference","reference"); //参考范围
        //map2.put("","machineId"); //
        map2.put("bar_code","barcode"); //
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_laboratory_report_item_info",dbUrl,dbUserName,dbUserPassword,"ods_laboratory_report_item_info",map2));

        //dwd-
        Map<String, String> map3 = new LinkedHashMap<>();
        map3.put("patient_id", "patientId");
        map3.put("id", "id");
        map3.put("apply_id", "applyId");
        map3.put("visit_id", "visitId");
        map3.put("recipe_id", "recipeId");
        map3.put("in_patient_no", "inPatientNo");

        map3.put("patient_name", "patientName");
        map3.put("department_name_raw", "departmentName");
        map3.put("department_name_standard", "departmentName");
        map3.put("department_code_raw", "departmentCode");
        map3.put("department_code_standard", "departmentCode");


        map3.put("reports_time", "checkTime"); //reportsTime
        map3.put("check_time", "checkTime");
        map3.put("apply_time", "applyTime");
        map3.put("inspect_time", "inspectTime");

        map3.put("scene_type_raw", "patientType");
        map3.put("scene_type_standard", "scene_type_standard"); //场景分类
        map3.put("apply_doctor_code", "applyDoctorCode");
        map3.put("apply_doctor_name", "applyDoctorName");
        map3.put("sample_type_raw", "sampleType");
        map3.put("status_raw","status"); //状态
        map3.put("report_name", "itemName");

        map3.put("data_update_date", "updateTime");
        map3.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map3.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_laboratory_report_info",dbUrl,dbUserName,dbUserPassword,"dwd_laboratory_report_info",map3));

        //dwd-
        Map<String, String> map4 = new LinkedHashMap<>();
        map4.put("patient_id","__patientId");


        map4.put("id","__id");
        map4.put("laboratory_report_id","__mainId");
        map4.put("sample_type_raw","__sample_type_raw");
        map4.put("reports_time","__reports_time");

        map4.put("project_group_code_raw","hisItemCode");

        map4.put("project_id_raw","code");
        map4.put("name_raw","name");
        map4.put("code_raw","code");

        map4.put("abnormal_indicator","abnormalIndicator");
        map4.put("abnormal","abnormal");

        map4.put("unit_raw","unit");
        map4.put("reference_raw","reference");
        map4.put("inspect_result_raw","inspectResult");

        map4.put("data_update_date", "updateTime");
        map4.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map4.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_laboratory_report_item_info",dbUrl,dbUserName,dbUserPassword,"dwd_laboratory_report_item_info",map4));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }


    private List<Map<String,Object>> getDataList(HdsContext context,List<LaboratoryReportHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();

        if("ods_laboratory_report_info".equals(config.getKey())){
            for(LaboratoryReportHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Report(dataItem, config.getMapping());
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }

        }else  if("ods_laboratory_report_item_info".equals(config.getKey())){
            for(LaboratoryReportHBO dataItem:dataList){
                if(CollUtil.isNotEmpty(dataItem.getReportItems())){
                    dataItem.getReportItems().forEach(item->{
                        Map<String,Object>  writeItemMap=this.getWriteMap_ReportItem(dataItem.getId(),dataItem.getSampleType(),item, config.getMapping());
                        if(writeItemMap!=null){
                            data.add(writeItemMap);
                        }
                    });
                }
            }

        }else if("dwd_laboratory_report_info".equals(config.getKey())){
            for(LaboratoryReportHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Report_Dwd(dataItem, config.getMapping());
                if(writeItemMap!=null){
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.LaboratoryReport.getCode(),dataItem.getId());
                    }
                    data.add(writeItemMap);
                }
            }



        }else if("dwd_laboratory_report_item_info".equals(config.getKey())){
            for(LaboratoryReportHBO dataItem:dataList){
                dataItem.getReportItems().forEach(item->{
                    Map<String,Object>  writeItemMap=this.getWriteMap_ReportItem_DWD(dataItem.getId(),dataItem.getPatientId(),
                            dataItem.getSampleType(),
                            dataItem.getReportsTime(),
                            item, config.getMapping());
                    if(writeItemMap!=null){
                        data.add(writeItemMap);
                    }
                });
            }

        }





        return data;
    }

    private  Map<String,Object> getWriteMap_Report(LaboratoryReportHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }

    private  Map<String,Object> getWriteMap_ReportItem(String id ,String sampleType, LaboratoryReportsItemHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });

        writeMap.put("laboratory_report_id",id);
        writeMap.put("sample_type",sampleType);

        String id2="ID";
        String barcode2="BC";
        String hisItemCode2="HC";
        String code2="C";

        if(StrUtil.isNotEmpty(dataItem.getId())){
            id2=dataItem.getId();
        }

        if(StrUtil.isNotEmpty(dataItem.getBarcode())){
            barcode2=dataItem.getBarcode();
        }

        if(StrUtil.isNotEmpty(dataItem.getHisItemCode())){
            hisItemCode2=dataItem.getHisItemCode();
        }

        if(StrUtil.isNotEmpty(dataItem.getCode())){
            code2=dataItem.getCode();
        }

        String idStr=StrUtil.format("D_{}_{}_{}_{}",id2,barcode2,hisItemCode2,code2);
        writeMap.put("id", SecureUtil.md5(idStr));



        if(!id.equals(dataItem.getId())){
            log.error("+++++ 报告ID不一致，主ID :{} ,明细ID:{}",id,dataItem.getId());
        }

        return writeMap;
    }


    private  Map<String,Object> getWriteMap_Report_Dwd(LaboratoryReportHBO dataItem, Map<String,String> mapping){
        if(dataItem==null || StrUtil.isEmpty(dataItem.getPatientId())){
            return null;
        }
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }


    private  Map<String,Object> getWriteMap_ReportItem_DWD(String id ,String patientId , String sampleType, Date reportsTime, LaboratoryReportsItemHBO dataItem, Map<String,String> mapping){
        if(StrUtil.isEmpty(patientId)){
            return null;
        }

        Map<String, Object> readMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readMap));
        });


        writeMap.put("patient_id",patientId);
        writeMap.put("laboratory_report_id",id);
        writeMap.put("sample_type_raw",sampleType);
        writeMap.put("reports_time",reportsTime);


        String id2="ID";
        String barcode2="BC";
        String hisItemCode2="HC";
        String code2="C";

        if(StrUtil.isNotEmpty(dataItem.getId())){
            id2=dataItem.getId();
        }

        if(StrUtil.isNotEmpty(dataItem.getBarcode())){
            barcode2=dataItem.getBarcode();
        }

        if(StrUtil.isNotEmpty(dataItem.getHisItemCode())){
            hisItemCode2=dataItem.getHisItemCode();
        }

        if(StrUtil.isNotEmpty(dataItem.getCode())){
            code2=dataItem.getCode();
        }

        String idStr=StrUtil.format("D_{}_{}_{}_{}",id2,barcode2,hisItemCode2,code2);
        writeMap.put("id",SecureUtil.md5(idStr));

        if(!id.equals(dataItem.getId())){
            log.error("+++++ 报告ID不一致，主ID :{} ,明细ID:{}",id,dataItem.getId());
        }

        return writeMap;
    }


}
