package com.xxh.cloud.sync.framework.modules.biz.data;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncConfigTypeEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.OperationDiagnoseBO;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.ElectronMedicalRecordHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.OperationRecordHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.SlifeFeeDetailHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 电子病例
 * */
@Service
public class SyncDataMedicalRecordService {




    public List<ElectronMedicalRecordHBO> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                               DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                               boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<ElectronMedicalRecordHBO> readData = BwwbSyncDataService.getElectronMedicalRecord(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());


        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }





        return readData.getData();
    }



    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","opid"); //关联id 【主键】
        //visitId
        map1.put("patient_id","patientId"); //
        map1.put("patient_name","patientName"); //
        map1.put("patient_type","patientType"); //
        map1.put("patient_card_no","idcard"); //
        map1.put("department_name","deptName"); //
        map1.put("department_code","deptCode"); //
        map1.put("doctor_code","docCode"); //
        map1.put("doctor_name","docName"); //
        map1.put("status","status"); //
        map1.put("status_name","statusName"); //
        map1.put("content","xmlContent"); //
        map1.put("data_update_date","updateTime"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_patient_medical_record_info",dbUrl,dbUserName,dbUserPassword,"ods_patient_medical_record_info",map1));


        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id","patientId"); //
        map2.put("id","opid"); //
        map2.put("patient_name","patientName"); //
        map2.put("patient_type_raw","patientType"); //
        map2.put("patient_type_standard",""); //
        map2.put("patient_card_no","idcard"); //
        map2.put("department_name_raw","deptName"); //
        map2.put("department_code_raw","deptCode"); //
        map2.put("department_name_standard","deptName"); //
        map2.put("department_code_standard","deptCode"); //
        map2.put("doctor_code","docCode"); //
        map2.put("doctor_name","docName"); //
        map2.put("status_raw","status"); //
        map2.put("status_standard","status"); //
        map2.put("status_name_raw","statusName"); //
        map2.put("data_update_date","updateTime"); //
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map2.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_patient_medical_record_info",dbUrl,dbUserName,dbUserPassword,"dwd_patient_medical_record_info",map2));

        Map<String,String> map3=new LinkedHashMap<>();
        map3.put("patient_id","patientId");
        map3.put("id","opid");
        map3.put("content_raw","xmlContent");
        map3.put("data_update_date","updateTime");
        map3.put("struct_flag",SyncDataKeyEnum.SDKCV_SyncFlag.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_patient_medical_record_content_info",dbUrl,dbUserName,dbUserPassword,"dwd_patient_medical_record_content_info",map3));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;

    }


    private List<Map<String,Object>> getDataList(HdsContext context,List<ElectronMedicalRecordHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_patient_medical_record_info".equals(config.getKey())){
            for(ElectronMedicalRecordHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("dwd_patient_medical_record_info".equals(config.getKey())){
            for(ElectronMedicalRecordHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Dwd(dataItem,mapping);
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_patient_medical_record_content_info".equals(config.getKey())){
            for(ElectronMedicalRecordHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapContent_Dwd(dataItem,mapping);
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }
        }


        return data;
    }


    //ods
    private  Map<String,Object> getWriteMap_Ods(ElectronMedicalRecordHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }



    //dwd
    private  Map<String,Object> getWriteMap_Dwd(ElectronMedicalRecordHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }

    //dwd-content
    private  Map<String,Object> getWriteMapContent_Dwd(ElectronMedicalRecordHBO dataItem, Map<String,String> mapping){
//        if(StrUtil.isEmpty(dataItem.getPatientId())){
//            return null;
//        }
//        if(StrUtil.isEmpty(dataItem.getVisitId())){
//            return null;
//        }
//
        if(StrUtil.isEmpty(dataItem.getXmlContent())){
            return null;
        }
//

//        Map<String,Object> writeMap=new HashMap<>();
//        writeMap.put("patient_id",dataItem.getPatientId());
//        writeMap.put("id",dataItem.getVisitId());
//        writeMap.put("content_raw",dataItem.getXmlContent());
//        return writeMap;

        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;


    }




}
