package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.OperationDiagnoseBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.OperationRecordHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 手术记录数据
 * */
@Service
public class SyncDataOperationRecordService {




    public List<OperationRecordHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<OperationRecordHBO> readData = BwwbSyncDataService.getOperationRecord(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }



    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","id"); //关联id 【主键】
        map1.put("visit_id","visitId"); //就诊记录号
        map1.put("apply_id","recipeId"); //记录流水号【关联手术申请】
        map1.put("in_patient_no","inPatientNo"); //住院号
        map1.put("patient_id","patientId"); //患者编号
        map1.put("patient_name","patientName"); //患者姓名

        map1.put("department_code","departmentCode"); //申请科室编码
        map1.put("department_name","departmentName"); //申请科室名称

        map1.put("apply_time","applyTime"); //申请手术时间

        map1.put("scene_type","patientType"); //1.门诊 2.急诊 4.住院

        map1.put("doctor_code","doctorCode"); //申请医生代码
        map1.put("doctor_name","doctorName"); //申请医生姓名

        map1.put("description","description"); //备注

        map1.put("status_code","status"); //申请状态
        map1.put("status_name","statusName"); //申请状态名称


        map1.put("code","code"); //手术编码
        map1.put("name","name"); //手术名称
        map1.put("operation_level","operationLevel"); //手术等级
        map1.put("est_opt_duration","estOptDuration"); //预计手术时间
        map1.put("anaesthesia_method","anaesthesiaMethod"); //麻醉方法
        map1.put("operation_type","operationType"); //手术类别

        map1.put("operation_doctor_id","operationDoctorId"); //手术医生编码
        map1.put("operation_doctor","operationDoctor"); //手术医生姓名

        map1.put("act_start_time","actStartTime"); //实际手术开始时间
        map1.put("act_end_time","actEndTime"); //实际手术结束时间
        map1.put("final_diagnosis","__diagnoseList"); //手术后诊

        map1.put("content","xmlContent"); //内容

        map1.put("data_update_date","updateTime"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_operation_record_info",dbUrl,dbUserName,dbUserPassword,"ods_operation_record_info",map1));


        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id","patientId"); //患者编号
        map2.put("id","id"); //关联id 【主键】
        map2.put("visit_id","visitId"); //就诊记录号
        map2.put("recipe_id","recipeId"); //记录流水号【关联手术申请】
        map2.put("in_patient_no","inPatientNo"); //住院号
        map2.put("patient_name","patientName"); //患者姓名
        map2.put("department_code_raw","departmentCode"); //申请科室编码
        map2.put("department_code_standard","departmentCode"); //申请科室编码
        map2.put("department_name_raw","departmentName"); //申请科室名称
        map2.put("department_name_standard","departmentName"); //申请科室名称
        map2.put("apply_time","applyTime"); //申请手术时间
        map2.put("pay_time","actEndTime"); //
        map2.put("scene_type_raw","patientType"); //1.门诊 2.急诊 4.住院
        map2.put("doctor_code","doctorCode"); //申请医生代码
        map2.put("doctor_name","doctorName"); //申请医生姓名
        map2.put("description","description"); //备注
        map2.put("status_code_raw","status"); //申请状态
        map2.put("status_name_raw","statusName"); //申请状态名称
        map2.put("final_diagnosis_raw","__diagnoseList"); //手术后诊
        map2.put("data_update_date","updateTime"); //
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map2.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_operation_record_info",dbUrl,dbUserName,dbUserPassword,"dwd_operation_record_info",map2));

        Map<String,String> map3=new LinkedHashMap<>();
        map3.put("patient_id","patientId");
        map3.put("operation_record_id","id");
        map3.put("id","id");
        map3.put("code_raw","code");
        map3.put("code_standard","code");

        map3.put("name_raw","name");
        map3.put("name_standard","name");

        map3.put("est_opt_duration","estOptDuration");
        map3.put("operation_type_raw","operationType"); //手术类别
        map3.put("operation_level_raw","operationLevel");
        map3.put("anaesthesia_method_raw","anaesthesiaMethod");
        map3.put("operation_doctor_code","operationDoctorId");
        map3.put("operation_doctor_name","operationDoctor");
        map3.put("act_start_time","actStartTime");
        map3.put("act_end_time","actEndTime");
        map3.put("data_update_date","updateTime"); //
        map3.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map3.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_operation_record_item_info",dbUrl,dbUserName,dbUserPassword,"dwd_operation_record_item_info",map3));


        Map<String,String> map4=new LinkedHashMap<>();
        map4.put("patient_id","patientId"); //
        map4.put("id","id"); //
        map4.put("content_raw","xmlContent"); //
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_operation_content_info",dbUrl,dbUserName,dbUserPassword,"dwd_operation_content_info",map4));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }



    private List<Map<String,Object>> getDataList(HdsContext context,List<OperationRecordHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_operation_record_info".equals(config.getKey())){
            for(OperationRecordHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("dwd_operation_record_info".equals(config.getKey())){
            for(OperationRecordHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Dwd(dataItem,mapping);
                if(writeItemMap!=null){
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.Operation.getCode(),dataItem.getId());
                    }
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_operation_record_item_info".equals(config.getKey())){
            for(OperationRecordHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapItem_Dwd(dataItem,mapping);
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }
        }else if("dwd_operation_content_info".equals(config.getKey())){
            for(OperationRecordHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Content_Dwd(dataItem,mapping);
                if(writeItemMap!=null){
                    data.add(writeItemMap);
                }
            }
        }


        return data;
    }

    private  Map<String,Object> getWriteMap_Ods(OperationRecordHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<OperationDiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getOperationDiagnoseBO(item));
            });
            writeMap.put("final_diagnosis", JSON.toJSONString(diagnoseList));
        }
        return writeMap;
    }


    private  Map<String,Object> getWriteMap_Dwd(OperationRecordHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<OperationDiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getOperationDiagnoseBO(item));
            });
            writeMap.put("final_diagnosis_raw", JSON.toJSONString(diagnoseList));
        }
        return writeMap;
    }



    private  Map<String,Object> getWriteMapItem_Dwd(OperationRecordHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }


    private  Map<String,Object> getWriteMap_Content_Dwd(OperationRecordHBO dataItem, Map<String,String> mapping){

//        if(StrUtil.isEmpty(dataItem.getPatientId())){
//            return null;
//        }
//
//        if(StrUtil.isEmpty(dataItem.getId())){
//            return null;
//        }
//
        if(dataItem.getXmlContent()==null || "".equals(dataItem.getXmlContent())){
            return null;
        }
//
//        Map<String,Object> writeMap=new HashMap<>();
//        writeMap.put("patient_id",dataItem.getPatientId());
//        writeMap.put("id",dataItem.getId());
//        writeMap.put("content_raw",dataItem.getXmlContent());
//        return writeMap;

        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;

    }

}
