package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.DiagnoseBO;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.OutRecipeRecordsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 出院小结记录
 * */
@Service
public class SyncDataOutSummaryService {



    public List<OutRecipeRecordsHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                 DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                 boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<OutRecipeRecordsHBO> readData = BwwbSyncDataService.getOutRecipeRecords(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }


        return readData.getData();
    }



    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","visitId");//住院流水号
        //map.put("visit_id","visitId");//就诊记录号
        map.put("in_patient_no","inPatientNo");//关联住院号
        map.put("patient_id","patientId");//患者编号
        map.put("admit_department_name","admitDepartmentName");//入院科室名称
        map.put("admit_department_code","admitDepartmentCode");//入院科室编码
        map.put("leave_department_name","outAdmitdepartmentName");//出院科室名称
        map.put("leave_department_code","outAdmitdepartmentCode");//出院科室编码
        map.put("leave_date","outDate");//出院时间
        map.put("leave_way","leaveWay");//出院方式编号
        map.put("leave_way_name","leaveWayName");//出院方式名称
        map.put("content","content");//出院总结
        map.put("diagnosis","__diagnoseList");//诊断列表
        map.put("status","inStatus");//住院状态
        map.put("data_update_date","updateTime");
        map.put("is_sync",SyncDataKeyEnum.SDKCV_SyncFlag.getCode());
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_discharge_record_info",dbUrl,dbUserName,dbUserPassword,"ods_discharge_record_info",map));



        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id", "patientId");
        map2.put("id", "visitId");
        map2.put("visit_id", "visitId");
        map2.put("in_patient_no", "inPatientNo");

        map2.put("admit_department_name_raw", "admitDepartmentName");
        map2.put("admit_department_name_standard", "admitDepartmentName");
        map2.put("admit_department_code_raw", "admitDepartmentCode");
        map2.put("admit_department_code_standard", "admitDepartmentCode");

        map2.put("out_department_name_raw", "outAdmitdepartmentName");
        map2.put("out_department_name_standard", "outAdmitdepartmentName");
        map2.put("out_department_code_raw", "outAdmitdepartmentCode");
        map2.put("out_department_code_standard", "outAdmitdepartmentCode");

        map2.put("leave_date", "outDate");
        map2.put("leave_way_raw", "leaveWay");//出院方式
        map2.put("leave_way_name_raw", "leaveWayName");

        map2.put("diagnosis_raw", "__diagnoseList");

        map2.put("data_update_date", "updateTime");
        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_discharge_record_info",dbUrl,dbUserName,dbUserPassword,"dwd_discharge_record_info",map2));
        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }



    private List<Map<String,Object>> getDataList(HdsContext context,List<OutRecipeRecordsHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_discharge_record_info".equals(config.getKey())){
            for(OutRecipeRecordsHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("dwd_discharge_record_info".equals(config.getKey())){
            for(OutRecipeRecordsHBO dataItem:dataList){
                if(context.isCleanMqFlag()){
                    context.addCleanObjId(MQCleanTableKeyEnum.Discharge.getCode(),dataItem.getVisitId());
                }
                Map<String,Object>  writeItemMap=this.getWriteMapDwd(dataItem,mapping);
                data.add(writeItemMap);
            }

        }

        return data;
    }

    private  Map<String,Object> getWriteMapOds(OutRecipeRecordsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis", JSON.toJSONString(diagnoseList));
        }
        return writeMap;
    }


    private  Map<String,Object> getWriteMapDwd(OutRecipeRecordsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });

        if(CollUtil.isNotEmpty(dataItem.getDiagnoseList())){
            List<DiagnoseBO> diagnoseList=new ArrayList<>();
            dataItem.getDiagnoseList().forEach(item->{
                diagnoseList.add(HospitalSyncDateConvert.getDiagnoseBO(item));
            });
            writeMap.put("diagnosis_raw", JSON.toJSONString(diagnoseList));
        }

        return writeMap;
    }

}
