package com.xxh.cloud.sync.framework.modules.biz.data;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.digest.MD5;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.bean.pe.SlifeTjSuggest;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.RecipeItemsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.SlifeDocumentOutPatientHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.sql.Struct;
import java.util.*;

@Service
public class SyncDataPastHisService {




    public List<SlifeDocumentOutPatientHBO> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                            DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                            boolean logFlag, String logBatchNo){


        BwwbSyncDataRespHBO<SlifeDocumentOutPatientHBO> readData = BwwbSyncDataService.getSlifeDocumentOutPatient(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }



    private List<Map<String,Object>> getDataList(HdsContext context, List<SlifeDocumentOutPatientHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();
        if("ods_patient_past_his_info".equals(config.getKey())){
            for(SlifeDocumentOutPatientHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }
        return data;
    }


    private  Map<String,Object> getWriteMap_Ods(SlifeDocumentOutPatientHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        String idStr= StrUtil.format("{}-{}",dataItem.getRegisterNo(),DateUtil.formatDateTime(dataItem.getTreatDate()));
        writeMap.put("id", SecureUtil.md5(idStr));
        return writeMap;
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id",SyncDataKeyEnum.SDKCV_UUID.getCode()); // id
        map1.put("register_id","registerNo"); // 挂号流水号
        map1.put("treat_date","treatDate"); //看诊时间
        map1.put("doc_code","docCode"); //医生编号
        map1.put("doc_name","docName"); //医生名称
        map1.put("status","status"); //状态
        map1.put("pid","pid"); //
        map1.put("patient_name","atientName"); //p患者名称
        map1.put("age","age"); //患者年龄
        map1.put("sex","sex"); //患者性别
        map1.put("id_card","idCard"); //患者身份证号
        map1.put("dept_code","deptCode"); //科室编号
        map1.put("dept_name","deptName"); //科室名称
        map1.put("illness_now","illnessNow"); //现病史
        map1.put("illness_tell","illnessTell"); //主诉
        map1.put("illness_history","illnessHistory"); //既往史
        map1.put("assistant_check","assistantCheck"); //辅助检查
        map1.put("content","xmlContent"); //病历原文
        map1.put("data_update_date","updateTimeStr"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_patient_past_his_info",dbUrl,dbUserName,dbUserPassword,"ods_patient_past_his_info",map1));
        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }









}
