package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncConfigTypeEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.PatientRecordHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 患者基本信息
 * */
@Service
public class SyncDataPatientInfoService {

    /**
     *
     * */
    public List<PatientRecordHBO> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig,
                                              String startDate, String endDate , DataSyncLogSaveDTO log,
                                              Map<String,List<Map<String,Object>>> writeDataMap,
                                              boolean logFlag, String logBatchNo){

        BwwbSyncDataRespHBO<PatientRecordHBO> readData = BwwbSyncDataService.getPatientRecord(readConfig,startDate,
                endDate,logFlag,logBatchNo);
        log.setParams(startDate+";"+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());



        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }


        if(SyncConfigTypeEnum.SCT_Kafka.getCode().equals(writerConfig.getDbType())){
            //todo:发送kafka
        }

        return readData.getData();
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("patient_id","patientId");
        map1.put("patient_name","patientName");
        map1.put("id_card_no","idCardNo");
        map1.put("patient_gender","patientGender");
        map1.put("birthday","birthday");
        map1.put("phone","phone");
        map1.put("work_phone","workPhone");
        map1.put("link_phone","linkPhone");
        map1.put("home_phone","homePhone");

        map1.put("data_update_date","updateTime");
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_patient_info",dbUrl,dbUserName,dbUserPassword,"ods_patient_info",map1));

        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id","patientId");
        map2.put("patient_name","patientName"); //
        map2.put("card_no_type_raw",HospitalSyncDateUtils.getDefaultValue("SFZ")); //
        map2.put("card_no_type_standard",HospitalSyncDateUtils.getDefaultValue("SFZ"));
        map2.put("card_no","idCardNo"); //
        map2.put("patient_gender_raw","patientGender"); //
        map2.put("birthday","birthday"); //
        map2.put("phone","phone"); //

        map1.put("work_phone","workPhone");
        map1.put("link_phone","linkPhone");
        map1.put("home_phone","homePhone");

        map2.put("data_update_date","updateTime"); //
        map2.put("main_person_id",HospitalSyncDateUtils.getDefaultValue("0"));
        map2.put("death_status",HospitalSyncDateUtils.getDefaultValue("Y"));
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map2.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_patient_info",dbUrl,dbUserName,dbUserPassword,"dwd_patient_info",map2));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);


        return config;
    }

    private List<Map<String,Object>> getDataList(HdsContext context,List<PatientRecordHBO> readDataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_patient_info".equals(config.getKey())){
            for(PatientRecordHBO readData:readDataList){
                Map<String,Object> writeObjMap=this.getWriteMapOds(readData,mapping);
                data.add(writeObjMap);
            }
            return data;
        } else if("dwd_patient_info".equals(config.getKey())){
            for(PatientRecordHBO readData:readDataList){
                Map<String,Object> writeObjMap=this.getWriteMapDwd(readData,mapping);
                if(context.isCleanMqFlag()){
                    context.addCleanObjId(MQCleanTableKeyEnum.Patient_Info.getCode(),readData.getPatientId());
                }
                if(writeObjMap!=null){
                    data.add(writeObjMap);
                }
            }
            return data;
        }

        return data;
    }


    private Map<String,Object> getWriteMapOds(PatientRecordHBO dataItem, Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        mapping.forEach((dp,ep)->{
            writeMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }

    private Map<String,Object> getWriteMapDwd(PatientRecordHBO dataItem, Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        mapping.forEach((dp,ep)->{
            writeMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }



}
