package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.RegisterRecordsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 挂号
 * */
@Service
public class SyncDataPatientRegisterService {



    /**
     *
     * */
    public List<RegisterRecordsHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig,
                                                String startDate, String endDate , DataSyncLogSaveDTO log,
                                                Map<String,List<Map<String,Object>>> writeDataMap, boolean logFlag, String logBatchNo){

        BwwbSyncDataRespHBO<RegisterRecordsHBO> readData = BwwbSyncDataService.getRegisterRecords(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+";"+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }


        return readData.getData();
    }


    /**
     *
     * */
    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();
        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","visitId"); //挂号流水号:门诊号/发票号
        map.put("patient_id","patientId"); //门诊卡号:就诊卡号
        map.put("status","transType"); //交易类型:1.收费；2.退费
        map.put("card_no","idCardNo"); // 身份证号
        map.put("patient_name","patientName"); //患者姓名
        map.put("scene_type","patientType");//患者类型：1.门诊；2，急诊
        map.put("company_code","companyNo");//医院编号
        map.put("company_name","companyName");//医院名称
        map.put("department_code","departmentCode");//科室代码
        map.put("department_name","departmentName");//科室名称
        map.put("reg_date","registerTime"); // 挂号时间
        map.put("reg_level_code","regLevelCode");//挂号级别
        map.put("reg_level_name","regLevelName");//挂号级别名称
        map.put("reg_way","ynBook");//挂号方式  0:现场挂号；1：预约挂号；2.特诊挂号
        map.put("first_sub_type","type"); //1.初诊；2.复诊

        map.put("visit_idx","seeOrder");// 看诊序号 int
        map.put("visit_status","ynSee"); //是否看诊:1.是；2.否
        map.put("visit_date","date"); //看诊时间

        map.put("doctor_code","doctorCode"); //医师编号
        map.put("doctor_name","doctorName"); //医师姓名

        map.put("data_update_date","updateTime");
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("ods_register_record_info",dbUrl,dbUserName,dbUserPassword,"ods_register_record_info",map));

        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id", "patientId");
        map2.put("id", "visitId");
        map2.put("patient_name", "patientName");
        map2.put("card_type_raw", HospitalSyncDateUtils.getDefaultValue("SFZ"));
        map2.put("card_type_standard", HospitalSyncDateUtils.getDefaultValue("SFZ")); //证件类型
        map2.put("card_no", "idCardNo");
        map2.put("visit_id", "visitId");
        map2.put("company_name", "companyName");
        map2.put("company_code", "companyNo");
        map2.put("department_name_raw", "departmentName");
        map2.put("department_name_standard", "departmentName");
        map2.put("department_code_raw", "departmentCode");
        map2.put("department_code_standard", "departmentCode");
        map2.put("doctor_code", "doctorCode");
        map2.put("doctor_name", "doctorName");

        map2.put("reg_level_code_raw", "regLevelCode");
        map2.put("reg_level_name_raw", "regLevelName");
        map2.put("reg_way_raw", "ynBook");
        map2.put("reg_date", "registerTime");
        map2.put("first_sub_type", "type");
        map2.put("scene_type_raw", "patientType");
        map2.put("visit_status_raw", "ynSee");
        map2.put("visit_date", "date");
        map2.put("status_raw", "transType");
        map2.put("data_update_date", "updateTime");
        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_register_record_info",dbUrl,dbUserName,dbUserPassword,"dwd_register_record_info",map2));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }

    private List<Map<String,Object>> getDataList(HdsContext context,List<RegisterRecordsHBO> readDataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();



        if("ods_register_record_info".equals(config.getKey())){
            for(RegisterRecordsHBO readData:readDataList){
                Map<String,Object> writeObjMap=this.getWriteMapOds(readData,mapping);
                data.add(writeObjMap);
            }
        }else if("dwd_register_record_info".equals(config.getKey())){
            for(RegisterRecordsHBO readData:readDataList){
                Map<String,Object> writeObjMap=this.getWriteMapDwd(readData,mapping);
                if(writeObjMap!=null){
                    data.add(writeObjMap);
                    if(context.isCleanMqFlag()){
                        context.addCleanObjId(MQCleanTableKeyEnum.Register.getCode(),readData.getVisitId());
                    }
                }


            }
        }
        return data;
    }


    private Map<String,Object> getWriteMapOds(RegisterRecordsHBO dataItem, Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        mapping.forEach((dp,ep)->{
            writeMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }


    private Map<String,Object> getWriteMapDwd(RegisterRecordsHBO dataItem, Map<String,String> mapping){
        Map<String,Object> writeMap=new HashMap<>();
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        mapping.forEach((dp,ep)->{
            writeMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }





}
