package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.OutPatientRecordsHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * (门诊/急诊)就诊记录同步
 * */
@Service
public class SyncDataVisitRecordService {


    public List<OutPatientRecordsHBO> getDataList(HdsContext context,HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig,
                                                  String startDate, String endDate , DataSyncLogSaveDTO log,
                                                  Map<String,List<Map<String,Object>>> writeDataMap, boolean logFlag, String logBatchNo){

        BwwbSyncDataRespHBO<OutPatientRecordsHBO> readData = BwwbSyncDataService.getOutPatientRecord(readConfig,startDate,endDate,logFlag,logBatchNo);
        log.setParams(startDate+";"+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }

        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }


    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        //ods
        Map<String,String> map=new LinkedHashMap<>();
        map.put("id","opid"); // 就诊号
        map.put("register_id","visitId"); // 挂号流水号
        map.put("patient_id","patientId"); // 患者编号
        map.put("patient_name","patientName"); //患者姓名
        map.put("patient_age","age");  //患者年龄
        map.put("patient_sex","sex");  //患者性别
        map.put("patient_card_no","idCard"); //身份证号

        map.put("visit_date","treatDateTime"); //就诊时间

        map.put("company_code","companyNo");  //医院
        map.put("company_name","companyName"); //医院

        map.put("department_code","departmentCode"); //科室
        map.put("department_name","departmentName"); //科室

        map.put("doctor_code","doctorCode"); //医生
        map.put("doctor_name","doctorName"); //医生

        map.put("status_name","statusName"); //状态
        map.put("status_code","status"); //状态

        map.put("scene_type","patientType"); //患者类型：1.门诊；2，急诊
        map.put("fee_type_code","feeTypeCode"); //收费分类编号
        map.put("fee_type_name","feeTypeName"); //收费分类名称

        map.put("data_update_date","updateTime");
        map.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_visit_record_info",dbUrl,dbUserName,dbUserPassword,"ods_visit_record_info",map));


        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("patient_id", "patientId");
        map2.put("id", "opid");
        map2.put("register_id", "visitId");
        map2.put("patient_name", "patientName");
        map2.put("patient_gender_raw", "sex");
        map2.put("patient_age", "age");
        map2.put("company_name", "companyName");
        map2.put("company_code", "companyNo");

        map2.put("department_name_raw", "departmentName");
        map2.put("department_name_standard", "departmentName");
        map2.put("department_code_raw", "departmentCode");
        map2.put("department_code_standard", "departmentCode");

        map2.put("doctor_code", "doctorCode");
        map2.put("doctor_name", "doctorName");
        map2.put("visit_date", "treatDateTime");
//        map2.put("visit_type_raw", "visit_type");
//        map2.put("visit_way_raw", "visit_way");//就诊方式
        map2.put("scene_type_raw", "patientType"); //场景分类
        map2.put("status_raw", "status");//状态
        map2.put("status_name_raw", "statusName");
        map2.put("data_update_date", "updateTime");
        map2.put("op_date", SyncDataKeyEnum.SDKCV_DateNow.getCode());
        map2.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());

        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_visit_record_info",dbUrl,dbUserName,dbUserPassword,"dwd_visit_record_info",map2));


        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);

        return config;
    }



    private List<Map<String,Object>> getDataList(HdsContext context,List<OutPatientRecordsHBO> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_visit_record_info".equals(config.getKey())){
            for(OutPatientRecordsHBO dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMapOds(dataItem,mapping);
                data.add(writeItemMap);
            }
        }else if("dwd_visit_record_info".equals(config.getKey())){
            for(OutPatientRecordsHBO dataItem:dataList){
                //
                if(context.isCleanMqFlag()){
                    context.addCleanObjId(MQCleanTableKeyEnum.Visit.getCode(),dataItem.getOpid());
                }

                Map<String,Object>  writeItemMap=this.getWriteMapDwd(dataItem,mapping);
                data.add(writeItemMap);
            }
        }

        return data;
    }


    private  Map<String,Object> getWriteMapOds(OutPatientRecordsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }

    private  Map<String,Object> getWriteMapDwd(OutPatientRecordsHBO dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeObjMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeObjMap.put(dp,HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeObjMap;
    }




}
