package com.xxh.cloud.sync.framework.modules.biz.data;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.bean.pe.SlifeTjPatientInfo;
import com.xxh.cloud.sync.framework.modules.read.api.bean.pe.SlifeTjRegister;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.DiagnosisHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.PhysicalExaminationSyncService;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class TjSyncPatientService {


    public List<SlifeTjPatientInfo> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                                DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                                boolean logFlag, String logBatchNo){



        BwwbSyncDataRespHBO<SlifeTjPatientInfo> readData =  PhysicalExaminationSyncService.get_TjPatientInfo(readConfig,startDate,endDate);

        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }
        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }



    private List<Map<String,Object>> getDataList(HdsContext context,List<SlifeTjPatientInfo> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();

        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_examination_patient_info".equals(config.getKey())){
            for(SlifeTjPatientInfo dataItem:dataList){
                Map<String,Object> writeItemMap=this.getWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }

        if("dwd_examination_patient_info".equals(config.getKey())){
            for(SlifeTjPatientInfo dataItem:dataList){
                if(context.isCleanMqFlag()){
                    context.addCleanObjId(MQCleanTableKeyEnum.ExaminationPatient.getCode(),dataItem.getVisitId());
                }
                Map<String,Object> writeItemMap=this.getWriteMap_Dwd(dataItem,mapping);
                data.add(writeItemMap);
            }
        }

        return data;
    }

    private  Map<String,Object> getWriteMap_Ods(SlifeTjPatientInfo dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }


    private  Map<String,Object> getWriteMap_Dwd(SlifeTjPatientInfo dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }



    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic){
        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        //ods
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("visit_id","visitId"); //
        map1.put("patient_name","patientName"); //
        map1.put("sex","sex"); //
        map1.put("birth_day","birthDay"); //
        map1.put("id_card","idCard"); //
        map1.put("data_create_date","createTime"); //
        map1.put("data_update_date","updateTime"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_examination_patient_info",dbUrl,dbUserName,dbUserPassword,"ods_examination_patient_info",map1));

        //dwd
        Map<String,String> map2=new LinkedHashMap<>();
        map2.put("visit_id","visitId"); //
        map2.put("patient_name","patientName"); //
        map2.put("sex","sex"); //
        map2.put("birth_day","birthDay");
        map2.put("id_card","idCard");
        map2.put("data_create_date","createTime");
        map2.put("data_update_date","updateTime");
        map2.put("main_person_id",HospitalSyncDateUtils.getDefaultValue("0"));
        map2.put("op_data_user",SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map2.put("op_date",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_examination_patient_info",dbUrl,dbUserName,dbUserPassword,"dwd_examination_patient_info",map2));

        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);

        return config;
    }




}
