package com.xxh.cloud.sync.framework.modules.biz.data;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.constant.MQCleanTableKeyEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncDataKeyEnum;
import com.xxh.cloud.etl.common.framework.modules.bean.ods.OperationDiagnoseBO;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.utils.HospitalSyncDateConvert;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.read.api.bean.pe.SlifeTjFinalReportResult;
import com.xxh.cloud.sync.framework.modules.read.api.bean.pe.SlifeTjSuggest;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.resp.OperationRecordHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbSyncDataService;
import com.xxh.cloud.sync.framework.modules.read.api.service.PhysicalExaminationSyncService;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class TjSyncSuggestService {



    public List<SlifeTjSuggest> getDataList(HdsContext context, HwdsReadConfigBean readConfig, HwdsWriteConfigBean writerConfig, String startDate, String endDate ,
                                            DataSyncLogSaveDTO log, Map<String,List<Map<String,Object>>> writeDataMap,
                                            boolean logFlag, String logBatchNo){

        BwwbSyncDataRespHBO<SlifeTjSuggest> readData =  PhysicalExaminationSyncService.get_TjSuggest(readConfig,startDate,endDate);

        log.setParams(startDate+","+endDate);
        log.setReadStatus(readData.getStatus());

        if(YN.NO.getCode().equals(readData.getStatus())){
            log.setStatus(YN.NO.getCode());
            log.setEndDate(new Date());
            log.setRemark("读取数据错误 : "+readData.getMsg());
            return null;
        }

        if(CollUtil.isEmpty(readData.getData())){
            log.setReadNum(0);
            log.setStatus(YN.YES.getCode());
            log.setEndDate(new Date());
            log.setRemark("无读取数据");
            return null;
        }
        log.setReadNum(readData.getData().size());

        for(HwdsWriteDBConfigBean writeDBConfig:writerConfig.getDbConfigs()){
            List<Map<String,Object>>  writeDataList = this.getDataList(context,readData.getData(),writeDBConfig);
            writeDataMap.put(writeDBConfig.getKey(),writeDataList);
        }

        return readData.getData();
    }



    private List<Map<String,Object>> getDataList(HdsContext context,List<SlifeTjSuggest> dataList , HwdsWriteDBConfigBean config){
        List<Map<String,Object>> data=new ArrayList<>();
        Map<String,String> mapping= config.columnEntityMapping();

        if("ods_examination_problem_suggest_info".equals(config.getKey())){
            for(SlifeTjSuggest dataItem:dataList){
                Map<String,Object>  writeItemMap=this.getWriteMap_Ods(dataItem,mapping);
                data.add(writeItemMap);
            }
        }


        return data;
    }


    private  Map<String,Object> getWriteMap_Ods(SlifeTjSuggest dataItem, Map<String,String> mapping){
        Map<String, Object> readObjMap= BeanUtil.beanToMap(dataItem);
        Map<String,Object> writeMap=new HashMap<>();
        mapping.forEach((dp,ep)->{
            writeMap.put(dp, HospitalSyncDateUtils.getMapVal(ep,readObjMap));
        });
        return writeMap;
    }


    public HwdsWriteConfigBean getWriteConfig(String dbType,String dbUrl,String dbUserName,String dbUserPassword ,String mqType ,String mqTopic) {

        List<HwdsWriteDBConfigBean> dbConfigs=new ArrayList<>();

        //ods
        Map<String,String> map1=new LinkedHashMap<>();
        map1.put("id","id"); //关联id 【主键】
        map1.put("problem","exceptName"); //
        map1.put("suggest","suggest"); //
        map1.put("op_data_user", SyncDataKeyEnum.SDKCV_UserNow.getCode());
        map1.put("op_update_data",SyncDataKeyEnum.SDKCV_DateNow.getCode());
        dbConfigs.add(new HwdsWriteDBConfigBean("ods_examination_problem_suggest_info",dbUrl,dbUserName,dbUserPassword,"ods_examination_problem_suggest_info",map1));


        //dwd
//        Map<String,String> map2=new LinkedHashMap<>();
//        map2.put("patient_id","patientId"); //患者编号
//        map2.put("id","id"); //关联id 【主键】
//        map2.put("visit_id","visitId"); //就诊记录号
//        dbConfigs.add(new HwdsWriteDBConfigBean("dwd_examination_problem_suggest_info",dbUrl,dbUserName,dbUserPassword,"dwd_examination_problem_suggest_info",map2));


        HwdsWriteConfigBean config = HospitalSyncDateUtils.getHwdsWriteConfigBean(dbType,mqType,mqTopic,dbConfigs);
        return config;
    }




}
