package com.xxh.cloud.sync.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.bean.MQCleanTableIds;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.etl.common.framework.modules.config.ProjectETLConfig;
import com.xxh.cloud.etl.common.framework.modules.write.bean.bos.HwdsWriteConfigBean;
import com.xxh.cloud.framework.activemq.constant.ActiveMQWayEnum;
import com.xxh.cloud.framework.activemq.utils.ActiveMQSendUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import com.xxh.cloud.framework.common.write.doris.DorisWriter;
import com.xxh.cloud.framework.common.write2.JdbcBatchWriteDTO;
import com.xxh.cloud.sync.framework.constant.SyncApiTypeEnum;
import com.xxh.cloud.etl.common.framework.constant.SyncConfigTypeEnum;
import com.xxh.cloud.sync.framework.modules.base.entity.DataSyncFlagInfo;
import com.xxh.cloud.sync.framework.modules.biz.bean.bos.HdsContext;
import com.xxh.cloud.sync.framework.modules.biz.bean.dto.HospitalSyncDataByStartEndDTO;
import com.xxh.cloud.sync.framework.modules.biz.bean.dto.HospitalSyncDataDTO;
import com.xxh.cloud.sync.framework.modules.biz.data.*;
import com.xxh.cloud.sync.framework.modules.biz.service.HospitalDataSyncBizService;
import com.xxh.cloud.etl.common.framework.utils.HospitalSyncDateUtils;
import com.xxh.cloud.sync.framework.modules.biz.service.HospitalJDBCBizService;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogSaveDTO;
import com.xxh.cloud.sync.framework.modules.config.bean.dto.DataSyncLogItemInfoAddDTO;
import com.xxh.cloud.sync.framework.modules.config.service.DataSyncFlagBizService;
import com.xxh.cloud.sync.framework.modules.config.service.DataSyncLogBizService;
import com.xxh.cloud.sync.framework.modules.read.api.bean.pe.*;
import com.xxh.cloud.sync.framework.modules.read.api.resp.*;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Log4j2
@Service
public class HospitalDataSyncBizServiceImpl implements HospitalDataSyncBizService {


    @Autowired
    private DataSyncFlagBizService dataSyncFlagBizService;

    @Autowired
    private DataSyncLogBizService dataSyncLogBizService;

    @Autowired
    private SyncDataPatientInfoService syncDataPatientInfoService;

    @Autowired
    private SyncDataPatientRegisterService syncDataPatientRegisterService;


    @Autowired
    private SyncDataVisitRecordService syncDataVisitRecordService;
    @Autowired
    private SyncDataInRecordService syncDataInRecordService;

    @Autowired
    private SyncDataInChangeService syncDataInChangeService;

    @Autowired
    private SyncDataInAppointmentService syncDataInAppointmentService;
    @Autowired
    private SyncDataOutSummaryService syncDataOutSummaryService;

    @Autowired
    private SyncDataExamApplyService syncDataExamApplyService;
    @Autowired
    private SyncDataExamAppointmentService syncDataExamAppointmentService;
    @Autowired
    private SyncDataExamReportService syncDataExamReportService;

    @Autowired
    private SyncDataLaboratoryApplyService syncDataLaboratoryApplyService;

    @Autowired
    private SyncDataLaboratoryReportService syncDataLaboratoryReportService;

    @Autowired
    private SyncDataAdviceMedicationService syncDataAdviceMedicationService;

    @Autowired
    private SyncDataAdviceTherapyService syncDataAdviceTherapyService;

    @Autowired
    private SyncDataOperationApplyService syncDataOperationApplyService;

    @Autowired
    private SyncDataOperationRecordService syncDataOperationRecordService;

    @Autowired
    private SyncDataDiagnosisService syncDataDiagnosisService;

    @Autowired
    private SyncDataPatientFeeService syncDataPatientFeeService;


    @Autowired
    private SyncDataMedicalRecordService syncDataMedicalRecordService;



    @Autowired
    private SyncDataDoctorAdviceService syncDataDoctorAdviceService;

    @Autowired
    private SyncDataPastHisService syncDataPastHisService;

    @Autowired
    private SyncDataFirstCourseIllnessService syncDataFirstCourseIllnessService;


//    @Autowired
//    private OdsExamineApplyCacheService odsExamineApplyCacheService;

    @Autowired
    private ProjectETLConfig projectETLConfig;

    @Autowired
    private HospitalJDBCBizService hospitalJDBCBizService;

    private HwdsReadConfigBean getReadConfig(String content){
        HwdsReadConfigBean config=new HwdsReadConfigBean();
        config.setDomain(projectETLConfig.getSyncDomain());
        config.setAppid(projectETLConfig.getSyncAppId());
        config.setSecret(projectETLConfig.getSyncSecret());
        return config;
    }



    /**
     *
     * */
    public void syncHospitalData2(HospitalSyncDataDTO params){
        DataSyncFlagInfo dataSyncConfig = dataSyncFlagBizService.getEntityByCode(params.getType().getCode());
        if(dataSyncConfig == null){
            log.debug("++++ 无同步配置信息,{}",params.getType().getCode());
            return;
        }

        String endDate= HospitalSyncDateUtils.getEnd(dataSyncConfig.getFlag(),dataSyncConfig.getSyncMaxMinute());
        if(StrUtil.isEmpty(endDate)){
            log.info("+++++ 不需要数据同步,{}",params.getType().getCode());
            return;
        }


        //config
        HwdsReadConfigBean readConfig = this.getReadConfig(dataSyncConfig.getReadConfig());
        HwdsWriteConfigBean writerConfig = JSON.parseObject(dataSyncConfig.getWriteConfig(), HwdsWriteConfigBean.class);

        this._syncData(params.getType(), readConfig, writerConfig, dataSyncConfig.getFlag(), endDate, params.isCleanMqFlag(),
                params.isLogFlag(), params.isRuleMqFlag(), IdUtil.fastSimpleUUID(), true);

    }

    @Autowired
    private TjSyncPatientService tjSyncPatientService;
    @Autowired
    private TjSyncRegisterService tjSyncRegisterService;
    @Autowired
    private TjSyncReportService tjSyncReportService;
    @Autowired
    private TjSyncFinalReportService tjSyncFinalReportService;

    @Autowired
    private TjSyncSuggestService tjSyncSuggestService;

    @Autowired
    private SyncActivityExamReportService syncActivityExamReportService;



    @Override
    public void syncHospitalData(HospitalSyncDataDTO params) {
        //todo:不执行规则
        //params.setRuleMqFlag(false);

        String logBatchNo= "SYNC"+IdUtil.getSnowflake(1,1).nextIdStr();

        DataSyncFlagInfo dataSyncConfig = dataSyncFlagBizService.getEntityByCode(params.getType().getCode());
        if(dataSyncConfig == null){
            log.debug("++++ 无同步配置信息,{}",params.getType().getCode());
            return;
        }

        String endDate= HospitalSyncDateUtils.getEnd(dataSyncConfig.getFlag(),dataSyncConfig.getSyncMaxMinute());
        if(StrUtil.isEmpty(endDate)){
            log.info("+++++ 不需要数据同步,{}",params.getType().getCode());
            return;
        }



        //config
        HwdsReadConfigBean readConfig = this.getReadConfig(dataSyncConfig.getReadConfig());
        HwdsWriteConfigBean writerConfig= JSON.parseObject(dataSyncConfig.getWriteConfig(), HwdsWriteConfigBean.class);

        DataSyncLogSaveDTO bizLog=new DataSyncLogSaveDTO();
        bizLog.setCode(params.getType().getCode());
        bizLog.setName(params.getType().getName());
        bizLog.setStartDate(new Date());


        Map<String,List<Map<String,Object>>> writeDataMap=new HashMap<>();

        HdsContext context=new HdsContext();
        context.setCleanMqFlag(params.isCleanMqFlag());


        //患者基本信息
        if(SyncApiTypeEnum.SAPI_PatientInfo == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),10);
            List<PatientRecordHBO> data= syncDataPatientInfoService.getDataList(context,readConfig,writerConfig,startDate,
                    endDate,bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                log.debug("++++++ 无读取数据:患者基本信息");
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //挂号记录同步
        if(SyncApiTypeEnum.SAPI_PatientRegister == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),10);
            List<RegisterRecordsHBO> data= syncDataPatientRegisterService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //(门诊/急诊)就诊记录同步
        if(SyncApiTypeEnum.SAPI_VisitRecord == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),10);
            List<OutPatientRecordsHBO> data= syncDataVisitRecordService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog, writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //预约住院信息记录
        if(SyncApiTypeEnum.SAPI_InAppointment == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),10);
            List<RecipeInpatientHBO> data= syncDataInAppointmentService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }


        //住院记录同步
        if(SyncApiTypeEnum.SAPI_InRecord == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),10);
            List<InPatientRecordsHBO> data= syncDataInRecordService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //住院变更记录同步
        if(SyncApiTypeEnum.SAPI_InChange == params.getType()){
            List<SlifePatientChangeHBO> data= syncDataInChangeService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }


        //出院小结记录
        if(SyncApiTypeEnum.SAPI_OutSummary == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),15);
            List<OutRecipeRecordsHBO> data= syncDataOutSummaryService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //检查申请
        if(SyncApiTypeEnum.SAPI_ExamApply == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),10);
            List<CheckApplyHBO> data= syncDataExamApplyService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
//            odsExamineApplyCacheService.addListByCache(data);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }


        //检查预约
        if(SyncApiTypeEnum.SAPI_ExamAppointment == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),10);
            List<CheckAppointmentHBO> data= syncDataExamAppointmentService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }


        //检查报告单数据
        if(SyncApiTypeEnum.SAPI_ExamReport == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),3);
            List<ExamineReportsHBO> data= syncDataExamReportService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //检验申请
        if(SyncApiTypeEnum.SAPI_LaboratoryApply == params.getType()){
            List<LaboratoryApplyBO> data= syncDataLaboratoryApplyService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //检验报告单
        if(SyncApiTypeEnum.SAPI_LaboratoryReport == params.getType()){
            String startDate=HospitalSyncDateUtils.getStart(dataSyncConfig.getFlag(),3);
            List<LaboratoryReportHBO> data= syncDataLaboratoryReportService.getDataList(context,readConfig,writerConfig,startDate,endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //医嘱用药数据
        if(SyncApiTypeEnum.SAPI_AdviceMedication == params.getType()){
            List<RecipeMedicineHBO> data= syncDataAdviceMedicationService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //医嘱治疗数据
        if(SyncApiTypeEnum.SAPI_AdviceTherapy == params.getType()){
            List<RecipeTherapyHBO> data= syncDataAdviceTherapyService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //医嘱手术申请数据
        if(SyncApiTypeEnum.SAPI_OperationApply == params.getType()){
            List<OperationApplyHBO> data= syncDataOperationApplyService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }


        //手术记录数据
        if(SyncApiTypeEnum.SAPI_OperationRecord == params.getType()){
            List<OperationRecordHBO> data= syncDataOperationRecordService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //诊断数据
        if(SyncApiTypeEnum.SAPI_Diagnosis == params.getType()){
            List<DiagnosisHBO> data= syncDataDiagnosisService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }


        //体检:体检患者
        if(SyncApiTypeEnum.SAPI_TJ_Patient == params.getType()){
            List<SlifeTjPatientInfo> data= tjSyncPatientService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //体检:体检注册
        if(SyncApiTypeEnum.SAPI_TJ_Register == params.getType()){
            List<SlifeTjRegister> data= tjSyncRegisterService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //体检:单项报告
        if(SyncApiTypeEnum.SAPI_TJ_Report == params.getType()){
            List<SlifeTjReportResult> data= tjSyncReportService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }
        //体检:总检报告
        if(SyncApiTypeEnum.SAPI_TJ_FinalReport == params.getType()){
            List<SlifeTjFinalReportResult> data= tjSyncFinalReportService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //体检:体检建议
        if(SyncApiTypeEnum.SAPI_TJ_Suggest == params.getType()){
            List<SlifeTjSuggest> data= tjSyncSuggestService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }


        //活动:
        if(SyncApiTypeEnum.SAPI_ActivityExamReport == params.getType()){
            List<ExamineReportsHBO> data= syncActivityExamReportService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //医嘱数据
        if(SyncApiTypeEnum.SAPI_DoctorAdvice == params.getType()){
            List<RecipeItemsHBO> data= syncDataDoctorAdviceService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //既往史数据
        if(SyncApiTypeEnum.SAPI_PastHis == params.getType()){
            List<SlifeDocumentOutPatientHBO> data= syncDataPastHisService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //首次病程
        if(SyncApiTypeEnum.SAPI_FirstCourseIllness == params.getType()){
            List<SlifeDocumentProgressNoteHBO> data= syncDataFirstCourseIllnessService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //收费信息
        if(SyncApiTypeEnum.SAPI_PatientFee == params.getType()){
            List<SlifeFeeDetailHBO> data= syncDataPatientFeeService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }

        //电子病例
        if(SyncApiTypeEnum.SAPI_MedicalRecord == params.getType()){
            List<ElectronMedicalRecordHBO> data= syncDataMedicalRecordService.getDataList(context,readConfig,writerConfig,dataSyncConfig.getFlag(),endDate,
                    bizLog,writeDataMap,params.isLogFlag(),logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                dataSyncFlagBizService.update(params.getType().getCode(),endDate);
                return;
            }
        }



        //todo:未同步
        //医嘱数据
        if(SyncApiTypeEnum.SAPI_DoctorAdvice==params.getType()){
//            HwdsWriteConfigBean config =syncDataDoctorAdviceService.getWriteConfig(dbType.getCode(),dbUrl,dbUserName,dbUserPassword,mqType,mqTopic);
//            saveEntity.setWriteConfig(JSON.toJSONString(config));
//            dataSyncFlagInfoService.save(saveEntity);
        }


        //Doris
        if(SyncConfigTypeEnum.SCT_Doris.getCode().equals(writerConfig.getDbType())){

            for(HwdsWriteDBConfigBean writeDBConfig: writerConfig.getDbConfigs()){

                writeDBConfig.setUrl(projectETLConfig.getDbUrl());
                writeDBConfig.setUsername(projectETLConfig.getDbUserName());
                writeDBConfig.setPassword(projectETLConfig.getDbUserPassword());

                List<Map<String,Object>> writeDataList= writeDataMap.get(writeDBConfig.getKey());
                if(CollUtil.isEmpty(writeDataList)){
                    bizLog.setStatus(YN.YES.getCode());
                    bizLog.setEndDate(new Date());
                    continue;
                }


                DataSyncLogItemInfoAddDTO logItemAddDTO=new DataSyncLogItemInfoAddDTO();
                logItemAddDTO.setStartDate(new Date());
                logItemAddDTO.setWriteTable(writeDBConfig.getTable());
                logItemAddDTO.setWriteNum(writeDataList.size());

                DorisWriter dorisWriter=new DorisWriter();

                JdbcBatchWriteDTO writerResult= hospitalJDBCBizService.batchWrite(writeDBConfig,writeDataList);
                //todo:wjj
//                DbTaskResult writerResult= dorisWriter.writer(writeDBConfig,writeDataList,params.isLogFlag(),logBatchNo);

                if(writerResult.isStatus()){
                    logItemAddDTO.setWriteStatus(YN.YES.getCode());
                    logItemAddDTO.setEndDate(new Date());
                    bizLog.addLogItem(logItemAddDTO);

                    bizLog.setStatus(YN.YES.getCode());
                    bizLog.setEndDate(new Date());
                }else {
                    logItemAddDTO.setWriteStatus(YN.NO.getCode());
                    logItemAddDTO.setWriteErrInfo(writerResult.getErrorInfo());
                    logItemAddDTO.setEndDate(new Date());
                    bizLog.addLogItem(logItemAddDTO);

                    bizLog.setStatus(YN.NO.getCode());
                    bizLog.setEndDate(new Date());
                }

            }
        }

        //
        if(context.isCleanMqFlag()){
            log.debug("+++++准备发送清洗数据MQ");
            this.sendCleantQueue(params.isRuleMqFlag(), context.getCleanObjIdsMap());
        }

        dataSyncLogBizService.save(bizLog);
        dataSyncFlagBizService.update(params.getType().getCode(),endDate);
    }





    private void _syncData(SyncApiTypeEnum type ,HwdsReadConfigBean readConfig,HwdsWriteConfigBean writerConfig,
                          String startDate ,String endDate,
                          boolean cleanMqFlag,boolean logFlag,boolean ruleMqFlag,String logBatchNo,boolean isUpdateFlag){

        DataSyncLogSaveDTO bizLog=new DataSyncLogSaveDTO();
        bizLog.setCode(type.getCode());
        bizLog.setName(type.getName());
        bizLog.setStartDate(new Date());

        Map<String,List<Map<String,Object>>> writeDataMap=new HashMap<>();

        HdsContext context=new HdsContext();
        context.setCleanMqFlag(cleanMqFlag);



        //患者基本信息
        if(SyncApiTypeEnum.SAPI_PatientInfo == type){
            List<PatientRecordHBO> data= syncDataPatientInfoService.getDataList(context,readConfig,writerConfig,
                    startDate, endDate,bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                log.debug("++++++ 无读取数据:患者基本信息");
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //挂号记录同步
        if(SyncApiTypeEnum.SAPI_PatientRegister == type){
            List<RegisterRecordsHBO> data= syncDataPatientRegisterService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //(门诊/急诊)就诊记录同步
        if(SyncApiTypeEnum.SAPI_VisitRecord == type){
            List<OutPatientRecordsHBO> data= syncDataVisitRecordService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog, writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //预约住院信息记录
        if(SyncApiTypeEnum.SAPI_InAppointment == type){
            List<RecipeInpatientHBO> data= syncDataInAppointmentService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }


        //住院记录同步
        if(SyncApiTypeEnum.SAPI_InRecord == type){
            List<InPatientRecordsHBO> data= syncDataInRecordService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }


        //出院小结记录
        if(SyncApiTypeEnum.SAPI_OutSummary == type){
            List<OutRecipeRecordsHBO> data= syncDataOutSummaryService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //检查申请
        if(SyncApiTypeEnum.SAPI_ExamApply == type){
            List<CheckApplyHBO> data= syncDataExamApplyService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }


        //检查预约
        if(SyncApiTypeEnum.SAPI_ExamAppointment == type){
            List<CheckAppointmentHBO> data= syncDataExamAppointmentService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }


        //检查报告单数据
        if(SyncApiTypeEnum.SAPI_ExamReport == type){
            List<ExamineReportsHBO> data= syncDataExamReportService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //检验申请
        if(SyncApiTypeEnum.SAPI_LaboratoryApply == type){
            List<LaboratoryApplyBO> data= syncDataLaboratoryApplyService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //检验报告单
        if(SyncApiTypeEnum.SAPI_LaboratoryReport == type){
            List<LaboratoryReportHBO> data= syncDataLaboratoryReportService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //医嘱用药数据
        if(SyncApiTypeEnum.SAPI_AdviceMedication == type){
            List<RecipeMedicineHBO> data= syncDataAdviceMedicationService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //医嘱治疗数据
        if(SyncApiTypeEnum.SAPI_AdviceTherapy == type){
            List<RecipeTherapyHBO> data= syncDataAdviceTherapyService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //医嘱手术申请数据
        if(SyncApiTypeEnum.SAPI_OperationApply == type){
            List<OperationApplyHBO> data= syncDataOperationApplyService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }


        //手术记录数据
        if(SyncApiTypeEnum.SAPI_OperationRecord == type){
            List<OperationRecordHBO> data= syncDataOperationRecordService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }

        //诊断数据
        if(SyncApiTypeEnum.SAPI_Diagnosis == type){
            List<DiagnosisHBO> data= syncDataDiagnosisService.getDataList(context,readConfig,writerConfig,
                    startDate,endDate, bizLog,writeDataMap,logFlag,logBatchNo);
            if(data==null){
                dataSyncLogBizService.save(bizLog);
                if(isUpdateFlag){
                    dataSyncFlagBizService.update(type.getCode(),endDate);
                }
                return;
            }
        }


        //Doris
        if(SyncConfigTypeEnum.SCT_Doris.getCode().equals(writerConfig.getDbType())){

            for(HwdsWriteDBConfigBean writeDBConfig: writerConfig.getDbConfigs()){
                writeDBConfig.setUrl(projectETLConfig.getDbUrl());
                writeDBConfig.setUsername(projectETLConfig.getDbUserName());
                writeDBConfig.setPassword(projectETLConfig.getDbUserPassword());

                List<Map<String,Object>> writeDataList= writeDataMap.get(writeDBConfig.getKey());
                if(CollUtil.isEmpty(writeDataList)){
                    bizLog.setStatus(YN.YES.getCode());
                    bizLog.setEndDate(new Date());
                    continue;
                }

                DataSyncLogItemInfoAddDTO logItemAddDTO=new DataSyncLogItemInfoAddDTO();
                logItemAddDTO.setStartDate(new Date());
                logItemAddDTO.setWriteTable(writeDBConfig.getTable());
                logItemAddDTO.setWriteNum(writeDataList.size());

                DorisWriter dorisWriter=new DorisWriter();


                JdbcBatchWriteDTO writerResult= hospitalJDBCBizService.batchWrite(writeDBConfig,writeDataList);
                //todo:wjj
                //DbTaskResult writerResult= dorisWriter.writer(writeDBConfig,writeDataList,logFlag,logBatchNo);


                if(writerResult.isStatus()){
                    logItemAddDTO.setWriteStatus(YN.YES.getCode());
                    logItemAddDTO.setEndDate(new Date());

                    bizLog.addLogItem(logItemAddDTO);
                    bizLog.setStatus(YN.YES.getCode());
                    bizLog.setEndDate(new Date());
                }else {
                    logItemAddDTO.setWriteStatus(YN.NO.getCode());
                    logItemAddDTO.setWriteErrInfo(writerResult.getErrorInfo());
                    logItemAddDTO.setEndDate(new Date());

                    bizLog.addLogItem(logItemAddDTO);
                    bizLog.setStatus(YN.NO.getCode());
                    bizLog.setEndDate(new Date());
                }

            }
        }

        //
        if(context.isCleanMqFlag()){
            this.sendCleantQueue(ruleMqFlag, context.getCleanObjIdsMap());
        }
        //
        dataSyncLogBizService.save(bizLog);
        //
        if(isUpdateFlag){
            dataSyncFlagBizService.update(type.getCode(),endDate);
        }
    }




    /**
     *
     * */
    @Override
    public void syncHospitalDataByStartEnd(HospitalSyncDataByStartEndDTO params) {
        DataSyncFlagInfo dataSyncConfig = dataSyncFlagBizService.getEntityByCode(params.getType().getCode());
        if (dataSyncConfig == null) {
            log.debug("++++ 无同步配置信息,{}", params.getType().getCode());
            return;
        }

        //config
        HwdsReadConfigBean readConfig = this.getReadConfig(dataSyncConfig.getReadConfig());
        HwdsWriteConfigBean writerConfig = JSON.parseObject(dataSyncConfig.getWriteConfig(), HwdsWriteConfigBean.class);

        this._syncData(params.getType(), readConfig, writerConfig, params.getStartDate(), params.getEndDate(), params.isCleanMqFlag(),
                params.isLogFlag(), params.isRuleMqFlag(), IdUtil.fastSimpleUUID(), false);

    }


    private void sendCleantQueue(boolean ruleFlag, Map<String,List<String>> cleanObjIdsMap){
        if(MapUtil.isEmpty(cleanObjIdsMap)){
            return;
        }
        log.debug("+++++ 数据清洗发送启动:");
        cleanObjIdsMap.forEach((key,list)->{
            sendCleantQueue(ruleFlag,key,list);
        });
    }

    private void sendCleantQueue(boolean ruleFlag,String key,List<String> ids){
        if(CollUtil.isEmpty(ids)){
            return;
        }
        log.debug("++++++ 发送清洗MQ: key ={} ; 内容={}",key,CollUtil.join(ids,","));
        List<List<String>> lists= ListUtil.split(ids,1000);
        for(List<String> objIds:lists){
            MQCleanTableIds cleanTableIds=new MQCleanTableIds(key,objIds);
            if(ruleFlag){
                log.debug("+++++++++ 发送清洗MQ 1: 需要调用引擎的清洗数据");
                ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName.Que_CleanTableId,
                        JSON.toJSONString(cleanTableIds));
            }else {
                log.debug("+++++++++ 发送清洗MQ 2: 不需要调用引擎的清洗数据");
                ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName.Que_CleanTableIdNot,
                        JSON.toJSONString(cleanTableIds));
            }

        }
    }


}
