package com.xxh.cloud.sync.framework.modules.task;

import cn.hutool.core.date.DateUtil;
import com.xxh.cloud.etl.common.framework.modules.config.ProjectETLConfig;
import com.xxh.cloud.sync.framework.constant.SyncApiTypeEnum;
import com.xxh.cloud.sync.framework.modules.biz.bean.dto.HospitalSyncDataDTO;
import com.xxh.cloud.sync.framework.modules.biz.service.HospitalDataSyncBizService;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

@Log4j2
@Service
public class SyncDataTask {

    @Autowired
    private HospitalDataSyncBizService hospitalDataSyncBizService;


    @Autowired
    private ProjectETLConfig projectETLConfig;

    /**
     * 患者基本信息
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    @XxlJob("syncPatient")
    public void syncPatient() {
        log.info("+++++ 患者基本信息:{}",DateUtil.now());
        if (projectETLConfig.isSyncTask()) {
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_PatientInfo);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }

    /**
     * 就诊记录
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncVisit")
    public void syncVisit() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync VisitRecord : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_VisitRecord);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }

    /**
     * 挂号记录
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncRegister")
    public void register() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync RegisterRecord : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_PatientRegister);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }




    /**
     * 预约住院信息记录
     */
    @XxlJob("syncInAppointment")
    @Scheduled(cron = "0 0/2 * * * ?")
    public void syncInAppointment() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync In Appointment : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_InAppointment);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 住院记录同步
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncIn")
    public void syncIn() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync In  : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_InRecord);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncInChange")
    public void syncInChange() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync In change  : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_InChange);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 出院小结记录
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncOut")
    public void syncOut() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync Out  : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_OutSummary);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }



    /**
     * 检查申请
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncExamApply")
    public void syncExamApply() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync ExamApply : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_ExamApply);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     *  检查预约
     * */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncExamAppointment")
    public void syncExamAppointment() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync ExamAppointment : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_ExamAppointment);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }



    /**
     * 检查报告单数据
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncExamReport")
    public void syncExamReport() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync ExamReport : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_ExamReport);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 检验申请
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncLaboratoryApply")
    public void syncLaboratoryApply() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync LaboratoryApply : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_LaboratoryApply);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 检验报告单
     */
    @Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncLaboratoryReport")
    public void syncLaboratoryReport() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync LaboratoryReport : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_LaboratoryReport);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     *  医嘱用药数据
     * */
    //@Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncMedication")
    public void syncMedication() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync AdviceMedication : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_AdviceMedication);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }



    /**
     * 医嘱治疗数据
     * */
    //@Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncTherapy")
    public void syncTherapy() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync AdviceTherapy : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_AdviceTherapy);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 医嘱手术申请数据
     * */
    //@Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncOpApply")
    public void syncOpApply() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync OperationApply : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_OperationApply);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 手术记录数据
     * */
    //@Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("syncOpRecord")
    public void syncOpRecord() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync OperationRecord : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_OperationRecord);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 诊断
     * */
    @XxlJob("syncDiagnosis")
    public void syncDiagnosis() {
        log.debug(" +++++++++ Sync Diagnosis : " + DateUtil.now());
        HospitalSyncDataDTO params = new HospitalSyncDataDTO();
        params.setLogFlag(false);
        params.setCleanMqFlag(true);
        params.setRuleMqFlag(true);
        params.setType(SyncApiTypeEnum.SAPI_Diagnosis);
        hospitalDataSyncBizService.syncHospitalData(params);
    }


    /**
     *  医嘱数据同步 SAPI_DoctorAdvice
     * */
    @XxlJob("syncDoctorAdvice")
    public void syncDoctorAdvice() {
        log.debug(" +++++++++ Sync doctorAdvice : " + DateUtil.now());
        HospitalSyncDataDTO params = new HospitalSyncDataDTO();
        params.setLogFlag(false);
        params.setCleanMqFlag(true);
        params.setRuleMqFlag(true);
        params.setType(SyncApiTypeEnum.SAPI_DoctorAdvice);
        hospitalDataSyncBizService.syncHospitalData(params);
    }


    /**
     * 既往史同步
     * */
    @XxlJob("syncPastHis")
    public void syncPastHis() {
        log.debug(" +++++++++ Sync past his : " + DateUtil.now());
        HospitalSyncDataDTO params = new HospitalSyncDataDTO();
        params.setLogFlag(false);
        params.setCleanMqFlag(true);
        params.setRuleMqFlag(true);
        params.setType(SyncApiTypeEnum.SAPI_PastHis);
        hospitalDataSyncBizService.syncHospitalData(params);
    }

    /**
     * 首次病程同步
     * */
    @XxlJob("syncFirstCourseIllness")
    public void syncFirstCourseIllness() {
        log.debug(" +++++++++ Sync first course illness : " + DateUtil.now());
        HospitalSyncDataDTO params = new HospitalSyncDataDTO();
        params.setLogFlag(false);
        params.setCleanMqFlag(true);
        params.setRuleMqFlag(true);
        params.setType(SyncApiTypeEnum.SAPI_FirstCourseIllness);
        hospitalDataSyncBizService.syncHospitalData(params);
    }



    /**
     * 收费信息
     * */
    //@Scheduled(cron = "0 0/2 * * * ?")
    @XxlJob("patientFee")
    public void patientFee() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync PatientFee : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_PatientFee);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }

    @XxlJob("medicalRecord")
    public void medicalRecord() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync MedicalRecord : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(true);
            params.setType(SyncApiTypeEnum.SAPI_MedicalRecord);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }



    /**
     * 体检:体检患者
     * */
    @XxlJob("syncTiJianPatient")
    public void syncTiJianPatient() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync syncTiJianPatient[体检患者] : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(true);
            params.setRuleMqFlag(false);
            params.setType(SyncApiTypeEnum.SAPI_TJ_Patient);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 体检:体检登记
     * */
    @XxlJob("syncTiJianRegister")
    public void syncTiJianRegister() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync syncTiJianRegister[体检登记] : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(false);
            params.setRuleMqFlag(false);
            params.setType(SyncApiTypeEnum.SAPI_TJ_Register);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 体检:体检报告
     * */
    @XxlJob("syncTiJianReport")
    public void syncTiJianReport() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync syncTiJianReport[体检报告] : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(false);
            params.setRuleMqFlag(false);
            params.setType(SyncApiTypeEnum.SAPI_TJ_Report);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 体检:体检总报告
     * */
    @XxlJob("syncTiJianFinalReport")
    public void syncTiJianFinalReport() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync syncTiJianFinalReport[体检总报告] : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(false);
            params.setRuleMqFlag(false);
            params.setType(SyncApiTypeEnum.SAPI_TJ_FinalReport);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }

    /**
     * 体检:体检建议
     * */
    @XxlJob("syncTiJianSuggest")
    public void syncTiJianSuggest() {
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync syncTiJianSuggest[体检建议] : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(false);
            params.setRuleMqFlag(false);
            params.setType(SyncApiTypeEnum.SAPI_TJ_Suggest);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }


    /**
     * 活动: 检查报告单
     * */
    @XxlJob("syncActivityExamReport")
    public void syncActivityExamReport(){
        if (projectETLConfig.isSyncTask()) {
            log.debug(" +++++++++ Sync syncActivityExamReport[检查报告单] : " + DateUtil.now());
            HospitalSyncDataDTO params = new HospitalSyncDataDTO();
            params.setLogFlag(false);
            params.setCleanMqFlag(false);
            params.setRuleMqFlag(false);
            params.setType(SyncApiTypeEnum.SAPI_ActivityExamReport);
            hospitalDataSyncBizService.syncHospitalData(params);
        }
    }





}
