package com.xxh.cloud.sync.framework.modules.task;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.common.mq.bean.MQDeptBean;
import com.xxh.cloud.common.mq.bean.MQDoctorBean;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.etl.common.framework.modules.config.ProjectETLConfig;
import com.xxh.cloud.framework.activemq.constant.ActiveMQWayEnum;
import com.xxh.cloud.framework.activemq.utils.ActiveMQSendUtils;
import com.xxh.cloud.sync.framework.modules.config.bean.bos.HwdsReadConfigBean;
import com.xxh.cloud.sync.framework.modules.read.api.bean.doctor.SlifeHisDepartment;
import com.xxh.cloud.sync.framework.modules.read.api.bean.doctor.SlifeSicknessDepartment;
import com.xxh.cloud.sync.framework.modules.read.api.resp.BwwbSyncDataRespHBO;
import com.xxh.cloud.sync.framework.modules.read.api.service.BwwbDoctorDeptService;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Log4j2
@Component
public class SyncDoc2DeptTask {



    @Autowired
    private ProjectETLConfig projectETLConfig;



    @XxlJob("docterDeptSync")
    public void docterDeptSync(){
        HwdsReadConfigBean config=new HwdsReadConfigBean();
        config.setDomain(projectETLConfig.getSyncDomain());
        config.setAppid(projectETLConfig.getSyncAppId());
        config.setSecret(projectETLConfig.getSyncSecret());
        BwwbSyncDataRespHBO<SlifeSicknessDepartment> data= BwwbDoctorDeptService.getDoctor2Dept(config);
        if(CollUtil.isEmpty(data.getData())){
            return;
        }

        List<MQDeptBean> deptList=new ArrayList<>();
        List<MQDoctorBean> doctorList=new ArrayList<>();

        for(SlifeSicknessDepartment department :data.getData()){
            MQDeptBean dept=new MQDeptBean();
            dept.setId(department.getId());
            dept.setDeptName(department.getDeptName());
            dept.setWebDeptId(department.getWebDeptId());
            //dept.setStatus();
            deptList.add(dept);

            if(CollUtil.isNotEmpty(department.getDeptList())){
                for(SlifeHisDepartment childDept:department.getDeptList()){
                    MQDeptBean dept2=new MQDeptBean();
                    dept2.setId(childDept.getDeptCode());
                    dept2.setDeptName(childDept.getDeptName());
                    dept2.setDeptType(childDept.getDeptType());
                    dept2.setDeptTypeName(childDept.getDeptTypeName());
                    dept2.setPid(department.getId());
                    dept2.setStatus(childDept.getStatus());
                    if(CollUtil.isNotEmpty(childDept.getEmpList())){
                        childDept.getEmpList().forEach(item->{
                            MQDoctorBean doctorBean=new MQDoctorBean();
                            doctorBean.setEmpCode(item.getEmpCode());
                            doctorBean.setEmpName(item.getEmpName());
                            doctorBean.setOaEmpCode(item.getOaEmpCode());
                            doctorBean.setPhoneNumber(item.getPhoneNumber());
                            doctorBean.setDeptId(childDept.getDeptCode());
                            doctorBean.setTopDeptId(department.getId());
                            doctorList.add(doctorBean);
                        });
                    }else {
                        //dept2.setStatus(0);
                    }
                    deptList.add(dept2);
                }
            }
        }


        if(CollUtil.isNotEmpty(deptList)){
            log.info("+++++ 发送部门");
            ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName.Que_Dept, JSON.toJSONString(deptList));
        }

        if(CollUtil.isNotEmpty(doctorList)){
            log.info("+++++ 发送医生");
            ActiveMQSendUtils.send(ActiveMQWayEnum.MQ_SW_Queue, HMQQueueName.Que_Doctor, JSON.toJSONString(doctorList));
        }
    }

}
