package com.xxh.cloud.etlDataSync.sys.manager.controller;

import cn.hutool.core.util.EnumUtil;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.XXEnumUtil;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.sync.framework.constant.SyncApiTypeEnum;
import com.xxh.cloud.sync.framework.modules.biz.bean.dto.HospitalHisDataStartDTO;
import com.xxh.cloud.sync.framework.modules.biz.bean.dto.HospitalHisDataStopDTO;
import com.xxh.cloud.sync.framework.modules.biz.bean.dto.HospitalSyncDataDTO;
import com.xxh.cloud.sync.framework.modules.biz.service.HospitalDataSyncBizService;
import com.xxh.cloud.sync.framework.modules.biz.service.HospitalDataSyncThreadBizService;
import com.xxh.cloud.sync.framework.modules.biz.service.OdsExamineApplyCacheService;
import com.xxh.cloud.sync.framework.modules.task.SyncDoc2DeptTask;
import com.xxh.cloud.sync.framework.modules.task.SyncWebEmployeeTask;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("/sys/sync/data/run")
public class SyncDataRunManagerController {

    @Autowired
    private HospitalDataSyncBizService hospitalDataSyncBizService;

    @Autowired
    private OdsExamineApplyCacheService odsExamineApplyCacheService;

    @Autowired
    private SyncDoc2DeptTask syncDoc2DeptTask;
    @Autowired
    private SyncWebEmployeeTask syncWebEmployeeTask;


    @Autowired
    private HospitalDataSyncThreadBizService hospitalDataSyncThreadBizService;



    //@SaCheckPermission("sys:sync:data:run:start")
    @PostMapping("/start")
    @ApiOperation(value = "执行同步", response = String.class)
    public Result<String> start(@RequestBody HospitalHisDataStartDTO params) {
        ValidatorUtils.validateEntity(params);
        SyncApiTypeEnum keyEnum= XXEnumUtil.getEnum(SyncApiTypeEnum.class,params.getKey());
        if(keyEnum==null){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        String data=hospitalDataSyncThreadBizService.start(keyEnum);
        return new Result<String>().ok(data);
    }

    //@SaCheckPermission("sys:sync:data:run:stop")
    @PostMapping("/stop")
    @ApiOperation(value = "关闭同步", response = String.class)
    public Result<String> stop(@RequestBody HospitalHisDataStopDTO params) {
        ValidatorUtils.validateEntity(params);
        SyncApiTypeEnum keyEnum= XXEnumUtil.getEnum(SyncApiTypeEnum.class,params.getKey());
        if(keyEnum==null){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        String data=hospitalDataSyncThreadBizService.stop(keyEnum);
        return new Result<String>().ok(data);
    }

    //@SaCheckPermission("sys:sync:data:run:info")
    @PostMapping("/info")
    @ApiOperation(value = "查看同步状态", response = String.class)
    public Result<String> info(@RequestBody HospitalHisDataStopDTO params) {
        ValidatorUtils.validateEntity(params);
        SyncApiTypeEnum keyEnum= XXEnumUtil.getEnum(SyncApiTypeEnum.class,params.getKey());
        if(keyEnum==null){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR);
        }
        String data= hospitalDataSyncThreadBizService.info(keyEnum);
        return new Result<String>().ok(data);
    }




    @GetMapping("syncWebDept")
    public Result<String> syncWebDept() {
        syncWebEmployeeTask.webDeptEmployee();
        return new Result<String>().ok("syncDept");
    }

    @GetMapping("syncDept")
    public Result<String> syncDept() {
        syncDoc2DeptTask.docterDeptSync();
        return new Result<String>().ok("syncDept");
    }

    @GetMapping("/patient")
    public Result<String> patient() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_PatientInfo);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_PatientInfo");
    }



    @GetMapping("/diagnosis")
    public Result<String> diagnosis() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_Diagnosis);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_Diagnosis");
    }


    //就诊记录同步
    @GetMapping("/visit")
    public Result<String> visit() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_VisitRecord);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_VisitRecord");
    }

    //挂号记录同步
    @GetMapping("/register")
    public Result<String> register() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_PatientRegister);

        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_VisitRecord");
    }


    //预约住院信息记录
    @GetMapping("/appointIn")
    public Result<String> appointIn() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_InAppointment);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_InAppointment");
    }




    //住院记录同步
    @GetMapping("/in")
    public Result<String> in() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_InRecord);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_InRecord");
    }

    @GetMapping("/inChange")
    public Result<String> inChange() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_InChange);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_InChange");
    }

    //出院小结记录
    @GetMapping("/out")
    public Result<String> out() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_OutSummary);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_OutSummary");
    }


    //检查申请
    @GetMapping("/examApply")
    public Result<String> examApply() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_ExamApply);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_ExamApply");
    }


    //检查预约
    @GetMapping("/examAppointment")
    public Result<String> examAppointment() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_ExamAppointment);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_ExamAppointment");
    }


    //检查报告单数据
    @GetMapping("/examReport")
    public Result<String> examReport() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_ExamReport);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_ExamReport");
    }


    //检验申请
    @GetMapping("/labApply")
    public Result<String> labApply() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_LaboratoryApply);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_LaboratoryApply");
    }

    //检验报告单
    @GetMapping("/labReport")
    public Result<String> labReport() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_LaboratoryReport);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_LaboratoryReport");
    }

    //医嘱用药数据
    @GetMapping("/medication")
    public Result<String> medication() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_AdviceMedication);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_AdviceMedication");
    }


    //医嘱治疗数据
    @GetMapping("/therapy")
    public Result<String> therapy() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_AdviceTherapy);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_AdviceTherapy");
    }

    //医嘱手术申请数据
    @GetMapping("/opApply")
    public Result<String> opApply() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_OperationApply);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_OperationApply");
    }


    //手术记录数据
    @GetMapping("/opRecord")
    public Result<String> opRecord() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_OperationRecord);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_OperationRecord");
    }

    //收费信息
    @GetMapping("/fee")
    public Result<String> fee() {
        HospitalSyncDataDTO params=new HospitalSyncDataDTO();
        params.setLogFlag(true);
        params.setCleanMqFlag(false);
        params.setRuleMqFlag(false);
        params.setType(SyncApiTypeEnum.SAPI_PatientFee);
        hospitalDataSyncBizService.syncHospitalData(params);
        return new Result<String>().ok("SAPI_PatientFee");
    }



}
